/***
 * Defines the format of messages to handle communications between C and Granules
 * Java programs.  This has been modified from the c++ version, so has some extraneous measures.
 * Contains several more attributes than the Java equivalent, to help
 * with arrays.
 *
 * @author K Ericson
 */
#include <stdio.h>
#include <memory.h>
#include <stdlib.h>
#include "helpers.h"

#ifndef _JAVAMESSAGES_C_
#define _JAVAMESSAGES_C_

struct javaMessage {
		int mType;
		char* input;
		char* output;
		char* stream;
		char* description;
		char fromJava;
		//helpers
		int inputSize;
		int outputSize;
		int streamSize;
		int descSize;
};

/***
 * As with the Java version, a <code>javaMessage</code> has the type
 * set at creation, and it cannot be changed later.
 * @param type
 * 		Message type as declared in Constants.h
 */
struct javaMessage javaMessage(int type) {
	struct javaMessage jm;
	jm.mType = type;
	jm.fromJava = (1==0);
	return jm;
}

/***
 * Generates a <code>javaMessage</code> from a <code>char*</code>
 * @param blob
 * 		Message in char* format
 */
struct javaMessage javaMessageFromBlob(char* blob) {
	struct javaMessage jm;
	char bob[4];
	char* temp = bob;//temp holder for int bytes
	int start = 0; //index to keep track of where we are in the char*

	memcpy(temp,blob+start,4);
	jm.mType = readInt(temp);
	fprintf(stderr,"type: %d\n",jm.mType);
	start +=4;

	int isize, osize;//input and output size respectively
	isize = -1;
	osize = -1;
	memcpy(temp,blob+start,4);
	isize = readInt(temp);
	jm.inputSize = isize;
	start +=4;
	if(isize > 0){
		jm.input = ( char*)malloc(sizeof(char)*isize);
		memcpy(jm.input, blob+start,isize);
	}
	else
		jm.input = NULL;
	start+=isize;

	memcpy(temp,blob+start,4);
	osize = readInt(temp);
	jm.outputSize = osize;
	start+=4;
	if(osize > 0)
	{
		jm.output = (char *)malloc(sizeof(char) * osize);
		memcpy(jm.output,blob+start,osize);
	}
	else
		jm.output = NULL;
	start+=osize;

	int ssize, dsize; //keeps track of length of stream and description strings
	memcpy(temp,blob+start,4);
	ssize = readInt(temp);
	start+=4;
	jm.streamSize = ssize;
	jm.stream = (char *)malloc(sizeof(char) * ssize);
	memcpy(jm.stream,blob+start,ssize);
	start+=ssize;

	memcpy(temp,blob+start,4);
	start+=4;
	dsize = readInt(temp);
	jm.descSize = dsize;
	jm.description = (char*)malloc(sizeof(char) * dsize);
	memcpy(jm.description,blob+start,dsize);
	start+=dsize;

	memcpy(&jm.fromJava,blob+start,sizeof(char));
	return jm;
}


//accessor functions - not needed with c structs
/*int getType(struct javaMessage *j){
	return j->mType;
}

void setInputSize(int i, struct javaMessage *j){
	j->inputSize = i;
}

int getInputSize(struct javaMessage *j){
	return j->inputSize;
}

void javaMessage::setInput(char* i, struct javaMessage *j){
	j->input = i;
}

char* javaMessage::getInput(struct javaMessage *j){
	return j->input;
}

void javaMessage::setOutputSize(int i,struct javaMessage *j) {
	j->outputSize = i;
}

int javaMessage::getOutputSize(struct javaMessage *j){
	return j->outputSize;
}

void javaMessage::setOutput( char* o,  struct javaMessage *j) {
	j->output = o;
}

 char* javaMessage::getOutput(struct javaMessage *j) {
	return j->output;
}

void javaMessage::setStream(char* s,struct javaMessage *j) {
	j->stream = s;
}

char* javaMessage::getStream(struct javaMessage *j) {
	return j->stream;
}

void javaMessage::setDescription(char* s,struct javaMessage *j) {
	j->description = s;
}

char* javaMessage::getDescription(struct javaMessage *j) {
	return j->description;
}

void javaMessage::setFromJava(bool b) {
	fromJava = b;
}

char javaMessage::isFromJava() {
	return fromJava;
}

//end accessor functions

/***
 * Helper function to convert the message into a char* - calculates the size
 * that the byte array will be.
 * @returns int
 * 		returns the size that the char* version of this javaMessage is
 */
int byteSize(struct javaMessage *j)
{
	int size;
	size = 5 * 4;	 //basic size - includes int types, as well as needed ints for sizes of other fields - too hardcoded?
	size += j->streamSize;
	size += j->descSize;
	size += j->inputSize;
	size += j->outputSize;
	size += sizeof(char);
	return size;
}


/***
 * generates the bytes to return the message
 * assumes the char* blob has already been initialized to the correct size
 * correct size is found with <code>byteSize()</code>
 */
void getBytes( char* blob, struct javaMessage *jm){
	int size = byteSize(jm);
	int start = 0;
	int s;
	
	writeInt(blob + start,jm->mType,&start);

	if(jm->inputSize >0)
	{
		s = jm->inputSize;
		writeInt(blob + start,s,&start);
		memcpy(blob + start,jm->input, s);
		start+=s;
	}
	else
	{
		s = 0;
		writeInt(blob + start,s,&start);
	}
	

	if (jm->outputSize > 0)
	{
		s = jm->outputSize;
		writeInt(blob + start,s,&start);
		memcpy(blob+start,jm->output, s);
		start+=s;
	}
	else
	{
		s = 0;
		writeInt(blob + start,s,&start);
	}
	
	s = jm->streamSize;
	writeInt(blob + start,s,&start);
	memcpy(blob+start,jm->stream, s);
	start+=s;
	

	s = jm->descSize;
	writeInt(blob + start,s,&start);
	memcpy(blob+start,jm->description, s);
	start+=s;

	
	memcpy(blob+start,&jm->fromJava, sizeof(char));
}

#endif //_JAVAMESSAGES_C_
