﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;

namespace csBridge
{
    /*
     * Defines the communications protocol between Java
     * and C# - should be used for all Granules Bridge communications
     */
    class JavaMessage
    {
        //basic types needed for a message
        private int messageType;
        private byte[] input;
        private byte[] output;
        private string streamID;
        private string description;
        private bool fromJava;

        public JavaMessage(int control) //force the message type to be set at creation - cannot be changed
        {
            messageType = control;
            fromJava = false;
            input = new byte[0];
            output = new byte[0];
            streamID = "";
            description = "";
        }

        /***
         * build a javaMessage from an input byte[]
         */
        public JavaMessage(byte[] incoming)
        {
            //should I redo with a memory stream
            //and a binary reader?
            int start = 0;
            messageType = BitConverter.ToInt32(incoming, start);
            messageType = IPAddress.NetworkToHostOrder(messageType);
            start += 4; //do I need to keep this up to date myself?
            int inputsize = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(incoming, start));
            start += 4;
            input = new byte[inputsize];
            Array.Copy(incoming, start, input, 0, inputsize);
            start += inputsize;
            int outputsize = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(incoming, start));
            start += 4;
            output = new byte[outputsize];
            Array.Copy(incoming, start, output, 0, outputsize);
            start += outputsize;
            int streamsize = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(incoming, start));
            start += 4;
            byte[] streamb = new byte[streamsize];
            Array.Copy(incoming, start, streamb, 0, streamsize);
            Array.Reverse(streamb);
            streamID = System.Text.Encoding.Default.GetString(streamb);
            start += streamsize;
            int descsize = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(incoming, start));
            start += 4;
            byte[] descb = new byte[descsize];
            Array.Copy(incoming, start, descb, 0, descsize);
            Array.Reverse(descb);
            description = System.Text.Encoding.Default.GetString(descb);
            start += descsize;
            fromJava = BitConverter.ToBoolean(incoming, start);
        }

        //Accessor functions
        public int getMessageType()
        {
            return messageType;
        }

        public void setInput(byte[] input)
        {
            this.input = input;
        }

        public byte[] getInput()
        {
            return input;
        }

        public void setOutput(byte[] output)
        {
            this.output = output;
        }

        public byte[] getOutput()
        {
            return output;
        }

        public void setStreamID(string sid)
        {
            streamID = sid;
        }

        public string getStreamID()
        {
            return streamID;
        }

        public void setDescription(string desc)
        {
            description = desc;
        }

        public string getDescription()
        {
            return description;
        }

        public bool isFromJava()
        {
            return fromJava;
        }
        //done with accessors

        /**
         * Converts message to a byte[] to send.
         * 
         * @return byte[]
         *              byte[] representation of the JavaMessage
         */
        public byte[] toByteArray()
        {
            MemoryStream ms = new MemoryStream();
            BinaryWriter br = new BinaryWriter(ms);

            br.Write(IPAddress.HostToNetworkOrder(messageType));
            br.Write(IPAddress.HostToNetworkOrder(input.Length));
            br.Write(input);
            br.Write(IPAddress.HostToNetworkOrder(output.Length));
            br.Write(output);
            byte[] streamchar = System.Text.UTF8Encoding.UTF8.GetBytes(streamID);
            br.Write(IPAddress.HostToNetworkOrder(streamchar.Length));
            br.Write(streamchar);
            byte[] descchar = System.Text.UTF8Encoding.UTF8.GetBytes(description);
            br.Write(IPAddress.HostToNetworkOrder(descchar.Length));
            br.Write(descchar);
            br.Write(fromJava);

            br.Flush();
            return ms.ToArray();
        }

        public string toString()
        {
            string val = "";

            val += "Message Type: " + messageType + "\n";
            val += "Input Length: " + input.Length + "\n";
            val += "Output Length: " + output.Length + "\n";
            val += "Stream ID: " + streamID + "\n";
            val += "Description: " + description + "\n";
            val += "From Java? " + fromJava;

            return val;
        }
    }
}
