/**
 * Software License, Version 1.0 Copyright 2010 Colorado State University. All
 * rights reserved. Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following conditions
 * are met: 1) All redistributions of source code must retain the above
 * copyright notice, the list of authors in the original source code, this list
 * of conditions and the disclaimer listed in this license; 2) All
 * redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the disclaimer listed in this license in the
 * documentation and/or other materials provided with the distribution; 3) Any
 * documentation included with all redistributions must include the following
 * acknowledgment: "This product includes software developed by the Department
 * of Computer Science at Colorado State University. For further information
 * contact the department at http://www.cs.colostate.edu/." Alternatively, this
 * acknowledgment may appear in the software itself, and wherever such
 * third-party acknowledgments normally appear. 4) The name Colorado State
 * University or Granules shall not be used to endorse or promote products
 * derived from this software without prior written permission from Colorado
 * State University. For written permission, please contact the Computer Science
 * Department at 1100 Center Avenue, Fort Collins, CO 80523-1873. 5) Products
 * derived from this software may not be called Granules, nor may Colorado State
 * University or Granules appear in their name, without prior written permission
 * of ARTI. Colorado State University provides no reassurances that the source
 * code provided does not infringe the patent or any other intellectual property
 * rights of any other entity. Colorado State University disclaims any liability
 * to any recipient for claims brought by any other entity based on infringement
 * of intellectual property rights or otherwise. LICENSEE UNDERSTANDS THAT
 * SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES AS TO CAPABILITIES OR
 * ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES NO WARRANTIES AND MAKES NO
 * REPRESENTATION THAT SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT,
 * COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. COLORADO STATE UNIVERSITY MAKES NO
 * WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES",
 * "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO
 * THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
 */
package ds.granules.learning.classification.common;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;
import java.util.regex.Pattern;


import org.apache.commons.lang3.mutable.MutableDouble;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.KeyValueTextInputFormat;
import org.apache.lucene.analysis.shingle.ShingleFilter;
import org.apache.lucene.analysis.tokenattributes.TermAttribute;
import org.apache.mahout.common.lucene.IteratorTokenStream;
import org.apache.mahout.math.function.ObjectIntProcedure;
import org.apache.mahout.math.map.OpenObjectIntHashMap;

import com.google.common.collect.Iterators;

import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.FileAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;

/**
 * First step to Mahout's Naive Bayes classification (both complimentary and not).
 * 
 * @author Kathleen Ericson
 *
 */
public class BayesPrepStage1 extends MapReduceBase{

  private static final Pattern SPACE_PATTERN = Pattern.compile("[ ]+");

  private int gramSize = 1;
  boolean isInitialized = false;
  private String datasetIdentifier = "FileDataset-ID-BayesPrep";
  private int WorkerNumber;
  private long runtime;

  private HashMap<String, Integer> labelMap = new HashMap<String, Integer>();
  private HashMap<String, Integer> featureMap = new HashMap<String, Integer>();
  private HashMap<String, Integer> featureFreqMap = new HashMap<String, Integer>();
  private HashMap<GranTuple, Integer> docFreqMap = new HashMap<GranTuple, Integer>();
  private HashMap<GranTuple, Double> weightMap = new HashMap<GranTuple, Double>();

  public BayesPrepStage1() {
    setDomain("Classification");
    setVersionInformation(075);
    setOperationIdentifier("BayesStageOne");
  }

  private void Initialize() {
    //grab data from props - make sure we can find the input file...
    // set WorkerNumber;
    Properties p = this.getProcessingDirectives();
    String num = p.getProperty("Worker Number");
    if(num != null)
      WorkerNumber = Integer.parseInt(num);
    else
      WorkerNumber = 9999;
    isInitialized = true;
  }


  @Override
  public void execute() throws ProcessingException {
    // TODO Auto-generated method stub
    //I'm sure I need more to lead into this...
    System.out.println("Starting with Bayes Prep Stage 1 work...");
    runtime = System.nanoTime();
    try {
      DatasetCollection dc = getDatasetCollection();
      if(!isInitialized) {
        Initialize();
      }
      FileAccess k = null;
      Dataset d = dc.getDataset(datasetIdentifier);
      System.out.println("Dataset d: " + d);
      if (d.getDatasetType() == Dataset.FILES) {
        k = (FileAccess)d;
      }
      else {
        System.out.println(WorkerNumber + ": Dataset issue: " + d.getDatasetType() + "initialized.  Returing...");
        return;
      }

      Enumeration fnames = k.getFilenames();
      System.out.println(fnames.toString());

      while (fnames.hasMoreElements()) {
        String f = (String) fnames.nextElement();
        loadTrainingset(f);
      }

      System.out.println("Finished, sending stuff on...");
      
      writeResults(labelMap, "LabelMap");
      System.out.println("sent label map");
      writeResults(featureMap, "FeatureMap");
      System.out.println("sent feature map");
      writeResults(featureFreqMap, "FeatureFreqMap");
      System.out.println("sent feature freq map");
      //System.out.println("Doc FreqMap: " + docFreqMap.toString());
      writeResults(docFreqMap, "DocFreqMap");
      System.out.println("sent doc freq map");
      //System.out.println("weightMap: " + weightMap.toString());
      writeResults(weightMap, "WeightMap");
      System.out.println("sent weight map");
      if(WorkerNumber == 0) {
        // only send from one of them
        writeResults(runtime, "StartTime");
      }

      long time = System.nanoTime() - runtime;
      FileWriter fw = new FileWriter(new File("BayesPrep1-worker" + WorkerNumber + ".timing"), true);
      fw.write(time +"\n");
      fw.flush();
      fw.close();

    } catch (MarshallingException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (MapReduceException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (DatasetException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }



  // read the initial input file line-by-line (stuff from BayesFeatureMapper.java)
  private void processLine(String keyin, String val) throws IOException {
    final String key = keyin;
    String value = val;
    
    //System.out.println("Processing:\nKey: " + key + "\tValue: " + val);

    // how many samples are we seeing from each label? (LABEL_COUNT)
    if(labelMap.containsKey(key)) {
      labelMap.put(key, labelMap.get(key) + 1);
    }
    else {
      //haven't seen this label yet
      labelMap.put(key, 1);
    }
    String[] tokens = SPACE_PATTERN.split(value.toString());
    OpenObjectIntHashMap<String> wordList = new OpenObjectIntHashMap<String>(tokens.length * gramSize);

    if (gramSize > 1) {
      ShingleFilter sf = new ShingleFilter(new IteratorTokenStream(Iterators.forArray(tokens)), gramSize);
      do {
        @SuppressWarnings("deprecation")
        String term = sf.getAttribute(TermAttribute.class).term();
        if (term.length() > 0) {
          if (wordList.containsKey(term)) {
            wordList.put(term, 1 + wordList.get(term));
          } else {
            wordList.put(term, 1);
          }
          addTuples(key, term);
        }
      } while (sf.incrementToken());
    } else {
      for (String term : tokens) {
        if (wordList.containsKey(term)) {
          wordList.put(term, 1 + wordList.get(term));
        } else {
          wordList.put(term, 1);
        }
        addTuples(key, term);
      }
    }
    //System.out.println("Finished first pass");

    // will this work?
    // can't see a way around this loop... ask Nate?
    final MutableDouble lengthNormalisationMut = new MutableDouble(0.0);
    wordList.forEachPair(new ObjectIntProcedure<String>() {
      @Override
      public boolean apply(String word, int dKJ) {
        lengthNormalisationMut.add(dKJ * dKJ);
        return true;
      }
    });
    
    //System.out.println("Done with first loop");

    final double lengthNormalisation = Math.sqrt(lengthNormalisationMut.doubleValue());

    // or this one... :-/
    wordList.forEachPair(new ObjectIntProcedure<String>() {
      @Override
      public boolean apply(String token, int dKJ) {
        // needs to compute lengthNormalisation first...
        GranTuple tuple = new GranTuple();
        tuple.add(key);
        tuple.add(token);
        Double f = new Double(Math.log(1.0 + dKJ) / lengthNormalisation);

        addToWeightMap(tuple, f);
        return true;
      }


    });
    
    //System.out.println("Done with second loop");

  }

  //this code performs BayesFeatureReducer.java gathering
  private void addToFeatureFreqMap(String token, int dKJ) {
    if (featureFreqMap.containsKey(token)) {
      featureFreqMap.put(token, featureFreqMap.get(token) + dKJ);
    }
    else {
      featureFreqMap.put(token, dKJ);
    }
  }

  private void addToFeatureMap(String token) {
    if(featureMap.containsKey(token)) {
      featureMap.put(token, featureMap.get(token) + 1);
    }
    else {
      featureMap.put(token, 1);
    }
  }

  private void addToDFMap(GranTuple dfTuple) {
    if(docFreqMap.containsKey(dfTuple)) {
      docFreqMap.put(dfTuple, docFreqMap.get(dfTuple) + 1); 
    }
    else {
      docFreqMap.put(dfTuple, 1);
    }
  }

  private void addToWeightMap(GranTuple tuple, Double f) {
    if(weightMap.containsKey(tuple)) {
      weightMap.put(tuple, weightMap.get(tuple) + f);
    }
    else {
      weightMap.put(tuple, f);
    }
  }

  private void addTuples(String key, String token) {

    // this can be done in the initial loop
    GranTuple dfTuple = new GranTuple();
    dfTuple.add(key);
    dfTuple.add(token);

    addToDFMap(dfTuple);

    //this can be done in the initial loop
    addToFeatureMap(token);

    // this can be done in the initial loop
    addToFeatureFreqMap(token, 1);

  }


  //load the files this mapper is responsible for
  private void loadTrainingset(String trainSet) {
    System.out.println("Loading clusters in mapper from " + trainSet);
    Configuration conf = new Configuration();
    Path path = new Path(trainSet);


    try {
      String line = "";
      DataInputStream d = new DataInputStream(path.getFileSystem(conf).open(path));
      BufferedReader reader = new BufferedReader(new InputStreamReader(d));
      while ((line = reader.readLine()) != null){
        int space = line.indexOf(' ');
        String key = line.substring(0, space);
        String value = line.substring(space + 1);
        processLine(key, value);
      }
      reader.close();


    } catch (Exception e) {
      System.out.println("Error: " + e.getMessage());
      e.printStackTrace();
    } 
  }    


  private void writeResults(HashMap map, String resultType)
  throws MarshallingException, MapReduceException {
    boolean completeResult = false;
    ResultPayload resultPayload = ResultPayload.getInstance();

    byte[] resultPayloadBytes = resultPayload.getBytes(map);

    Results results = createResults(completeResult, false);
    results.setResultPayload(resultPayloadBytes);
    results.setResultType(ResultPayload.HASHMAP);
    results.setRespondTo(Integer.toString(resultPayloadBytes.length));
    this.writeResults(resultType, results);
  }

  private void writeResults(long runtime2, String resultType) throws MarshallingException, MapReduceException {
    boolean completeResult = true;
    ResultPayload resultPayload = ResultPayload.getInstance();
    byte[] resultPayloadBytes = resultPayload.getBytes(runtime2);

    Results results = createResults(completeResult, false);
    results.setResultPayload(resultPayloadBytes);
    results.setResultType(ResultPayload.LONG);
    this.writeResults(resultType, results);

  }



}
