/**
 * Software License, Version 1.0 Copyright 2010 Colorado State University. All
 * rights reserved. Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following conditions
 * are met: 1) All redistributions of source code must retain the above
 * copyright notice, the list of authors in the original source code, this list
 * of conditions and the disclaimer listed in this license; 2) All
 * redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the disclaimer listed in this license in the
 * documentation and/or other materials provided with the distribution; 3) Any
 * documentation included with all redistributions must include the following
 * acknowledgment: "This product includes software developed by the Department
 * of Computer Science at Colorado State University. For further information
 * contact the department at http://www.cs.colostate.edu/." Alternatively, this
 * acknowledgment may appear in the software itself, and wherever such
 * third-party acknowledgments normally appear. 4) The name Colorado State
 * University or Granules shall not be used to endorse or promote products
 * derived from this software without prior written permission from Colorado
 * State University. For written permission, please contact the Computer Science
 * Department at 1100 Center Avenue, Fort Collins, CO 80523-1873. 5) Products
 * derived from this software may not be called Granules, nor may Colorado State
 * University or Granules appear in their name, without prior written permission
 * of ARTI. Colorado State University provides no reassurances that the source
 * code provided does not infringe the patent or any other intellectual property
 * rights of any other entity. Colorado State University disclaims any liability
 * to any recipient for claims brought by any other entity based on infringement
 * of intellectual property rights or otherwise. LICENSEE UNDERSTANDS THAT
 * SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES AS TO CAPABILITIES OR
 * ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES NO WARRANTIES AND MAKES NO
 * REPRESENTATION THAT SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT,
 * COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. COLORADO STATE UNIVERSITY MAKES NO
 * WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES",
 * "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO
 * THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
 */
package ds.granules.learning.classification.common;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.Set;
import java.util.zip.Deflater;


import cgl.narada.event.NBEvent;

import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.learning.clustering.kmeans.GranulesObservations;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

/**
 * Second stage of Bayes classification.  Reduces from prep stage 1.
 * 
 * @author Kathleen Ericson
 *
 */
public class BayesPrepStage2 extends MapReduceBase  {

  private String datasetIdentifier = "StreamingDataset-ID-Bayes";
  private boolean initialized = false;
  private int inputCount = 0;
  private int inputLabelsSeen = 0;
  private int inputFeaturesSeen = 0;
  private int inputFeatureFreqSeen = 0;
  private int docFreqSeen = 0;
  private int weightSeen = 0;

  //passed in
  private HashMap<String, Integer> labelMap = new HashMap<String, Integer>();
  private HashMap<String, Integer> featureMap = new HashMap<String, Integer>();
  private HashMap<String, Integer> featureFreqMap = new HashMap<String, Integer>();
  private HashMap<GranTuple, Integer> docFreqMap = new HashMap<GranTuple, Integer>();
  private HashMap<GranTuple, Double> weightMap = new HashMap<GranTuple, Double>();


  //created
  private HashMap<GranTuple, Double> docFreqWeightMap = new HashMap<GranTuple, Double>();
  private HashMap<GranTuple, Double> combinedWeightMap = new HashMap<GranTuple, Double>();

  private Double totalWeight = 0.0;

  public BayesPrepStage2() {
    setDomain("Classification");
    setVersionInformation(075);
    setOperationIdentifier("BayesStageTwo");
  }

  @Override
  public void execute() throws ProcessingException {
    // TODO Auto-generated method stub
    // first collect everything from all mappers
    // then culls terms which are below given threshold
    // ** Remember to remove from all maps **

    //after getting everything from all the mappers, can do tfidf stuff

    System.out.println("Executing Bayes Prep Stage 2");
    if (!initialized) {
      initialize();
    }

    StreamingAccess mapReduceDataset = null;
    DatasetCollection dc = getDatasetCollection();
    Dataset d;
    try {
      d = dc.getDataset(datasetIdentifier);
      if(d.getDatasetType() == Dataset.STREAMS) {
        mapReduceDataset = (StreamingAccess)d;
      }
    } catch (DatasetException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }



    while (mapReduceDataset.isDataAvailable()) {
      NBEvent nbEvent;
      try {
        nbEvent = mapReduceDataset.getStreamingData();
        processResultsFromMap(nbEvent);
      } catch (DatasetException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (MarshallingException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (MapReduceException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

    }
  }

  private void initialize() {
    // TODO Auto-generated method stub
    Properties p = this.getProcessingDirectives();
    String num = p.getProperty("Number Inputs");
    inputCount = Integer.parseInt(num);
    initialized = true;
  }

  private void processResultsFromMap(NBEvent nbEvent) throws MarshallingException, IOException, MapReduceException {
    String s = (String) nbEvent.getContentSynopsis();
    byte[] resultBytes = nbEvent.getContentPayload();
    Results results = ResultsFactory.getInstance().getResults(resultBytes);
    byte[] resultPayloadBytes = results.getResultPayload();
    // may need this later...
    //byte[] decompressedBytes = decompressPayload(resultPayloadBytes, Integer
    //    .parseInt(results.getRespondTo()));
    ResultPayload resultPayload = ResultPayload.getInstance();

    if (results.getResultType() != ResultPayload.HASHMAP) {
      System.out
      .println("BayesPrepStage2: Incorrect ResultType received on Synopsis: " + s);
      return;
    }
    //System.out.println("Synopsis: " + s);

    if(s.equals("LabelMap")) {
      handleLabelMap(resultPayload, resultPayloadBytes);
      inputLabelsSeen++;
    }
    if(s.equals("FeatureMap")) {
      handleFeatureMap(resultPayload, resultPayloadBytes);
      inputFeaturesSeen++;
    }
    if(s.equals("FeatureFreqMap")) {
      handleFeatureFreqMap(resultPayload, resultPayloadBytes);  
      inputFeatureFreqSeen++;
    }
    if(s.equals("DocFreqMap")) {
      handleDocFreqMap(resultPayload, resultPayloadBytes);
      docFreqSeen++;
    }
    if(s.equals("WeightMap")) {
      // do stuff
      handleWeightMap(resultPayload, resultPayloadBytes);
      weightSeen++;
    }

//    System.out.println("seen counters: " + inputLabelsSeen + "/" + inputCount);
//    System.out.println("\t" + inputFeaturesSeen + "/" + inputCount);
//    System.out.println("\t" + inputFeatureFreqSeen + "/" + inputCount);
//    System.out.println("\t" + docFreqSeen + "/" + inputCount);
//    System.out.println("\t" + weightSeen + "/" + inputCount);
    if(!(inputFeaturesSeen < inputCount || inputFeatureFreqSeen < inputCount
        || inputLabelsSeen < inputCount || docFreqSeen < inputCount
        || weightSeen < inputCount)) {

      // If I do cull out things below a threshold, this is the place to do it...
      tfIdfMapper();
      // Quick check of what i have in weightMap and docFreqWeightMap
  //    Object[] weightKeys = weightMap.keySet().toArray();
  //    Object[] docFreqWeightKeys = docFreqWeightMap.keySet().toArray();
  //    System.out.println("weightKeys[0] - " + weightKeys[0].toString());
  //    System.out.println("docFreqWeightKeys[0] - " + docFreqWeightKeys[0].toString());
      tfIdfReducer();
      //send out new weights, not sure what to do with sums yet...

      writeResults(docFreqWeightMap, "DocFreqWeightMap");
      writeResults(combinedWeightMap, "CombinedWeightMap");
      System.out.println("Sent combinedWeightMap\nsize: " + combinedWeightMap.size());
      int vocabCount = featureMap.size();
      writeResults(vocabCount, "VocabCount");
    }
  }

 

  private void handleWeightMap(ResultPayload resultPayload,
      byte[] resultPayloadBytes) throws MarshallingException {
    HashMap<GranTuple, Double> tmpMap = (HashMap<GranTuple, Double>)resultPayload.getHashMap(resultPayloadBytes);
    for(GranTuple k : tmpMap.keySet()) {
      if(weightMap.containsKey(k)) {
        weightMap.put(k, weightMap.get(k) + tmpMap.get(k));
      }
      else {
        weightMap.put(k, tmpMap.get(k));
      }
    }
  }

  private void handleDocFreqMap(ResultPayload resultPayload,
      byte[] resultPayloadBytes) throws MarshallingException {
    HashMap<GranTuple, Integer> tmpMap = (HashMap<GranTuple, Integer>)resultPayload.getHashMap(resultPayloadBytes);
    for(GranTuple k : tmpMap.keySet()) {
      if(docFreqMap.containsKey(k)) {
        docFreqMap.put(k, docFreqMap.get(k) + tmpMap.get(k));
      }
      else {
        docFreqMap.put(k, tmpMap.get(k));
      }
    }

  }

  private void handleFeatureFreqMap(ResultPayload resultPayload,
      byte[] resultPayloadBytes) throws MarshallingException {
    HashMap<String, Integer> tmpMap = (HashMap<String, Integer>)resultPayload.getHashMap(resultPayloadBytes);
    for(String k : tmpMap.keySet()) {
      if(featureFreqMap.containsKey(k)) {
        featureFreqMap.put(k, featureFreqMap.get(k) + tmpMap.get(k));
      }
      else {
        featureFreqMap.put(k, tmpMap.get(k));
      }
    }
  }

  private void handleFeatureMap(ResultPayload resultPayload,
      byte[] resultPayloadBytes) throws MarshallingException {
    HashMap<String, Integer> tmpMap = (HashMap<String, Integer>)resultPayload.getHashMap(resultPayloadBytes);
    for(String k : tmpMap.keySet()) {
      if(featureMap.containsKey(k)) {
        featureMap.put(k, featureMap.get(k) + tmpMap.get(k));
      }
      else {
        featureMap.put(k, tmpMap.get(k));
      }
    }

  }

  private void handleLabelMap(ResultPayload resultPayload, byte[] hashbytes) throws MarshallingException {
    // TODO Auto-generated method stub
    HashMap<String, Integer> tmpMap = (HashMap<String, Integer>)resultPayload.getHashMap(hashbytes);
    for (String k : tmpMap.keySet()) {
      if(labelMap.containsKey(k)) {
        labelMap.put(k, labelMap.get(k) + tmpMap.get(k));
      }
      else {
        labelMap.put(k, tmpMap.get(k));
      }
    }
  }

  private void tfIdfMapper() {
    // loop through the docFreqMap
    for(GranTuple k : docFreqMap.keySet()) {
      String label = k.stringAt(0);
      int labelCount = labelMap.get(label);
      double logIdf = Math.log(labelCount / docFreqMap.get(k));
      docFreqWeightMap.put(k, logIdf);
    }
  }

  private void tfIdfReducer() {
    // look for unique label/word tuples in weightMap and docFreqWeightMap -> multiply them together
    ArrayList<GranTuple> combinedKeys = new ArrayList<GranTuple>(); 
    combinedKeys.addAll(docFreqWeightMap.keySet());
    Set<GranTuple> tmpSet = weightMap.keySet();
    combinedKeys.addAll(tmpSet);
    for (GranTuple key : combinedKeys) {
      // let it throw an error if there isn't a matching key in weightMap -> something went wrong if it's not
      if(weightMap.containsKey(key) && docFreqWeightMap.containsKey(key))
      {
        double tmpTfidf = docFreqWeightMap.get(key) * weightMap.get(key);
        combinedWeightMap.put(key, tmpTfidf);
      }
      else if (!weightMap.containsKey(key) && docFreqWeightMap.containsKey(key)) {
        //       System.out.println("WeightMap is missing key: " + key.toString());
        // don't update combinedWeightMap?
        combinedWeightMap.put(key, docFreqWeightMap.get(key));
      }
      else if(!docFreqWeightMap.containsKey(key) && weightMap.containsKey(key)) {
        combinedWeightMap.put(key, weightMap.get(key));
      }
      if(combinedWeightMap.get(key) == null) {
        System.out.println("in stage2, removed a key: " + key);
        combinedWeightMap.remove(key);
      }
    }
  }

  // I'll probably need something bigger for this, since only tfidf stuff goes on to 
  // stage 3, but the rest is needed later.
  private void writeResults(HashMap map, String label) throws MarshallingException, MapReduceException {
    boolean completeResult = false;
    ResultPayload resultPayload = ResultPayload.getInstance();

    byte[] resultPayloadBytes = resultPayload.getBytes(map);
    
    Results results = createResults(completeResult, false);
    if(label.equals("CombinedWeightMap")) {
      byte[] compressedBytes = compressPayload(resultPayloadBytes);

      results.setRespondTo(Integer.toString(resultPayloadBytes.length));
      results.setResultPayload(compressedBytes);
    }
    else {
      results.setResultPayload(resultPayloadBytes);
    }
    
    results.setResultType(ResultPayload.HASHMAP);
    this.writeResults(label, results);
  }
  
  private void writeResults(int vocabCount, String label) throws MarshallingException, MapReduceException {
    boolean completeResult = false;
    ResultPayload resultPayload = ResultPayload.getInstance();

    byte[] resultPayloadBytes = resultPayload.getBytes(vocabCount);
    
    Results results = createResults(completeResult, false);
    
    results.setResultPayload(resultPayloadBytes);
    
    results.setResultType(ResultPayload.INT);
    this.writeResults(label, results);
  }
  
  private byte[] compressPayload(byte[] payload) {
    Deflater compresser = new Deflater(Deflater.DEFAULT_COMPRESSION);
    compresser.setInput(payload);
    compresser.finish();
    int payloadLength = payload.length;

    byte[] output = new byte[payloadLength];
    int compressedDataLength = compresser.deflate(output);

    if (compressedDataLength == payloadLength) {
      System.out.println("not deflating");
      return payload;
    }

    byte[] compressedBytes = new byte[compressedDataLength];
    System.arraycopy(output, 0, compressedBytes, 0, compressedDataLength);
    return compressedBytes;
  }

}
