/**
 * Software License, Version 1.0 Copyright 2010 Colorado State University. All
 * rights reserved. Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following conditions
 * are met: 1) All redistributions of source code must retain the above
 * copyright notice, the list of authors in the original source code, this list
 * of conditions and the disclaimer listed in this license; 2) All
 * redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the disclaimer listed in this license in the
 * documentation and/or other materials provided with the distribution; 3) Any
 * documentation included with all redistributions must include the following
 * acknowledgment: "This product includes software developed by the Department
 * of Computer Science at Colorado State University. For further information
 * contact the department at http://www.cs.colostate.edu/." Alternatively, this
 * acknowledgment may appear in the software itself, and wherever such
 * third-party acknowledgments normally appear. 4) The name Colorado State
 * University or Granules shall not be used to endorse or promote products
 * derived from this software without prior written permission from Colorado
 * State University. For written permission, please contact the Computer Science
 * Department at 1100 Center Avenue, Fort Collins, CO 80523-1873. 5) Products
 * derived from this software may not be called Granules, nor may Colorado State
 * University or Granules appear in their name, without prior written permission
 * of ARTI. Colorado State University provides no reassurances that the source
 * code provided does not infringe the patent or any other intellectual property
 * rights of any other entity. Colorado State University disclaims any liability
 * to any recipient for claims brought by any other entity based on infringement
 * of intellectual property rights or otherwise. LICENSEE UNDERSTANDS THAT
 * SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES AS TO CAPABILITIES OR
 * ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES NO WARRANTIES AND MAKES NO
 * REPRESENTATION THAT SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT,
 * COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. COLORADO STATE UNIVERSITY MAKES NO
 * WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES",
 * "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO
 * THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
 */
package ds.granules.learning.classification.common;

import java.io.IOException;
import java.util.HashMap;
import java.util.zip.DataFormatException;
import java.util.zip.Inflater;


import cgl.narada.event.NBEvent;
import cgl.narada.service.ServiceException;

import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

/**
 * Final common stage of Bayes classification.
 * 
 * @author Kathleen Ericson
 *
 */
public class BayesPrepStage3 extends MapReduceBase{

  private String datasetIdentifier = "StreamingDataset-ID-Bayes";

  // Sigma_j
  private HashMap<String, Double> featureWeightMap = new HashMap<String, Double>();
  // Sigma_k
  private HashMap<String, Double> labelWeightMap = new HashMap<String, Double>();
  // Sigma_kSigma_j
  private double totalSum = 0.0;
  private boolean initialized = false;
  private int nullCount = 0;
  private boolean gotMap1 = false;
  private boolean gotMap2 = false;

  String outputDir; // place to save out the sigma files needed later?

  public BayesPrepStage3() {
    setDomain("Classification");
    setVersionInformation(075);
    setOperationIdentifier("BayesStage3");
  }

  @Override
  public void execute() throws ProcessingException {
    // TODO Auto-generated method stub
    //only needs tfidf outputs: docFreqWeightMap and combinedWeightMap
    System.out.println("Executing BayesPrepStage3");

    StreamingAccess mapReduceDataset = null;
    DatasetCollection dc = getDatasetCollection();
    Dataset d;
    try {
      d = dc.getDataset(datasetIdentifier);
      if(d.getDatasetType() == Dataset.STREAMS) {
        mapReduceDataset = (StreamingAccess)d;
      }
    } catch (DatasetException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }

    while (mapReduceDataset.isDataAvailable()) {
      NBEvent nbEvent;
      try {
        nbEvent = mapReduceDataset.getStreamingData();
        processResultsFromMap(nbEvent);
      } catch (DatasetException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (MarshallingException e) {
        System.out.println("Caught Marshalling exception");
        System.out.println(e.getMessage());
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (MapReduceException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (NumberFormatException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (ServiceException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

    }


  }

  private void processResultsFromMap(NBEvent nbEvent) throws IOException, MarshallingException, MapReduceException, NumberFormatException, ServiceException {
    String s = (String) nbEvent.getContentSynopsis();
    byte[] resultBytes = nbEvent.getContentPayload();
    Results results = ResultsFactory.getInstance().getResults(resultBytes);
    byte[] resultPayloadBytes = results.getResultPayload();
    
    
    // may need this later...
    //byte[] decompressedBytes = decompressPayload(resultPayloadBytes, Integer
    //    .parseInt(results.getRespondTo()));
    ResultPayload resultPayload = ResultPayload.getInstance();

    if (results.getResultType() != ResultPayload.HASHMAP) {
      System.out
      .println("BayesPrepStage3: Incorrect ResultType received on Synopsis: " + s);
      return;
    }
    
    System.out.println("Synopsis: " + s);
    if(s.equals("DocFreqWeightMap") || s.equals("CombinedWeightMap")) {
      HashMap<GranTuple, Double> tmpMap = null;
      if(s.equals("DocFreqWeightMap")) {
        gotMap1 = true;
        tmpMap = (HashMap<GranTuple, Double>)resultPayload.getHashMap(resultPayloadBytes);
      }
      if(s.equals("CombinedWeightMap")) {
        gotMap2 = true;
        System.out.println("trying to decompress");
        byte [] decompressedBytes = decompressPayload(resultPayloadBytes, Integer.parseInt(results.getRespondTo()));
        System.out.println("trying to make hashtable");
        tmpMap = (HashMap<GranTuple, Double>)resultPayload.getHashMap(decompressedBytes);
      }
     // System.out.println("size of byte[]: " + resultPayloadBytes.length);
      
      System.out.println("Got tmpMap");
      WeightSummerMap(tmpMap);
      //replace with an inputcount if needed
      // setReceivedOutputFrom(results.getInstanceIdentifier());
    }

    System.out.println("Got 1?\t" + gotMap1);
    System.out.println("Got 2?\t" + gotMap2);
    if(gotMap1 && gotMap2) {
      // send on info to the last part
      // just write SigmaJ to file? 
      System.out.println("Sending featureWeightMap");
      writeResults(featureWeightMap, "SigmaJ");
      System.out.println("Sending labelWeightMap");
      writeResults(labelWeightMap, "SigmaK");
      System.out.println("Sending totalSum");
      writeResults(totalSum, "SigmaKSigmaJ");
      System.out.println("Done sending stuff from prep3");
    }

  }

  //handles a working set passed from BayesPrepStage2
  private void WeightSummerMap(HashMap<GranTuple, Double> workingSet) {
    System.out.println("in WeightSummer map");
    for(GranTuple key : workingSet.keySet()) {
      String label = key.stringAt(0);
      String feature = key.stringAt(1);
      Double value = workingSet.get(key);
      //System.out.println("In Stage 3, value=" + value);
      // first update the featureWeightMap
      if(featureWeightMap==null) {
        System.out.println("In BayesPrep3, featureWeightMap is NULL!!!!");
      }
      if(featureWeightMap.containsKey(feature) && value != null) {
        featureWeightMap.put(feature, featureWeightMap.get(feature) + value);
      }
      else if(value != null) {
        // haven't seen this feature yet
        featureWeightMap.put(feature, value);
      }
      else {
        System.out.println("Value for " + key + " is null!");
      }
      // Next update labelWeightMap
      if(labelWeightMap.containsKey(label) && value != null) {
        labelWeightMap.put(label, labelWeightMap.get(label) + value);
      }
      else if (value != null){
        // haven't seen this label yet
        labelWeightMap.put(label, value);
      }
      else {
        System.out.println("Value for " + key + " is null!");
      }
      //finally, update the running sum
      if(value != null) {
        totalSum += value;
      }
      else {
        nullCount++;
        System.out.println("Got another null! - " + nullCount);
      }
    }
  }

  private void writeResults(HashMap map, String label) throws MarshallingException, MapReduceException {
    boolean completeResult = true;
    ResultPayload resultPayload = ResultPayload.getInstance();

    byte[] resultPayloadBytes = resultPayload.getBytes(map);

    Results results = createResults(completeResult, false);
    results.setResultPayload(resultPayloadBytes);
    results.setResultType(ResultPayload.HASHMAP);
    results.setRespondTo(Integer.toString(resultPayloadBytes.length));
    this.writeResults(label, results);
  }

  private void writeResults(double toSend, String label) throws MarshallingException, MapReduceException {
    boolean completeResult = false;
    ResultPayload resultPayload = ResultPayload.getInstance();

    byte [] resultPayloadBytes = resultPayload.getBytes(toSend);

    Results results = createResults(completeResult, false);
    results.setResultPayload(resultPayloadBytes);
    results.setResultType(ResultPayload.DOUBLE);
    results.setRespondTo(Integer.toString(resultPayloadBytes.length));
    this.writeResults(label, results);
  }

  private byte[] decompressPayload(byte[] compressedPayload, int originalLength)
  throws ServiceException {
    Inflater decompresser = new Inflater();
   // System.out.println("in decompress, original length: " + originalLength);
    decompresser.setInput(compressedPayload);
  //  System.out.println("set input, making holder byte array");
    byte[] originalPayload = new byte[originalLength];
    System.out.println("about to inflate");
    try {
      int resultLength = decompresser.inflate(originalPayload);
      System.out.println("Done inflating!");
      System.out.println("input length: " + originalLength + " decompressed length: " + resultLength);
    } catch (DataFormatException dfe) {
      System.out.println("decompression failed :'(");
      throw new ServiceException("Reducer Decompression failed => "
          + dfe.toString());
    }
  //  System.out.println("returning decompressed payload");
    
    return originalPayload;

  }

}
