/**
 * Software License, Version 1.0 Copyright 2010 Colorado State University. All
 * rights reserved. Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following conditions
 * are met: 1) All redistributions of source code must retain the above
 * copyright notice, the list of authors in the original source code, this list
 * of conditions and the disclaimer listed in this license; 2) All
 * redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the disclaimer listed in this license in the
 * documentation and/or other materials provided with the distribution; 3) Any
 * documentation included with all redistributions must include the following
 * acknowledgment: "This product includes software developed by the Department
 * of Computer Science at Colorado State University. For further information
 * contact the department at http://www.cs.colostate.edu/." Alternatively, this
 * acknowledgment may appear in the software itself, and wherever such
 * third-party acknowledgments normally appear. 4) The name Colorado State
 * University or Granules shall not be used to endorse or promote products
 * derived from this software without prior written permission from Colorado
 * State University. For written permission, please contact the Computer Science
 * Department at 1100 Center Avenue, Fort Collins, CO 80523-1873. 5) Products
 * derived from this software may not be called Granules, nor may Colorado State
 * University or Granules appear in their name, without prior written permission
 * of ARTI. Colorado State University provides no reassurances that the source
 * code provided does not infringe the patent or any other intellectual property
 * rights of any other entity. Colorado State University disclaims any liability
 * to any recipient for claims brought by any other entity based on infringement
 * of intellectual property rights or otherwise. LICENSEE UNDERSTANDS THAT
 * SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES AS TO CAPABILITIES OR
 * ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES NO WARRANTIES AND MAKES NO
 * REPRESENTATION THAT SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT,
 * COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. COLORADO STATE UNIVERSITY MAKES NO
 * WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES",
 * "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO
 * THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
 */
package ds.granules.learning.clustering.dirichlet;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.FileUtil;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.PathFilter;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.mahout.clustering.Cluster;
import org.apache.mahout.clustering.ModelDistribution;
import org.apache.mahout.clustering.dirichlet.DirichletCluster;
import org.apache.mahout.clustering.dirichlet.DirichletState;
import org.apache.mahout.clustering.dirichlet.UncommonDistributions;
import org.apache.mahout.clustering.dirichlet.models.AbstractVectorModelDistribution;
import org.apache.mahout.clustering.dirichlet.models.DistributionDescription;
import org.apache.mahout.clustering.fuzzykmeans.SoftCluster;
import org.apache.mahout.common.IOUtils;
import org.apache.mahout.common.distance.DistanceMeasure;
import org.apache.mahout.math.Vector;
import org.apache.mahout.math.VectorWritable;

import cgl.narada.event.NBEvent;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

/**
 * An example demonstrating the <code>Map</code> role within a Word Count
 * operation.
 * 
 * @author Shrideep Pallickara
 */
public class DirichletReducer extends MapReduceBase {
  private int iterations = 0;
  private int MaxIters;
  String workerType = "Reducer ";
  private DistanceMeasure measure;
  private String DM;
  private double convergenceDelta;
  boolean initialized;
  // HashMap<Integer, SoftCluster> currentClusters; //hashmap of clusterID,
  // cluster
  // boolean converged;
  String clusterFile;
  String synopsis = "ClusterInfo";
  long runTime;
  private DirichletState state;
  Cluster[] newClusters;
  String modelFactory;
  String modelPrototype;
  int modelNum;
  double m;

  boolean isFaultTolerant = false;

  public DirichletReducer() {
    setDomain("Generic");
    setVersionInformation(200);
    setOperationIdentifier("KMeansMahoutReducer");
    initialized = false;
  }

  private void initialize() {
    //System.out.println("REDUCER: in initialize()");
    runTime = 0;
    iterations = 0;
    Properties p = this.getProcessingDirectives();
    String dm = p.getProperty("Distance Measure");
    ClassLoader ccl = Thread.currentThread().getContextClassLoader();
    try {
      measure = ccl.loadClass(dm).asSubclass(DistanceMeasure.class)
      .newInstance();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    DM = dm;
    modelFactory = p.getProperty("Model Distribution");
    modelPrototype = p.getProperty("Model Prototype");
    modelNum = Integer.parseInt(p.getProperty("Num Models"));
    MaxIters = Integer.parseInt(p.getProperty("Max Iterations"));
    convergenceDelta = Double.parseDouble(p.getProperty("Convergence Delta"));
    clusterFile = p.getProperty("Initial Clusters");
    m = Double.parseDouble(p.getProperty("Alpha"));
    // loadInitialClusters(clusterFile);
    loadState(clusterFile);
    initialized = true;
   // System.out.println("REDUCER: done initializing");
  }

  /**
   * Loads the initial clusters from file
   * @param initClusters2
   *             File containing initial clusters
   */
  private void loadState(String initClusters2) {
    double alpha = 1;

    Configuration conf = new Configuration();
    Path clusterPath = new Path(initClusters2, "*");

    try {
/*      ModelDistribution<VectorWritable> modelDistribution;
      ClassLoader ccl = Thread.currentThread().getContextClassLoader();
      Class<? extends AbstractVectorModelDistribution> cl = ccl.loadClass(
          modelFactory).asSubclass(AbstractVectorModelDistribution.class);
      modelDistribution = cl.newInstance();

      Class<? extends Vector> vcl = ccl.loadClass(modelPrototype).asSubclass(
          Vector.class);
      Constructor<? extends Vector> v = vcl.getConstructor(int.class);
      ((AbstractVectorModelDistribution) modelDistribution)
      .setModelPrototype(new VectorWritable(v.newInstance(modelNum)));

      state = new DirichletState(modelDistribution, modelNum, m);
      // get all filtered file names in result list     
      
 */
      DistributionDescription description = new DistributionDescription(modelFactory, modelPrototype, DM, modelNum);
      try {
        state = new DirichletState(description, modelNum, m);
        }
        catch (Exception e) {
          System.out.println("caught something!!!");
          e.printStackTrace();
        }
   //     System.out.println("made a new state");
        // get all filtered file names in result list   
        FileSystem fs = clusterPath.getFileSystem(conf);
        PathFilter clusterFileFilter = new PathFilter() {
          @Override
          public boolean accept(Path path) {
            return path.getName().startsWith("part");
          }
        };
        FileStatus[] matches = fs.listStatus(FileUtil.stat2Paths(fs.globStatus(
            clusterPath, clusterFileFilter)), clusterFileFilter);
        System.out.println("There are " + matches.length + " matches in " + clusterPath + "!");

        for (FileStatus match : matches) {
          //System.out.println("working on match: " + match.getPath());
          SequenceFile.Reader reader = null;
          try {
            reader = new SequenceFile.Reader(fs, match.getPath(), conf);
            Writable key = new Text();
            DirichletCluster c = new DirichletCluster();
            while (reader.next(key, c)) {
              // get the cluster info
              state.getClusters().set(Integer.parseInt(key.toString()), c);
              c = new DirichletCluster();
            }
          } catch (Exception e) {
            System.out.println("Error: " + e.getMessage());
            e.printStackTrace();
          } finally {
            reader.close();
          }

        }
      } catch (IOException e) {
        System.out.println("Problem with IO");
        throw new IllegalStateException(e);
      }
      state.setMixture(UncommonDistributions.rDirichlet(state.totalCounts(), m));
    }

  public void execute() {
    try {
      //System.out.println("REDUCER: in execute()");
      if (!initialized)
        initialize();
      //System.out.println("REDUCER: back in execute()");
      StreamingAccess mapReduceDataset = getMapReduceDataset();

      while (mapReduceDataset.isDataAvailable()) {
       // System.out.println("REDUCER: getting NBEvent");
        NBEvent nbEvent = mapReduceDataset.getStreamingData();
        processResultsFromMap(nbEvent);
      }

    } catch (DatasetException e) {
      e.printStackTrace();
    } catch (IOException e) {
      System.out.println("Problems converting bytes to longs");
      e.printStackTrace();
    } catch (MarshallingException e) {
      System.out.println("Problems Marshalling: " + e.getMessage());
      e.printStackTrace();
    } catch (MapReduceException e) {
      System.out.println("Problems writing Results: " + e.getMessage());
      e.printStackTrace();
    }

  }

  private synchronized void processResultsFromMap(NBEvent nbEvent)
  throws IOException, MarshallingException, MapReduceException {
   // System.out.println("REDUCER: processing data from map");
    if (!initialized)
      initialize();
    byte[] resultBytes = nbEvent.getContentPayload();
    Results results = ResultsFactory.getInstance().getResults(resultBytes);
    byte[] resultPayloadBytes = results.getResultPayload();

    ResultPayload resultPayload = ResultPayload.getInstance();

    if (results.getResultType() != ResultPayload.HASHMAP) {
      System.out
      .println("Mahout KMeans Reducer: Incorrect ResultType received");
      return;
    }

    /**
     * Remainder of processing assumes that the resultPayload type is of type
     * ResultPayload.INT
     */
    long startTime = System.nanoTime();
    HashMap<Integer, GranulesDirichletCluster> incomingMap = (HashMap<Integer, GranulesDirichletCluster>) resultPayload
    .getHashMap(resultPayloadBytes);
    // cummulativeValue += value;

    setReceivedOutputFrom(results.getInstanceIdentifier());

    handleIncoming(incomingMap);
    runTime += System.nanoTime() - startTime;

    //System.out.println("REDUCER: is awaiting outputs?\t" + isAwaitingOutputs());
    if (!isAwaitingOutputs()) {
      resetOutputTracker();
      iterations++;
      System.out.println("Reducer: Finished Iteration: " + iterations + "/"
          + MaxIters);

      startTime = System.nanoTime();

      List<DirichletCluster> newclust = state.getClusters();

      runTime += System.nanoTime() - startTime;
      if (iterations < MaxIters) {
        writeResult(newclust, false);
        if(isFaultTolerant) {
          writeToFile();
        }
      }
      else {
        // ready to start again!!
        initialized = false;
        //Timing code
        FileWriter fw = new FileWriter(new File("Reducer.timing"),
            true);
        fw.write(runTime + "\n");
        fw.flush();
        fw.close();
        byte[] rpb = resultPayload.getBytes("Done");

        Results r = createResults(true, false);
        r.setResultPayload(rpb);
        r.setResultType(ResultPayload.STRING);

        this.writeResults("Termination", r);
        writeToFile();
        state = null;
        newClusters = null;
        System.gc();
      }
    }

  }

  /**
   * Write final cluster state out to file.
   * Currently following Hadoop reducer output format.
   */
  private void writeToFile() {
    Configuration c = new Configuration();
    Path p = new Path("../GranulesOut/part-r-" + iterations);
    SequenceFile.Writer writer;
    try {
      writer = new SequenceFile.Writer(p.getFileSystem(c), c, p, Text.class,
          SoftCluster.class);
      Writable w;
      for (DirichletCluster clus : state.getClusters()) {
        w = new Text(String.valueOf(clus.getId()));
        writer.append(w, clus);
      }

      writer.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Writes results of this iteration of clustering back to mappers.
   * @param newclust
   *             <code>List</code> of new clusters
   * @param processingComplete
   *             Is processing complete
   * @throws MarshallingException
   *             If there is a problem converting to <code>byte[]</code>
   * @throws MapReduceException
   *             If there is a problem communicating with the mappers
   * @throws IOException
   *             Any other IO exception
   */
  private void writeResult(List<DirichletCluster> newclust,
      boolean processingComplete) throws MarshallingException,
      MapReduceException, IOException {
    List<GranulesDirichletCluster> sending = new ArrayList<GranulesDirichletCluster>();
    for (DirichletCluster c : newclust)
      sending.add(new GranulesDirichletCluster(c));
    ResultPayload resultPayload = ResultPayload.getInstance();
    byte[] resultPayloadBytes = resultPayload.getBytes(sending.toArray());

    Results results = createResults(true, processingComplete);
    results.setResultPayload(resultPayloadBytes);
    results.setResultType(ResultPayload.OBJECT_ARRAY);

    writeResults(synopsis, results);
  }

  /**
   * Process results from a single mapper
   * @param incomingMap
   *             <code>HashMap</code> from a mapper
   */
  private void handleIncoming(HashMap<Integer, GranulesDirichletCluster> incomingMap) {
    if (newClusters == null)
      newClusters = (Cluster[]) state.getModelFactory().sampleFromPosterior(
          state.getModels());
    for (int key : incomingMap.keySet()) {
      DirichletCluster p = incomingMap.get(key).getCluster();
      Cluster model = (Cluster) p.getModel();
      DirichletCluster c = state.getClusters().get(key);
      c.setModel(model);
    }
  }

}