package ds.granules.naadsm;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.InetAddress;
import java.util.LinkedList;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.event.Checkpoint;
import ds.granules.naadsm.event.Restore;
import ds.granules.naadsm.event.SplitEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;
import ds.granules.naadsm.event.EndOfDayEvent;
import ds.granules.naadsm.event.EndOfSimulationEvent;
import ds.granules.naadsm.event.EventType;
import ds.granules.naadsm.event.EventTypeFactory;
import ds.granules.naadsm.event.StartNewDayEvent;

public class Controller extends MapReduceBase {

	private int numOfSplitWorkers;
	private int currentDay;
	private Integer numOfFinishedSplittingWorkers;
	private LinkedList<Boolean> finishedWorker;
	private LinkedList<Boolean> doneWorker;
	private boolean finished;
	private int numberOfWorkers;
	private int numberOfSpareWorkers;
	private long[] executionTime;
	// LoadGenerator loadGenerator;
	private Results exchangeResults;
	private byte[] buffer;
	long startTime;
	long endTime;
	File record;
	BufferedWriter writer;
	long time1;
	long time2;
	String hostName;

	private synchronized int getNextSpare() {
		numberOfWorkers++;
		numberOfSpareWorkers--;
		return numberOfWorkers - 1;
	}

	public Controller() {

		startTime = System.currentTimeMillis();
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		// loadGenerator = null;
		finished = false;
		exchangeResults = createResults(false, false);
		buffer = new byte[0];

	}

	@Override
	public void execute() throws ProcessingException {

		// TODO Auto-generated method stub
		StreamingAccess access = null;
		String datasetIdentifier = "CONTROLLER_DATA";
		DatasetCollection datasetCollection = getDatasetCollection();
		try {
			Dataset dataset = datasetCollection.getDataset(datasetIdentifier);
			if (dataset == null) {
				System.out
						.println("OMG!!!!!!!!!!!!!!!!!!!!!NULL DATASET!!!!!!!!!!!!!!!!!!!!!!!!");
				return;
			}
			if (dataset.getDatasetType() == Dataset.STREAMS) {
				access = (StreamingAccess) dataset;
			} else {
				System.err.println("Incorrect dataset:"
						+ dataset.getDatasetType());
				return;
			}
			if (access != null) {
				while (dataset.isDataAvailable()) {
					NBEvent nbEvent = access.getStreamingData();
					if (nbEvent == null) {
						System.out
								.println("####################Receive null message from Stream dataset");
						continue;
					}
					System.out.println(nbEvent.getContentSynopsis());
					if (nbEvent.getContentSynopsis().equals("Sync/Bootstrap")) {
						bootstrapProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().equals(
							"Sync/UpStream")) {
						// System.out.println("Got from worker");
						upStreamProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().toString()
							.equals("Sync/Checkpoint")) {
						// System.out
						// .println("Got checkpoint message from workers");
						checkpointProcess(nbEvent);

					}
					if (finished)
						access.close();

				}
			}
		} catch (DatasetException e) {
			// TODO Auto-generated catch block
			System.out.println("DatasetException");
			e.printStackTrace();
		} catch (ServiceException e) {
			// TODO Auto-generated catch block
			System.out.println("ServiceException");
			e.printStackTrace();
		} catch (NBEventException e) {
			System.out.println("NBEventException");
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("IOException");
			e.printStackTrace();
		} catch (MapReduceException e) {
			System.out.println("MapReduceException");
			e.printStackTrace();
		} catch (MarshallingException e) {
			System.out.println("MarshallingException");
			e.printStackTrace();
		}

	}

	private byte[] pickupEndOfDay(byte[] message) {
		int length = message.length;
		byte[] endOfDayArray = new byte[EventType.EventSize[EventType.END_OF_DAY]];
		for (int i = 0; i < EventType.EventSize[EventType.END_OF_DAY]; i++) {
			endOfDayArray[i] = message[length
					- EventType.EventSize[EventType.END_OF_DAY] + i];
		}
		return endOfDayArray;
	}

	private byte[] combineByte(byte[] message1, byte[] message2) {
		byte[] result;
		if (message1.length == 0) {
			result = new byte[message2.length
					- EventType.EventSize[EventType.END_OF_DAY]];
			for (int i = 0; i < message2.length
					- EventType.EventSize[EventType.END_OF_DAY]; i++) {
				result[i] = message2[i];
			}
		} else {
			result = new byte[message1.length + message2.length
					- EventType.EventSize[EventType.END_OF_DAY]];
			for (int i = 0; i < message1.length; i++) {
				result[i] = message1[i];
			}
			for (int i = 0; i < message2.length
					- EventType.EventSize[EventType.END_OF_DAY]; i++) {
				result[i + message1.length] = message2[i];
			}
		}
		return result;
	}

	private synchronized void checkpointProcess(NBEvent nbEvent)
			throws MapReduceException, IOException, MarshallingException {
		Results resultsEx = ResultsFactory.getInstance().getResults(
				nbEvent.getContentPayload());
		byte[] reportBytes = resultsEx.getResultPayload();
		Checkpoint checkpoint = new Checkpoint();
		checkpoint = checkpoint.getBoundaries(reportBytes);
		// System.out.println("Received checkpoint message length:" +
		// checkpoint.marshalling().length);
		Restore restore = new Restore();
		restore.getBoundaries(checkpoint.addBoundaries());
		int newWorker = this.getNextSpare();
		System.out.println("Number of new worker: " + newWorker);
		restore.worker_ID = newWorker;
		exchangeResults = createResults(false, false);
		exchangeResults.setResultPayload(restore.addBoundaries());
		writeResults("Sync/DownStream/" + newWorker, exchangeResults);
		synchronized (numOfFinishedSplittingWorkers) {
			numOfFinishedSplittingWorkers++;
		}
		System.out.println("done: " + numOfFinishedSplittingWorkers);
		System.out.println("need: " + numOfSplitWorkers);

		if (numOfFinishedSplittingWorkers == numOfSplitWorkers) {
			byte[] bytes;
			StartNewDayEvent startNewDay = new StartNewDayEvent(0,
					currentDay + 1);
			System.out.println("All done:" + " Day " + currentDay);
			bytes = startNewDay.marshalling();
			byte[] finalResult = new byte[buffer.length
					+ EventType.EventSize[EventType.START_NEW_DAY] + 1];
			for (int i = 0; i < buffer.length; i++) {
				finalResult[i] = buffer[i];
			}
			for (int i = 0; i < bytes.length; i++) {
				finalResult[buffer.length + i] = bytes[i];
			}
			finalResult[buffer.length
					+ EventType.EventSize[EventType.START_NEW_DAY]] = 0;
			buffer = new byte[0];
			System.out.println("FinalResult.length = " + finalResult.length);
			Results ckptMessage = createResults(false, false);

			ckptMessage.setResultPayload(finalResult);
			writeResults("Sync/DownStream", ckptMessage);
			// Record something after sending message to workers
			// writer.write("" + endOfDayEvent.getDayNumber());
			// writer.newLine();
			// writer.write("" + finalResult.length);
			// writer.newLine();
			time2 = System.currentTimeMillis();
			// writer.write("" + (int) (time2 - time1));
			System.out.println("Day: " + currentDay + "  Length: "
					+ finalResult.length + "  Time: " + (int) (time2 - time1));
			time1 = time2;
		}
	}

	private synchronized void upStreamProcess(NBEvent nbEvent)
			throws MapReduceException, ServiceException, NBEventException,
			IOException, MarshallingException {
		int numberOfWorkers = this.numberOfWorkers;
		Results resultsEx = ResultsFactory.getInstance().getResults(
				nbEvent.getContentPayload());
		byte[] reportBytes = resultsEx.getResultPayload();

		buffer = combineByte(buffer, reportBytes);
		EndOfDayEvent endOfDayEvent = (EndOfDayEvent) EventTypeFactory
				.createEventType(pickupEndOfDay(reportBytes));
		currentDay = endOfDayEvent.getDayNumber();
		int num = endOfDayEvent.getWorkerID();
		long timeTemp = System.currentTimeMillis();
		long elapsedTime = timeTemp - time1;
		executionTime[num] = elapsedTime;
		// try {
		// File outputWorker = new File("/s/" + hostName
		// + "/a/nobackup/granules/output" + num + ".log");
		// BufferedWriter outWorker = new BufferedWriter(new FileWriter(
		// outputWorker, true));
		// outWorker.write("" + elapsedTime);
		// outWorker.newLine();
		// outWorker.close();
		// } catch (IOException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		if (endOfDayEvent.isFinished()) {

			finishedWorker.set(endOfDayEvent.getWorkerID(), true);
			doneWorker.set(endOfDayEvent.getWorkerID(), true);
			boolean flag = true;
			for (int i = 0; i < numberOfWorkers; i++) {
				if (!finishedWorker.get(i)) {
					flag = false;
					break;
				}
			}
			if (flag) {
				System.out.println("All finished:" + " Day "
						+ endOfDayEvent.getDayNumber());
				endTime = System.currentTimeMillis();
				System.out.println("Elapsed Time =" + (endTime - startTime));

				EndOfSimulationEvent endOfSimulationEvent = new EndOfSimulationEvent();
				byte[] bytes = endOfSimulationEvent.marshalling();
				byte[] finalResult = new byte[buffer.length
						+ EventType.EventSize[EventType.END_OF_SIMULATION] + 1];

				for (int i = 0; i < buffer.length; i++) {
					finalResult[i] = buffer[i];
				}
				for (int i = 0; i < bytes.length; i++) {
					finalResult[buffer.length + i] = bytes[i];
				}
				finalResult[buffer.length
						+ EventType.EventSize[EventType.END_OF_SIMULATION]] = 1;
				exchangeResults = createResults(true, true);
				exchangeResults.setResultPayload(finalResult);
				writeResults("Sync/DownStream", exchangeResults);

				Results results = createResults(true, true);
				String res = "END";
				results.setResultPayload(res.getBytes());
				writeResults("Sync/Result", results);
				writer.write("" + endOfDayEvent.getDayNumber());
				writer.newLine();
				writer.write("" + finalResult.length);
				writer.newLine();
				time2 = System.currentTimeMillis();
				writer.write("" + (int) (time2 - time1));
				System.out.println("Day: " + endOfDayEvent.getDayNumber()
						+ "  Length: " + finalResult.length + "  Time: "
						+ (int) (time2 - time1));
				time1 = time2;
				writer.newLine();
				System.out.println("Writer Closed!");
				writer.close();
				System.out
						.println("Invoked Termination Condition Reached on Controller.");
				return;
			}
		} else {
			doneWorker.set(endOfDayEvent.getWorkerID(), true);
		}

		boolean flag = true;
		for (int i = 0; i < numberOfWorkers; i++) {
			if (!doneWorker.get(i)) {
				flag = false;
				break;
			}
		}
		System.out.println("NUMBER OF WORKERS = " + numberOfWorkers);
		for (int i = 0; i < numberOfWorkers; i++) {
			System.out.println(doneWorker.get(i));
		}
		if (flag) {
			for (int i = 0; i < numberOfWorkers; i++) {
				doneWorker.set(i, false);
				finishedWorker.set(i, false);
			}
			// See which one should be splitted
			numOfSplitWorkers = 0;
			// System.out.println("!!!");
			if (numberOfSpareWorkers > 0) {
				long minTime = Long.MAX_VALUE;
				for (int i = 0; i < numberOfWorkers; i++) {
					if (minTime > executionTime[i]) {
						minTime = executionTime[i];
					}
				}
				int workerLeft = numberOfSpareWorkers;
				int workerActive = numberOfWorkers;
				for (int i = 0; i < workerActive; i++) {
					if (workerLeft == 0) {
						// System.out.println("I am going out!");
						break;
					}
					if (executionTime[i] > 20000) {
						numOfSplitWorkers++;
						SplitEvent splitEvent = new SplitEvent();
						splitEvent.worker_ID = i;
						splitEvent.howToSplit = 1;
						splitEvent.coordinate = 0;
						// System.out.println("???");
						exchangeResults = createResults(false, false);
						exchangeResults.setResultPayload(splitEvent
								.marshalling());
						// byte[] bb = splitEvent.marshalling();
						// System.out.println(bb.length);
						// for (int j = 0; j < bb.length; j++) {
						// System.out.print("" + bb[j] + " ");
						// }
						// System.out.println();
						writeResults("Sync/DownStream/" + i, exchangeResults);
						System.out.println("SPLIT :" + i);
						workerLeft--;
						System.out.println("LEFT WORKER = " + workerLeft);
					}
				}
				// System.out.println("I am out");
			}
			// System.out.println("I am out again");
			numOfFinishedSplittingWorkers = 0;
			// System.out.println("L " + numOfFinishedSplittingWorkers);
			// System.out.println("H " + numOfSplitWorkers);
			if (numOfSplitWorkers == 0) {

				// System.out.println("PASSED SPLIT!");
				numOfFinishedSplittingWorkers = 0;

				byte[] bytes;
				StartNewDayEvent startNewDay = new StartNewDayEvent(0,
						endOfDayEvent.getDayNumber() + 1);
				System.out.println("All done:" + " Day "
						+ endOfDayEvent.getDayNumber());
				bytes = startNewDay.marshalling();
				byte[] finalResult = new byte[buffer.length
						+ EventType.EventSize[EventType.START_NEW_DAY] + 1];
				for (int i = 0; i < buffer.length; i++) {
					finalResult[i] = buffer[i];
				}
				for (int i = 0; i < bytes.length; i++) {
					finalResult[buffer.length + i] = bytes[i];
				}
				finalResult[buffer.length
						+ EventType.EventSize[EventType.START_NEW_DAY]] = 0;
				buffer = new byte[0];
				System.out
						.println("FinalResult.length = " + finalResult.length);
				exchangeResults = createResults(false, false);

				exchangeResults.setResultPayload(finalResult);
				writeResults("Sync/DownStream", exchangeResults);
				// Record something after sending message to workers
				// writer.write("" + endOfDayEvent.getDayNumber());
				// writer.newLine();
				// writer.write("" + finalResult.length);
				// writer.newLine();
				time2 = System.currentTimeMillis();
				// writer.write("" + (int) (time2 - time1));
				System.out.println("Day: " + endOfDayEvent.getDayNumber()
						+ "  Length: " + finalResult.length + "  Time: "
						+ (int) (time2 - time1));
				time1 = time2;
				// writer.newLine();
			}
		}
	}

	private void bootstrapProcess(NBEvent nbEvent) throws MapReduceException,
			ServiceException, NBEventException {
		// TODO Auto-generated method stub
		hostName = "";
		try {
			InetAddress address = InetAddress.getLocalHost();
			hostName = address.getHostName();
		} catch (Exception e) {
		}
		// record = new File("/s/" + hostName +
		// "/a/nobackup/granules/record.txt");
		// try {
		// writer = new BufferedWriter(new FileWriter(record));
		// } catch (IOException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		numberOfWorkers = 1;
		numberOfSpareWorkers = Integer.parseInt(this.getProcessingDirectives()
				.get("Number of Workers").toString()) - 1;
		executionTime = new long[numberOfSpareWorkers + numberOfWorkers];
		finishedWorker = new LinkedList<Boolean>();
		doneWorker = new LinkedList<Boolean>();
		for (int i = 0; i < numberOfWorkers + numberOfSpareWorkers; i++) {
			finishedWorker.add(new Boolean(false));
			doneWorker.add(new Boolean(false));
		}
		// if (loadGenerator == null) {
		// loadGenerator = new LoadGenerator(this.getProcessingDirectives());
		// }
		StartNewDayEvent startEvent = new StartNewDayEvent(0, 1);
		byte[] startMessage = new byte[EventType.EventSize[EventType.START_NEW_DAY] + 1];
		// byte[] l = EventTypeFactory
		// .int2bytes(EventType.EventSize[EventType.START_NEW_DAY]);
		// startMessage[0] = l[0];
		// startMessage[1] = l[1];
		// startMessage[2] = l[2];
		// startMessage[3] = l[3];
		byte[] start = startEvent.marshalling();
		for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
			startMessage[i] = start[i];
		}
		startMessage[EventType.EventSize[EventType.START_NEW_DAY]] = 0;
		// loadGenerator.createEvent("Sync/DownStream",
		// startEvent.marshalling());
		// Results exchangeResults = createResults(false, false);
		exchangeResults.setResultPayload(startMessage);
		writeResults("Sync/DownStream", exchangeResults);
		time1 = System.currentTimeMillis();

	}

}
