package ds.granules.naadsm;

import java.io.IOException;
import java.util.Properties;

import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.DatasetFactory;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.CommunicationsException;
import ds.granules.exception.DeploymentException;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.InstanceDeployer;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProgressTracker;

public class Deployer extends InstanceDeployer {
	private ProgressTracker progressTracker;
	private Worker[] workers;
	private Controller[] controller;
	private Properties props;
	private int finishedJob;

	public Deployer(Properties properties) throws CommunicationsException,
			IOException, MarshallingException, DeploymentException {
		finishedJob = 0;
		props = properties;
		initialize(properties);
	}

	public Worker initializeWorker(int workerNum, int number, int up, int down,
			int left, int right) throws DatasetException, MapReduceException {
		// TODO Auto-generated method stub
		Worker worker = new Worker();
		Properties props = (Properties) this.props.clone();
		String key = "Worker Number";
		String value = new String();
		value = "" + workerNum;
		props.put(key, value);
		String u = "Upper";
		String uV = "" + up;
		props.put(u, uV);
		String d = "Down";
		String dV = "" + down;
		props.put(d, dV);
		String l = "Left";
		String lV = "" + left;
		props.put(l, lV);
		String r = "Right";
		String rV = "" + right;
		props.put(r, rV);

		worker.setProcessingDirectives(props);

		DatasetFactory datasetFactory = DatasetFactory.getInstance();
		StreamingAccess streamAccess = datasetFactory.createStreamingDataset(
				"WORKER_DATA" + value, "WORKER DATA");

		String downstreamSynopsis2 = "Sync/DownStream";
		streamAccess.addInputStream(downstreamSynopsis2,
				StreamingAccess.STRING_SYNOPSIS);
		String downstreamSynopsis3 = "Sync/DownStream/" + workerNum;
		streamAccess.addInputStream(downstreamSynopsis3,
				StreamingAccess.STRING_SYNOPSIS);

		DatasetCollection datasetCollection = worker.getDatasetCollection();
		datasetCollection.addDataset(streamAccess);
		worker.setDatasetCollection(datasetCollection);
		worker.setAsExecuteWhenDataAvailable();
		return worker;
	}

	public Controller initializeController(int numOfWorkers)
			throws DatasetException, MapReduceException {
		Controller controller = new Controller();
		Properties props = (Properties) this.props.clone();
		String key = "Number of Workers";
		String value = "" + numOfWorkers;
		props.put(key, value);
		controller.setProcessingDirectives(props);

		DatasetFactory datasetFactory = DatasetFactory.getInstance();
		StreamingAccess streamAccess = datasetFactory.createStreamingDataset(
				"CONTROLLER_DATA", "CONTROLLER DATA");

		String bootstrapSynopsis = "Sync/Bootstrap";
		String upstreamSynopsis = "Sync/UpStream";
		streamAccess.addInputStream(bootstrapSynopsis,
				StreamingAccess.STRING_SYNOPSIS);
		streamAccess.addInputStream(upstreamSynopsis,
				StreamingAccess.STRING_SYNOPSIS);
		String chkptSynopsis = "Sync/Checkpoint";
		streamAccess.addInputStream(chkptSynopsis,
				StreamingAccess.STRING_SYNOPSIS);
		
//		String[] upstreamSynopsises = new String[numOfWorkers];
//		for (int i = 0; i < numOfWorkers; i++) {
//			upstreamSynopsises[i] = "Sync/UpStream/" + i;
//			System.out.println(upstreamSynopsises[i]);
//			streamAccess.addInputStream(upstreamSynopsises[i],
//					StreamingAccess.STRING_SYNOPSIS);
//		}

		DatasetCollection datasetCollection = controller.getDatasetCollection();
		datasetCollection.addDataset(streamAccess);
		controller.setDatasetCollection(datasetCollection);
		controller.setAsExecuteWhenDataAvailable();
		return controller;
	}

	public void prepareInstancesAndDeploy(int number, int up, int down,
			int left, int right) {
		int numOfWorkers = number;
		MapReduceBase[] maps = new MapReduceBase[1 + numOfWorkers];

		this.controller = new Controller[1];
		this.workers = new Worker[numOfWorkers];
		try {
			for (int i = 0; i < numOfWorkers; i++) {
				// workers[i] = initializeWorker(i);
				maps[i] = initializeWorker(i, number, up, down, left, right);
			}

			// controller[0] = initializeController(numOfWorkers);
			maps[numOfWorkers] = initializeController(numOfWorkers);

			progressTracker = deployOperations(maps);
			// progressTracker = deployOperations(workers);
			// progressTracker = deployOperations(controller);
		} catch (CommunicationsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DeploymentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MarshallingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DatasetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MapReduceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	@Override
	public void completedSuccessfully(String jobIdentifier,
			ProgressTracker progressTracker) {
		finishedJob++;
		if (finishedJob == 1) {
			System.exit(0);
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		String hostname = args[0];
		String portnum = args[1];
		String num = args[2];
		String upS = args[3];
		String downS = args[4];
		String leftS = args[5];
		String rightS = args[6];
		Properties streamProperties = new Properties();
		streamProperties.put("hostname", hostname);
		streamProperties.put("portnum", portnum);
		int number = Integer.parseInt(num);
		int up = Integer.parseInt(upS);
		int down = Integer.parseInt(downS);
		int left = Integer.parseInt(leftS);
		int right = Integer.parseInt(rightS);

		try {
			Deployer deployer = new Deployer(streamProperties);
			deployer.prepareInstancesAndDeploy(number, up, down, left, right);
			// while (true) {
			// }

		} catch (CommunicationsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MarshallingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DeploymentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
