package ds.granules.naadsm;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaGenericComputation;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.event.Checkpoint;
import ds.granules.naadsm.event.EventType;
import ds.granules.naadsm.event.EventTypeFactory;
import ds.granules.naadsm.event.Restore;
import ds.granules.naadsm.event.SplitEvent;
import ds.granules.naadsm.event.StartNewDayEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

public class Worker extends MapReduceBase implements JavaMessageHandler {
	JavaGenericComputation jc;
	private float leftB;
	private float rightB;
	private float upperB;
	private float lowerB;
	private boolean active;
	private boolean finished;
	private Process proc;
	private InputStream is;
	private OutputStream os;

	private byte[] buffer;
	FileWriter writer;
	BufferedWriter out;

	boolean started;
	String sysCommand;
	// LoadGenerator loadGenerator;
	private Results exchangeResults;
	private boolean silent;
	private boolean silent1;

	File output;

	private byte[] combineByte(byte[] message1, byte[] message2) {
		byte[] result;
		if (message1.length == 0) {
			result = new byte[message2.length];
			for (int i = 0; i < message2.length; i++) {
				result[i] = message2[i];
			}
		} else {
			result = new byte[message1.length + message2.length];
			for (int i = 0; i < message1.length; i++) {
				result[i] = message1[i];
			}
			for (int i = 0; i < message2.length; i++) {
				result[i + message1.length] = message2[i];
			}
		}
		return result;
	}

	public Worker() {
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		// sysCommand =
		// "/s/chopin/e/proj/granules/simonsui/dis-naadsm/naadsm-3.2.6/src/naadsm -h /s/chopin/e/proj/granules/simonsui/dis-naadsm/660000_population_scaled_from_kansas_9_infected.xml /s/chopin/e/proj/granules/simonsui/dis-naadsm/Kansas_params_SlowerDetection_HigherInfectionRate.xml";

		started = false;
		finished = false;
		silent = true;
		active = false;
		jc = new JavaGenericComputation("Worker");
		buffer = new byte[0];
		// loadGenerator = null;
		exchangeResults = createResults(false, false);
	}

	@Override
	public void execute() throws ProcessingException {
		long time1 = System.currentTimeMillis();
		try {
			if (!started) {
				leftB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Left"));
				rightB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Right"));
				upperB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Upper"));
				lowerB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Down"));
				startSpare();
				started = true;
			}
			StreamingAccess access = null;
			String datasetIdentifier = "WORKER_DATA"
					+ this.getProcessingDirectives().getProperty(
							"Worker Number");
			DatasetCollection datasetCollection = getDatasetCollection();
			if (datasetCollection == null)
				return;
			try {
				Dataset dataset = datasetCollection
						.getDataset(datasetIdentifier);
				if (dataset == null) {
					System.err
							.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!DATASET IS NULL!!!!!!!!!!!!!!!");
					return;
				}
				if (dataset.getDatasetType() == Dataset.STREAMS) {
					access = (StreamingAccess) dataset;
				} else {
					System.err.println("Incorrect dataset:"
							+ dataset.getDatasetType());
					return;
				}
				if (access != null) {
					while (dataset.isDataAvailable()) {
						NBEvent nbEvent = access.getStreamingData();
						if (nbEvent == null) {
							// System.out.println("*******************Receive null message from Stream dataset");
							continue;
						}
						// System.out.println("***"+nbEvent.getContentSynopsis()+" Message Size = "+nbEvent.getContentPayloadSize()+"***");
						int num = Integer.parseInt(this
								.getProcessingDirectives().getProperty(
										"Worker Number"));
						if (num == 0)
							this.active = true;
						if (active
								&& nbEvent.getContentSynopsis().equals(
										"Sync/DownStream")) {
							downStreamProcess(nbEvent);
							if (finished)
								;
							// access.close();
						} else if (nbEvent.getContentSynopsis().equals(
								"Sync/DownStream/" + num)) {
							byte[] resultsPayload = nbEvent.getContentPayload();
							Results results = ResultsFactory.getInstance()
									.getResults(resultsPayload);
							byte[] bb = results.getResultPayload();
							// System.out.println(bb.length);
							// for (int i = 0; i < bb.length; i++) {
							// System.out.print("" + bb[i] + " ");
							// }
							// System.out.println();
							EventType eventType = EventTypeFactory
									.createEventType(bb);

							if (eventType.getType() == EventType.RESTORE) {
								activation(nbEvent);
							} else if (eventType.getType() == EventType.SPLIT) {
								System.out.println("SPLIT MESSAGE!");
								float coordinator = (leftB + rightB) / 2;
								SplitEvent splitEvent = (SplitEvent) eventType;
								splitEvent.coordinate = coordinator;

								JavaByteMessage jbm = new JavaByteMessage(
										ControlMessage.DATA);
								jbm.setInput(splitEvent.marshalling());
								byte[] totalMessage = jbm.toByteArray();
								if (!silent) {
									// out.write("----------------ALL Send to NAADSM:-------------------\n");
									// for (int i = 0; i < totalMessage.length;
									// i++)
									// out.write(Integer.toHexString(totalMessage[i])
									// + " ");
									// out.newLine();
									// out.flush();
									System.out
											.println("----------------ALL Send to NAADSM:-------------------");
									for (int i = 0; i < totalMessage.length; i++)
										System.out.print(Integer
												.toHexString(totalMessage[i])
												+ " ");
									System.out.println();
									System.out
											.println("---------------------------------------------------");
								}

								jc.sendMessage(jbm);
							}

						}
					}
				} else {
					System.err.println("The dataset is null!");
				}
			} catch (DatasetException e) {
				// TODO Auto-generated catch block
				System.out.println("DatasetException");
				e.printStackTrace();
			} catch (ServiceException e) {
				// TODO Auto-generated catch block
				System.out.println("ServiceException");
				e.printStackTrace();
			} catch (NBEventException e) {
				System.out.println("NBEventException");
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("IOException");
				e.printStackTrace();
			} catch (MapReduceException e) {
				System.out.println("MapReduceException");
				e.printStackTrace();
			} catch (MarshallingException e) {
				System.out.println("MarshallingException");
				e.printStackTrace();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		String hostName = "";
		try {
			InetAddress address = InetAddress.getLocalHost();
			hostName = address.getHostName();
		} catch (Exception e) {
		}
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));

		long time2 = System.currentTimeMillis();
		long elapsedTime = time2 - time1;
		System.out.println(num + ":" + elapsedTime);
		// try {
		// output = new File("/s/" + hostName +
		// "/a/nobackup/granules/output"
		// + num + ".log");
		// output = new File("output" + num + ".log");
		// writer = new FileWriter(output, true);
		// out = new BufferedWriter(writer);
		// writer.write("" + elapsedTime);
		//
		// out.close();
		// writer.close();
		// } catch (IOException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }

//		System.out.println("I reach the end");
	}

	private void activation(NBEvent nbEvent) throws IOException,
			MarshallingException {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));

		this.active = true;
		Restore restore = new Restore();
		restore = restore.getBoundaries(ResultsFactory.getInstance()
				.getResults(nbEvent.getContentPayload()).getResultPayload());
		this.leftB = restore.left;
		this.rightB = restore.right;
		this.lowerB = restore.down;
		this.upperB = restore.up;
		// System.out.println("Restore message length: "
		// + restore.marshalling().length);
		JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
		byte[] bytes = restore.marshalling();
		jbm.setInput(bytes);
		if (!silent) {
			// out.write("----------------Send to NAADSM:-------------------\n");
			// for (int i = 0; i < bytes.length; i++)
			// out.write(Integer.toHexString(bytes[i]) + " ");
			// out.newLine();
			System.out
					.println("----------------Send to NAADSM:-------------------");
			for (int i = 0; i < bytes.length; i++)
				System.out.print(Integer.toHexString(bytes[i]) + " ");
			System.out.println();
			System.out
					.println("---------------------------------------------------");
		}
		byte[] totalMessage = jbm.toByteArray();
		if (!silent) {
			// out.write("----------------ALL Send to NAADSM:-------------------\n");
			// for (int i = 0; i < totalMessage.length; i++)
			// out.write(Integer.toHexString(totalMessage[i]) + " ");
			// out.newLine();
			// out.flush();
			System.out
					.println("----------------ALL Send to NAADSM:-------------------");
			for (int i = 0; i < totalMessage.length; i++)
				System.out.print(Integer.toHexString(totalMessage[i]) + " ");
			System.out.println();
			System.out
					.println("---------------------------------------------------");
		}

		System.out.println("************ Worker " + num + " is active!");
		jc.sendMessage(jbm);
	}

	private void startSpare() {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		sysCommand = "/s/chopin/e/proj/granules/simonsui/dis-naadsm/naadsm-3.2.10__Dynamic/bin/naadsm -V 0 --rng-seed 3 -o naadsm_output_.log -h /s/chopin/e/proj/granules/simonsui/dis-naadsm/660000_population_scaled_from_kansas_9_infected.xml /s/chopin/e/proj/granules/simonsui/dis-naadsm/Kansas_params_SlowerDetection_HigherInfectionRate.xml";
		try {
			jc.startExecution(sysCommand, ControlMessage.UPIPES, this);
			System.out.println("Worker " + num + " is spared!");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void downStreamProcess(NBEvent nbEvent) throws ServiceException,
			NBEventException, IOException, MapReduceException,
			MarshallingException, NullPointerException {
		// TODO Auto-generated method stub
		// System.out.println(nbEvent.getEventHeaders().getEventId());
		byte[] extraSend = null;
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// System.out.println("Worker " + num + " did receive something.");
		// if (loadGenerator == null) {
		// loadGenerator = new LoadGenerator(this.getProcessingDirectives());
		// }
		byte[] eventPayload = nbEvent.getContentPayload();
		if (eventPayload == null)
		{
			System.out.println("Payload NULL");
			return;
		}
		Results resultsEx = ResultsFactory.getInstance().getResults(
				eventPayload);
		if (resultsEx == null) {
			// System.out.println("RESULT NULL");
			return;
		}

		byte[] send = resultsEx.getResultPayload();
		// System.out.println("Begin: Send.length = "+send.length+" Worker ID = "+num);

		if (send == null) {
			// System.out.println("Send NULL");
			return;
		}

		if (buffer != null) {
			extraSend = new byte[send.length + buffer.length];
			// byte[] temp = EventTypeFactory.int2bytes(send.length
			// + buffer.length);
			// extraSend[0] = temp[0];
			// extraSend[1] = temp[1];
			// extraSend[2] = temp[2];
			// extraSend[3] = temp[3];
			for (int i = 0; i < buffer.length; i++) {
				extraSend[i] = buffer[i];
			}
			for (int i = 0; i < send.length; i++) {
				extraSend[i + buffer.length] = send[i];
			}
			buffer = null;
			// send = extraSend;
		} else {
			extraSend = send;
		}
		// if (!silent) {
		// out.write("----------------Send IS:-------------------\n");
		// for (int i = 0; i < send.length; i++)
		// out.write(Integer.toHexString(send[i]) + " ");
		// out.newLine();
		// System.out.println("----------------Send IS:-------------------");
		// for (int i = 0; i < send.length; i++)
		// System.out.print(Integer.toHexString(send[i]) + " ");
		// System.out.println();
		// System.out
		// .println("---------------------------------------------------");
		// }
		if (extraSend[extraSend.length - 1] == 0) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] start = new byte[EventType.EventSize[EventType.START_NEW_DAY]];
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				start[i] = extraSend[extraSend.length - 1
						- EventType.EventSize[EventType.START_NEW_DAY] + i];
			}
			StartNewDayEvent startEvent = (StartNewDayEvent) EventTypeFactory
					.createEventType(start);
			// System.out.println("START:");

			startEvent.setWorker(num);

			start = startEvent.marshalling();
			// if (!silent) {
			// out.write("----------------START-------------------\n");
			// for (int i = 0; i < start.length; i++)
			// out.write(Integer.toHexString(start[i]) + " ");
			// out.newLine();
			// System.out.println("----------------START-------------------");
			// for (int i = 0; i < start.length; i++)
			// System.out.print(Integer.toHexString(start[i]) + " ");
			// System.out.println();
			// System.out
			// .println("---------------------------------------------------");
			// }
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				bytes[bytes.length
						- EventType.EventSize[EventType.START_NEW_DAY] + i] = start[i];
			}
			// byte[] temp = EventTypeFactory.int2bytes(bytes.length - 4);
			// bytes[0] = temp[0];
			// bytes[1] = temp[1];
			// bytes[2] = temp[2];
			// bytes[3] = temp[3];
			jbm.setInput(bytes);
			if (!silent) {
				// out.write("----------------Send to NAADSM:-------------------\n");
				// for (int i = 0; i < bytes.length; i++)
				// out.write(Integer.toHexString(bytes[i]) + " ");
				// out.newLine();
				System.out
						.println("----------------Send to NAADSM:-------------------");
				for (int i = 0; i < bytes.length; i++)
					System.out.print(Integer.toHexString(bytes[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				// out.write("----------------ALL Send to NAADSM:-------------------\n");
				// for (int i = 0; i < totalMessage.length; i++)
				// out.write(Integer.toHexString(totalMessage[i]) + " ");
				// out.newLine();
				// out.flush();
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			jc.sendMessage(jbm);

		} else if (extraSend[extraSend.length - 1] == 1) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			// byte[] temp = EventTypeFactory.int2bytes(bytes.length - 4);
			// bytes[0] = temp[0];
			// bytes[1] = temp[1];
			// bytes[2] = temp[2];
			// bytes[3] = temp[3];
			jbm.setInput(bytes);
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				// out.write("----------------ALL Send to NAADSM:-------------------\n");
				// for (int i = 0; i < totalMessage.length; i++)
				// out.write(Integer.toHexString(totalMessage[i]) + " ");
				// out.newLine();
				// out.flush();
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			jc.sendMessage(jbm);
		}

	}

	@Override
	public void handleMessage(final byte[] msg) {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// TODO Auto-generated method stub
		// fudging things to make asynchronous
		// System.out.println("Got Reply!");
		final byte[] par = msg;
		if (!silent) {
			try {
				// out.write("----------------Got From NAADSM:-------------------\n");
				// for (int i = 0; i < par.length; i++)
				// out.write(Integer.toHexString(par[i]) + " ");
				// out.newLine();
				// out.flush();
				System.out
						.println("----------------Got From NAADSM:-------------------");
				for (int i = 0; i < par.length; i++)
					System.out.print(Integer.toHexString(par[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			} catch (Exception e) {
				// TOFDO Auto-generated catch block
				e.printStackTrace();
			}
		}
		synchronized (exchangeResults) {
			JavaByteMessage jbm = new JavaByteMessage(par);

			byte[] bytes = jbm.getInput();

			boolean isCheckpoint;
			try {
				if (EventTypeFactory.createEventType(bytes).getType() == EventType.CHECKPOINT) {
					isCheckpoint = true;
				} else
					isCheckpoint = false;
			} catch (Exception e) {
				isCheckpoint = false;
			}
			// System.out.println("CHECKPOINT?:" + isCheckpoint);
			try {

				if (isCheckpoint) {
					Checkpoint checkpoint = (Checkpoint) EventTypeFactory
							.createEventType(bytes);
					checkpoint.left = (leftB + rightB) / 2;
					checkpoint.right = rightB;
					checkpoint.up = upperB;
					checkpoint.down = lowerB;
					System.out.println("Checkpoint message bytes number: "
							+ checkpoint.marshalling().length);
					this.rightB = (this.leftB + this.rightB) / 2;
					byte[] res = checkpoint.addBoundaries();
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(res);
					System.out.println("Worker sends out bytes number: "
							+ res.length);

					// System.out.println("Sync/UpStream/" + num);
					// System.out.println("Before sending checkpoint");
					writeResults("Sync/Checkpoint", exchangeResults);
					// System.out.println("After sending checkpoint");
				} else {
					// System.out.println("Send to Controller");
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(bytes);
					writeResults("Sync/UpStream", exchangeResults);
				}
			} catch (MapReduceException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}
}
