/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicmerge;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.dynamicmerge.event.Checkpoint;
import ds.granules.naadsm.dynamicmerge.event.EndOfDayEvent;
import ds.granules.naadsm.dynamicmerge.event.EndOfSimulationEvent;
import ds.granules.naadsm.dynamicmerge.event.EventType;
import ds.granules.naadsm.dynamicmerge.event.EventTypeFactory;
import ds.granules.naadsm.dynamicmerge.event.OutputVariableValue;
import ds.granules.naadsm.dynamicmerge.event.RequestForFTCheckpoint;
import ds.granules.naadsm.dynamicmerge.event.Restore;
import ds.granules.naadsm.dynamicmerge.event.SplitEvent;
import ds.granules.naadsm.dynamicmerge.event.StartNewDayEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;
import ds.granules.util.MD5Checksum;

/**
 * This is the controller for distributed NAADSM. It is responsible for
 * synchronization at the end of simulation days and finalizing data at the end
 * of simulation. Also it controls the split action. It will judge which worker
 * is going to split at the beginning of the next simulation day.
 * 
 * @author Zhiquan (Simon) Sui
 * 
 */
public class Controller extends MapReduceBase {

	private int numOfSplitWorkers;
	private int currentDay;
	private int numOfFinishedSplitAndMergeWorkers;
	private int numOfMergeWorkers;
	private boolean finished;
	private Results exchangeResults;
	private byte[] buffer;
	private boolean checkpointRequest;
	private boolean suspended;
	long startTime;
	long endTime;
	File record;
	BufferedWriter writer;
	long time1;
	long time2;
	String hostName;
	WorkerList workerList;
	HashMap<Integer, Integer> splitMap;
	HashMap<Integer, Integer> mergeMap;
	private boolean recordLog;

	// /**
	// * Get the number of next spare worker
	// *
	// * @return number of the worker
	// */
	// private synchronized int getNextSpare() {
	// numberOfWorkers++;
	// numberOfSpareWorkers--;
	// return numberOfWorkers - 1;
	// }

	/**
	 * Constructor. Initialize variables and byte array.
	 */
	public Controller() {
		startTime = System.currentTimeMillis();
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		finished = false;
		exchangeResults = createResults(false, false);
		buffer = new byte[0];
		splitMap = new HashMap<Integer, Integer>();
		mergeMap = new HashMap<Integer, Integer>();
		checkpointRequest = false;
		recordLog = true;
	}

	@Override
	public void execute() throws ProcessingException {
		StreamingAccess access = null;
		String datasetIdentifier = "CONTROLLER_DATA";
		DatasetCollection datasetCollection = getDatasetCollection();
		try {
			Dataset dataset = datasetCollection.getDataset(datasetIdentifier);
			if (dataset == null) {
				System.out.println("NULL DATASET!");
				return;
			}
			if (dataset.getDatasetType() == Dataset.STREAMS) {
				access = (StreamingAccess) dataset;
			} else {
				System.err.println("Incorrect dataset:"
						+ dataset.getDatasetType());
				return;
			}
			if (access != null) {
				while (dataset.isDataAvailable()) {
					NBEvent nbEvent = access.getStreamingData();
					if (nbEvent == null) {
						System.out
								.println("Receive null message from Stream dataset");
						continue;
					}

//					System.out.println(nbEvent.getContentSynopsis());

					/* Judge the type of the received message. */
					if (nbEvent.getContentSynopsis().equals(
							"Async/ResumeRequest")) {
						System.out.println("(i) Resuming operation.");
						buffer = new byte[0];
						this.suspended = false;
					} else if (suspended) {
						System.out.println("(i) Suspended; Ignoring message: "
								+ nbEvent.getContentSynopsis());
						continue;
					} else if (nbEvent.getContentSynopsis().equals(
							"Async/SuspendRequest")) {
						System.out.println("(i) Suspending operation.");
						this.suspended = true;

						ResultsFactory resultsFactory = ResultsFactory
								.getInstance();
						Results results = resultsFactory.getResults(nbEvent
								.getContentPayload());
						byte[] resultBytes = results.getResultPayload();

						ByteArrayInputStream byteIn = new ByteArrayInputStream(
								resultBytes);
						ObjectInputStream objIn = new ObjectInputStream(byteIn);
						WorkerList workers = null;
						try {
							workers = (WorkerList) objIn.readObject();
						} catch (Exception e) {
							System.out
									.println("(!) Could not recover Worker List.");
							e.printStackTrace();
						}
						objIn.close();
						this.workerList = workers;
						this.currentDay = workers.currentDay;

						Results suspendResponse;
						suspendResponse = createResults(true, true);
						suspendResponse.setResultPayload(new byte[1]);
						writeResults("Sync/ControllerSuspendResponse",
								suspendResponse);
					} else if (nbEvent.getContentSynopsis().equals(
							"Sync/Bootstrap")) {
						bootstrapProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().equals(
							"Sync/UpStream")) {
						upStreamProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().toString()
							.equals("Sync/Checkpoint")) {
						checkpointProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().equals(
							"Async/CheckpointRequest")) {
						checkpointRequest = true;

						Results workerInfo;
						workerInfo = createResults(true, true);

						ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
						ObjectOutput objOut = new ObjectOutputStream(byteOut);
						objOut.writeObject(workerList);
						objOut.close();

						byte[] workerListPayload = byteOut.toByteArray();
						workerInfo.setResultPayload(workerListPayload);

						writeResults("Sync/WorkerInfo", workerInfo);
					}
					if (finished)
						access.close();

				}
			}
		} catch (DatasetException e) {
			System.out.println("DatasetException");
			e.printStackTrace();
		} catch (ServiceException e) {
			System.out.println("ServiceException");
			e.printStackTrace();
		} catch (NBEventException e) {
			System.out.println("NBEventException");
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("IOException");
			e.printStackTrace();
		} catch (MapReduceException e) {
			System.out.println("MapReduceException");
			e.printStackTrace();
		} catch (MarshallingException e) {
			System.out.println("MarshallingException");
			e.printStackTrace();
		}

	}

	/**
	 * From a whole message, pick up the END_OF_DAY message
	 * 
	 * @param message
	 *            The whole message
	 * @return END_OF_DAY message
	 */
	private byte[] pickupEndOfDay(byte[] message) {
		int length = message.length;
		byte[] endOfDayArray = new byte[EventType.EventSize[EventType.END_OF_DAY]];
		for (int i = 0; i < EventType.EventSize[EventType.END_OF_DAY]; i++) {
			endOfDayArray[i] = message[length
					- EventType.EventSize[EventType.END_OF_DAY] + i];
		}
		return endOfDayArray;
	}

	private float[] pickupOutputVariableValue(byte[] message) {
		byte[] outputVariableValueArray = new byte[EventType.EventSize[EventType.OUTPUT_VARIABLE_VALUE]];
		float[] output = new float[OutputVariableValue.VAR_NUM + 1];
		int offset = message.length - EventType.EventSize[EventType.END_OF_DAY]
				- EventType.EventSize[EventType.OUTPUT_VARIABLE_VALUE];
		while (true) {
			try {
				// System.out.println("Message length: " + message.length);
				// System.out.println("Offset: " + offset);
				// System.out.print("Last message: ");
				for (int i = 0; i < EventType.EventSize[EventType.OUTPUT_VARIABLE_VALUE]; i++) {
					outputVariableValueArray[i] = message[offset + i];
					// System.out.print(outputVariableValueArray[i] + " ");
				}
				// System.out.println();
				EventType eventType = EventTypeFactory
						.createEventType(outputVariableValueArray);
				// System.out.println("Event type: " + eventType.getType());
				if (eventType.getType() != EventType.OUTPUT_VARIABLE_VALUE) {
					break;
				} else {
					OutputVariableValue outputVariableValue = (OutputVariableValue) eventType;
					if (outputVariableValue.getVarCode() == 1
							|| outputVariableValue.getVarCode() == 3
							|| (outputVariableValue.getVarCode() >= 6 && outputVariableValue
									.getVarCode() <= 14)) {
						// System.out.println(outputVariableValue.getWorkerID()
						// + " " + outputVariableValue.getVarCode()
						// + "   " + outputVariableValue.getIValue());
						output[outputVariableValue.getVarCode()] = outputVariableValue
								.getIValue();
					} else {
						// System.out.println(outputVariableValue.getWorkerID()
						// + " " + outputVariableValue.getVarCode()
						// + "   " + outputVariableValue.getFValue());
						output[outputVariableValue.getVarCode()] = outputVariableValue
								.getFValue();
					}
					offset = offset
							- EventType.EventSize[EventType.OUTPUT_VARIABLE_VALUE];
				}
			} catch (Exception e) {
				break;
			}
		}
		return output;

	}

	/**
	 * Combine two messages into one byte array
	 * 
	 * @param message1
	 *            First message
	 * @param message2
	 *            Second message
	 * @return Combined byte array
	 */
	public static byte[] combineByte(byte[] message1, byte[] message2) {
		byte[] result;
		if (message1.length == 0) {
			result = new byte[message2.length
					- EventType.EventSize[EventType.END_OF_DAY]];
			for (int i = 0; i < message2.length
					- EventType.EventSize[EventType.END_OF_DAY]; i++) {
				result[i] = message2[i];
			}
		} else {
			result = new byte[message1.length + message2.length
					- EventType.EventSize[EventType.END_OF_DAY]];
			for (int i = 0; i < message1.length; i++) {
				result[i] = message1[i];
			}
			for (int i = 0; i < message2.length
					- EventType.EventSize[EventType.END_OF_DAY]; i++) {
				result[i + message1.length] = message2[i];
			}
		}
		return result;
	}

	/**
	 * Process function when checkpoint message is arrived.
	 * 
	 * @param nbEvent
	 *            Received message
	 * @throws MapReduceException
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private synchronized void checkpointRequestProcess(NBEvent nbEvent)
			throws MapReduceException, IOException, MarshallingException {
		checkpointRequest = false;
		numOfFinishedSplitAndMergeWorkers = 0;
		byte[] bytes;
		splitMap.clear();
		mergeMap.clear();
		workerList.startAll();
		StartNewDayEvent startNewDay = new StartNewDayEvent(0, currentDay + 1);
		System.out.println();
		System.out.println("All done:" + " Day " + currentDay);
		bytes = startNewDay.marshalling();
		byte[] finalResult = new byte[buffer.length
				+ EventType.EventSize[EventType.START_NEW_DAY] + 1];
		for (int i = 0; i < buffer.length; i++) {
			finalResult[i] = buffer[i];
		}
		for (int i = 0; i < bytes.length; i++) {
			finalResult[buffer.length + i] = bytes[i];
		}
		finalResult[buffer.length
				+ EventType.EventSize[EventType.START_NEW_DAY]] = 0;
		buffer = new byte[0];
		System.out.println("FinalResult.length = " + finalResult.length);
		Results ckptMessage = createResults(false, false);

//		try {
//			System.out.println("~~~~~~~CHECKSUM CONTROLLER~~~~~~~"
//					+ MD5Checksum.getMD5Checksum(finalResult));
//		} catch (NoSuchAlgorithmException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
		ckptMessage.setResultPayload(finalResult);
		writeResults("Sync/DownStream", ckptMessage);
		time2 = System.currentTimeMillis();
		System.out.println("Day: " + currentDay + "  Length: "
				+ finalResult.length + "  Time: " + (int) (time2 - time1));

		File file = new File("execution_time.txt");
		FileWriter fw;
		if (!file.exists()) {
			file.createNewFile();
			fw = new FileWriter(file, true);
		} else
			fw = new FileWriter(file, true);
		fw.write(currentDay + "  " + finalResult.length + "  "
				+ (int) (time2 - time1) + "\n");
		fw.close();
		time1 = time2;
	}

	/**
	 * Process function when checkpoint message is arrived.
	 * 
	 * @param nbEvent
	 *            Received message
	 * @throws MapReduceException
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private synchronized void checkpointProcess(NBEvent nbEvent)
			throws MapReduceException, IOException, MarshallingException {
		Results resultsEx = ResultsFactory.getInstance().getResults(
				nbEvent.getContentPayload());
		byte[] reportBytes = resultsEx.getResultPayload();
		Checkpoint checkpoint = new Checkpoint();
		checkpoint = checkpoint.unmarshalling(reportBytes);
		// checkpoint = checkpoint.getBoundaries(reportBytes);
		// System.out.println("---------------------------------");
		// System.out.println(checkpoint.getWorkerID());
		// System.out.println(checkpoint.getType());
		// System.out.println(checkpoint.up);
		// System.out.println(checkpoint.down);
		// System.out.println(checkpoint.left);
		// System.out.println(checkpoint.right);
		// System.out.println("---------------------------------");

		Restore restore = new Restore();
		// restore.setWorkerID(num);
		// restore.getBoundaries(checkpoint.addBoundaries());

		int newWorker;

		System.out.println("#### Lists content ####");
		for (int i = 0; i < workerList.list.size(); i++) {
			System.out.print(workerList.list.get(i).getWorkerNumber() + " ");
		}
		System.out.println();
		for (int i = 0; i < workerList.activeList.size(); i++) {
			System.out.print(workerList.activeList.get(i).getWorkerNumber()
					+ " ");
		}
		System.out.println();
		System.out.println("#### ############# ####");
		// Integer splitWorker = workerList.activeList.indexOf(workerList.list
		// .get(checkpoint.getWorkerID()));
		// System.out.println("----- Split:" + checkpoint.getWorkerID());

		if (splitMap.containsKey(checkpoint.getWorkerID())) {
			restore.setOverwrite(true);
			newWorker = splitMap.get(checkpoint.getWorkerID());
		} else if (mergeMap.containsKey(new Integer(checkpoint.getWorkerID()))) {
			restore.setOverwrite(false);
			newWorker = mergeMap.get(checkpoint.getWorkerID());
		} else {
			System.out.println("STUCK");
			return;
		}
		// int newWorker = this.getNextSpare();
		System.out.println("Number of new worker: " + newWorker);
		restore.setWorkerID(newWorker);
		restore.setContent(checkpoint.getContent());
		// System.out.println("---------------------------------");
		// System.out.println(restore.getWorkerID());
		// System.out.println(restore.getType());
		// System.out.println(restore.up);
		// System.out.println(restore.down);
		// System.out.println(restore.left);
		// System.out.println(restore.right);
		// System.out.println("---------------------------------");
		exchangeResults = createResults(false, false);
		exchangeResults.setResultPayload(restore.marshalling());
		writeResults("Sync/DownStream/" + newWorker, exchangeResults);
		numOfFinishedSplitAndMergeWorkers++;
		/* After all the split and merge work is done, go into the next day */
		System.out.println(numOfFinishedSplitAndMergeWorkers + " ^^^ "
				+ numOfSplitWorkers + " ^^^ " + numOfMergeWorkers);
		if (numOfFinishedSplitAndMergeWorkers == numOfSplitWorkers
				+ numOfMergeWorkers) {
			// !!!!!!!!!!!!!!!!!!! THIS IS A PLACE FOR FT
			// CHKPT!!!!!!!!!!!!!!!!!!!!!!!
			// if (currentDay == 10)
			// checkpointRequest = true;
			if (checkpointRequest) {
				System.out.println("STH DIFF");
				Results checkpointRequestResult = createResults(false, false);
				RequestForFTCheckpoint request = new RequestForFTCheckpoint();
				request.setGoInactive(false);
				request.setType((byte) 7);
				checkpointRequestResult.setResultPayload(request.marshalling());
				writeResults("Sync/CheckpointRequest", checkpointRequestResult);
				checkpointRequest = false;
			}

			numOfFinishedSplitAndMergeWorkers = 0;
			byte[] bytes;
			splitMap.clear();
			mergeMap.clear();
			workerList.startAll();
			StartNewDayEvent startNewDay = new StartNewDayEvent(0,
					currentDay + 1);
			System.out.println();
			System.out.println("All done:" + " Day " + currentDay);
			bytes = startNewDay.marshalling();
			byte[] finalResult = new byte[buffer.length
					+ EventType.EventSize[EventType.START_NEW_DAY] + 1];
			for (int i = 0; i < buffer.length; i++) {
				finalResult[i] = buffer[i];
			}
			for (int i = 0; i < bytes.length; i++) {
				finalResult[buffer.length + i] = bytes[i];
			}
			finalResult[buffer.length
					+ EventType.EventSize[EventType.START_NEW_DAY]] = 0;
			buffer = new byte[0];
			System.out.println("FinalResult.length = " + finalResult.length);
			Results ckptMessage = createResults(false, false);

//			try {
//				System.out.println("~~~~~~~CHECKSUM CONTROLLER~~~~~~~"
//						+ MD5Checksum.getMD5Checksum(finalResult));
//			} catch (NoSuchAlgorithmException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			ckptMessage.setResultPayload(finalResult);
			writeResults("Sync/DownStream", ckptMessage);
			time2 = System.currentTimeMillis();
			System.out.println("Day: " + currentDay + "  Length: "
					+ finalResult.length + "  Time: " + (int) (time2 - time1));

			File file = new File("execution_time.txt");
			FileWriter fw;
			if (!file.exists()) {
				file.createNewFile();
				fw = new FileWriter(file, true);
			} else
				fw = new FileWriter(file, true);
			fw.write(currentDay + "  " + finalResult.length + "  "
					+ (int) (time2 - time1) + "\n");
			fw.close();
			time1 = time2;

		}
	}

	/**
	 * If synopsis of received message is UPSTREAM, the events will be proceeded
	 * in this function.
	 * 
	 * @param nbEvent
	 *            Received message containing all the information
	 * @throws MapReduceException
	 * @throws ServiceException
	 * @throws NBEventException
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private synchronized void upStreamProcess(NBEvent nbEvent)
			throws MapReduceException, ServiceException, NBEventException,
			IOException, MarshallingException {
		int numberOfWorkers = workerList.numberOfWorkers;
		Results resultsEx = ResultsFactory.getInstance().getResults(
				nbEvent.getContentPayload());
		byte[] reportBytes = resultsEx.getResultPayload();

		try {
			float[] outputVariables = pickupOutputVariableValue(reportBytes);
			byte[] temp = pickupEndOfDay(reportBytes);
			// for (int i = 0; i < temp.length; i++) {
			// System.out.print(temp[i] + " ");
			// }
			// System.out.println();
			EndOfDayEvent endOfDayEvent = (EndOfDayEvent) EventTypeFactory
					.createEventType(temp);

			int day = endOfDayEvent.getDayNumber();
			if (day > currentDay + 2 || day < currentDay) {
				System.out.println("Day mismatch!");
				return;
			}

			currentDay = day;
			workerList.currentDay = day;

			buffer = combineByte(buffer, reportBytes);

			// System.out.println("day: " + currentDay);

			int num = endOfDayEvent.getWorkerID();
			workerList.stop(num);
			if (recordLog) {
				File file = new File("MLData.txt");
				FileWriter fw;
				if (!file.exists()) {
					file.createNewFile();
					fw = new FileWriter(file, true);
				} else
					fw = new FileWriter(file, true);
				fw.write(num + " ");
				for (int i = 1; i < OutputVariableValue.VAR_NUM + 1; i++) {
					fw.write(outputVariables[i] + " ");
				}
				fw.write(workerList.executionTime(num) + "\n");
				fw.close();
			}
			if (endOfDayEvent.isFinished()) {
				workerList.finishWorker(endOfDayEvent.getWorkerID());
				/* Check END_OF_SIMULATION condition */
				if (workerList.isAllFinished()) {
					System.out.println("All finished:" + " Day "
							+ endOfDayEvent.getDayNumber());
					endTime = System.currentTimeMillis();
					System.out
							.println("Elapsed Time =" + (endTime - startTime));

					EndOfSimulationEvent endOfSimulationEvent = new EndOfSimulationEvent(
							num);
					byte[] bytes = endOfSimulationEvent.marshalling();
					byte[] finalResult = new byte[buffer.length
							+ EventType.EventSize[EventType.END_OF_SIMULATION]
							+ 1];

					for (int i = 0; i < buffer.length; i++) {
						finalResult[i] = buffer[i];
					}
					for (int i = 0; i < bytes.length; i++) {
						finalResult[buffer.length + i] = bytes[i];
					}
					finalResult[buffer.length
							+ EventType.EventSize[EventType.END_OF_SIMULATION]] = 1;
					exchangeResults = createResults(true, true);
					exchangeResults.setResultPayload(finalResult);
					try {
						System.out.println("~~~~~~~CHECKSUM CONTROLLER~~~~~~~"
								+ MD5Checksum.getMD5Checksum(finalResult));
					} catch (NoSuchAlgorithmException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					writeResults("Sync/DownStream", exchangeResults);

					Results results = createResults(true, true);
					String res = "END";
					results.setResultPayload(res.getBytes());
					writeResults("Sync/Result", results);
					// writer.write("" + endOfDayEvent.getDayNumber());
					// writer.newLine();
					// writer.write("" + finalResult.length);
					// writer.newLine();
					time2 = System.currentTimeMillis();
					// writer.write("" + (int) (time2 - time1));
					System.out.println("Day: " + endOfDayEvent.getDayNumber()
							+ "  Length: " + finalResult.length + "  Time: "
							+ (int) (time2 - time1));
					File file = new File("execution_time.txt");
					FileWriter fw;
					if (!file.exists()) {
						file.createNewFile();
						fw = new FileWriter(file, true);
					} else
						fw = new FileWriter(file, true);
					fw.write(endOfDayEvent.getDayNumber() + "  "
							+ finalResult.length + "  " + (int) (time2 - time1)
							+ "\n");
					fw.close();
					time1 = time2;
					// writer.newLine();
					// System.out.println("Writer Closed!");
					// writer.close();
					File file1 = new File(
							"/s/chopin/e/proj/granules/simonsui/Granules/trunk/bin/workers.txt");
					FileWriter fw1;
					if (!file1.exists()) {
						file1.createNewFile();
						fw1 = new FileWriter(file1, true);
					} else
						fw1 = new FileWriter(file1, true);
					System.out
							.println("************** Working Time *************");
					for (int i = 0; i < numberOfWorkers; i++) {
						System.out.print(workerList.executionTime(i) / 1000
								+ " ");
						fw1.write(workerList.executionTime(i) + " ");
					}
					fw1.write("\n");
					System.out.println();
					System.out
							.println("************** ************ *************");
					fw1.close();
					System.out
							.println("Invoked Termination Condition Reached on Controller.");
					return;
				}
			}

			int maxTimeWorker = 0;
			int secondTimeWorker = 0;
			Set<Integer> mergeSet = new HashSet<Integer>();
			if (workerList.isAllDone()) {
				long minTime = Long.MAX_VALUE;
				long maxTime = Long.MIN_VALUE;
				long secondMaxTime = Long.MIN_VALUE;
				for (int i = 0; i < numberOfWorkers; i++) {
					if (minTime > workerList.executionTime(i)) {
						minTime = workerList.executionTime(i);
					}
					if (maxTime < workerList.executionTime(i)) {
						secondMaxTime = maxTime;
						secondTimeWorker = maxTimeWorker;
						maxTime = workerList.executionTime(i);
						maxTimeWorker = i;
					} else if (secondMaxTime < workerList.executionTime(i)) {
						secondMaxTime = workerList.executionTime(i);
						secondTimeWorker = i;
					}
				}
				File file = new File("workers.txt");
				FileWriter fw;
				if (!file.exists()) {
					file.createNewFile();
					fw = new FileWriter(file, true);
				} else
					fw = new FileWriter(file, true);
				System.out.println("************** Working Time *************");
				for (int i = 0; i < numberOfWorkers; i++) {
					System.out.print(workerList.executionTime(i) / 1000 + " ");
					fw.write(workerList.executionTime(i) + " ");
				}
				fw.write("\n");
				System.out.println();
				System.out.println("************** ************ *************");
				fw.close();
				// See which two should be merged

				numOfMergeWorkers = 0;

				if (workerList.numberOfActiveWorkers >= 2 && maxTime > 5000) {
					for (int i = 0; i < workerList.numberOfActiveWorkers() - 1; i++) {
						// i now is the position in activeList!!! We have to
						// convert i
						int posInList = workerList.activeList.get(i)
								.getWorkerNumber();
						int pos2InList = workerList.activeList.get(i + 1)
								.getWorkerNumber();
						if (workerList.activeList.get(i).executionTime
								+ workerList.activeList.get(i + 1).executionTime < 0.75 * maxTime) {
							mergeMap.put(new Integer(pos2InList), new Integer(
									posInList));
							mergeSet.add(new Integer(pos2InList));
							mergeSet.add(new Integer(posInList));
							workerList.mergeWorkers(posInList, pos2InList);
							numOfMergeWorkers++;
							byte type = 7;
							RequestForFTCheckpoint request = new RequestForFTCheckpoint(
									pos2InList, type, true);

							exchangeResults = createResults(false, false);
							exchangeResults.setResultPayload(request
									.marshalling());
							writeResults("Sync/DownStream/" + pos2InList,
									exchangeResults);
							System.out.println("Merge:" + posInList + " and "
									+ pos2InList);
						}
					}
				}

				// See which one should be splitted
				numOfSplitWorkers = 0;
				if (workerList.numberOfSpareWorkers() > 0) {
					// int workerLeft = workerList.numberOfSpareWorkers();
					LinkedList<WorkerNode> sortedList = new LinkedList<WorkerNode>();
					sortedList.addAll(workerList.list);
					for (int i = 0; i < sortedList.size() - 1; i++)
						for (int j = i + 1; j < sortedList.size(); j++)
							if (sortedList.get(i).executionTime < sortedList
									.get(j).executionTime) {
								WorkerNode tempNode = sortedList.get(i);
								sortedList.set(i, sortedList.get(j));
								sortedList.set(j, tempNode);
							}
					int workerActive = workerList.numberOfActiveWorkers();
					// LinkedList<WorkerNode> activeList = new
					// LinkedList<WorkerNode>();
					// activeList.addAll(workerList.activeList);
					for (int i = 0; i < sortedList.size(); i++) {
						// i now is the position in activeList!!! We have to
						// convert i
						if (!sortedList.get(i).active)
							continue;
						int posInList = sortedList.get(i).getWorkerNumber();
						int posInActiveList = workerList.activeList
								.indexOf(sortedList.get(i));
						if (workerList.numberOfSpareWorkers() == 0) {
							// || workerList.activeList.get(i).justMerged) {
							// System.out.println("Number of spare workers = "
							// + workerList.numberOfSpareWorkers());
							continue;
						} else if ((workerActive == 1)
								|| (sortedList.get(i).executionTime >= 0.75 * maxTime && !mergeSet
										.contains(new Integer(posInList)))) {
							System.out.println("posInList = " + posInList
									+ ", i = " + i);
							SplitEvent splitEvent = new SplitEvent();
							splitEvent.worker_ID = posInList;
							splitEvent.howToSplit = 1;
							splitEvent.coordinate = workerList
									.splitWorker(posInList);
							System.out.println("SPLIT on "
									+ splitEvent.coordinate);
							workerActive++;
							numOfSplitWorkers++;
							System.out.println("SPLIT: "
									+ new Integer(posInList)
									+ " to "
									+ new Integer(workerList.activeList.get(
											posInActiveList + 1)
											.getWorkerNumber()));
							splitMap.put(
									new Integer(posInList),
									new Integer(workerList.activeList.get(
											posInActiveList + 1)
											.getWorkerNumber()));
							exchangeResults = createResults(false, false);
							exchangeResults.setResultPayload(splitEvent
									.marshalling());
							writeResults("Sync/DownStream/" + posInList,
									exchangeResults);
							System.out.println("SPLIT :" + posInList);
							System.out.println("LEFT WORKER = "
									+ workerList.numberOfSpareWorkers());
						}
					}
				}
				mergeSet.clear();

				/* If no worker needs to be split, just go into the next day */
				if (numOfSplitWorkers + numOfMergeWorkers == 0) {
					// if (currentDay == 10)
					// checkpointRequest = true;
					if (checkpointRequest) {
						System.out.println("STH DIFF");
						Results checkpointRequestResult = createResults(false,
								false);
						RequestForFTCheckpoint request = new RequestForFTCheckpoint();
						request.setGoInactive(false);
						request.setType((byte) 7);
						checkpointRequestResult.setResultPayload(request
								.marshalling());
						writeResults("Sync/CheckpointRequest",
								checkpointRequestResult);
						checkpointRequest = false;
					}

					numOfFinishedSplitAndMergeWorkers = 0;
					byte[] bytes;
					StartNewDayEvent startNewDay = new StartNewDayEvent(0,
							endOfDayEvent.getDayNumber() + 1);
					System.out.println();
					System.out.println("All done:" + " Day "
							+ endOfDayEvent.getDayNumber());
					workerList.startAll();
					bytes = startNewDay.marshalling();
					byte[] finalResult = new byte[buffer.length
							+ EventType.EventSize[EventType.START_NEW_DAY] + 1];
					for (int i = 0; i < buffer.length; i++) {
						finalResult[i] = buffer[i];
					}
					for (int i = 0; i < bytes.length; i++) {
						finalResult[buffer.length + i] = bytes[i];
					}
					finalResult[buffer.length
							+ EventType.EventSize[EventType.START_NEW_DAY]] = 0;
					buffer = new byte[0];
					System.out.println("FinalResult.length = "
							+ finalResult.length);
					exchangeResults = createResults(false, false);

					exchangeResults.setResultPayload(finalResult);
					try {
						System.out.println("~~~~~~~CHECKSUM CONTROLLER~~~~~~~"
								+ MD5Checksum.getMD5Checksum(finalResult));
					} catch (NoSuchAlgorithmException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					writeResults("Sync/DownStream", exchangeResults);
					time2 = System.currentTimeMillis();
					System.out.println("Day: " + endOfDayEvent.getDayNumber()
							+ "  Length: " + finalResult.length + "  Time: "
							+ (int) (time2 - time1));

					File file1 = new File("execution_time.txt");
					FileWriter fw1;
					if (file1.exists()) {
						file1.createNewFile();
						fw1 = new FileWriter(file1, true);
					} else
						fw1 = new FileWriter(file1, true);
					fw1.write(endOfDayEvent.getDayNumber() + "  "
							+ finalResult.length + "  " + (int) (time2 - time1)
							+ "\n");
					fw1.close();
					time1 = time2;
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Once receive a BOOTSTRAP message, start the first day.
	 * 
	 * @param nbEvent
	 *            Received message containing all the information.
	 * @throws MapReduceException
	 * @throws ServiceException
	 * @throws NBEventException
	 */
	private void bootstrapProcess(NBEvent nbEvent) throws MapReduceException,
			ServiceException, NBEventException {
		try {
			hostName = "";
			try {
				InetAddress address = InetAddress.getLocalHost();
				hostName = address.getHostName();
			} catch (Exception e) {
			}
			workerList = new WorkerList(Integer.parseInt(this
					.getProcessingDirectives().get("Number of Workers")
					.toString()));

			// numberOfWorkers = 1;
			// numberOfSpareWorkers =
			// Integer.parseInt(this.getProcessingDirectives()
			// .get("Number of Workers").toString()) - 1;
			// executionTime = new long[numberOfSpareWorkers + numberOfWorkers];
			// finishedWorker = new LinkedList<Boolean>();
			// doneWorker = new LinkedList<Boolean>();
			// for (int i = 0; i < numberOfWorkers + numberOfSpareWorkers; i++)
			// {
			// finishedWorker.add(new Boolean(false));
			// doneWorker.add(new Boolean(false));
			// }
			StartNewDayEvent startEvent = new StartNewDayEvent(0, 1);
			byte[] startMessage = new byte[EventType.EventSize[EventType.START_NEW_DAY] + 1];
			byte[] start = startEvent.marshalling();
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				startMessage[i] = start[i];
			}
			startMessage[EventType.EventSize[EventType.START_NEW_DAY]] = 0;
			exchangeResults.setResultPayload(startMessage);
			writeResults("Sync/DownStream", exchangeResults);
			workerList.activateWorker(0, 0);
			float leftB = Float.parseFloat(this.getProcessingDirectives()
					.getProperty("Left"));
			float rightB = Float.parseFloat(this.getProcessingDirectives()
					.getProperty("Right"));
			float upperB = Float.parseFloat(this.getProcessingDirectives()
					.getProperty("Upper"));
			float lowerB = Float.parseFloat(this.getProcessingDirectives()
					.getProperty("Down"));
			workerList.list.get(0).setBoundaries(upperB, lowerB, leftB, rightB);
			workerList.startAll();
			time1 = System.currentTimeMillis();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
