/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicmerge;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaGenericComputation;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.dynamicmerge.event.Checkpoint;
import ds.granules.naadsm.dynamicmerge.event.EventType;
import ds.granules.naadsm.dynamicmerge.event.EventTypeFactory;
import ds.granules.naadsm.dynamicmerge.event.RequestForFTCheckpoint;
import ds.granules.naadsm.dynamicmerge.event.Restore;
import ds.granules.naadsm.dynamicmerge.event.StartNewDayEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;
import ds.granules.util.MD5Checksum;

/**
 * This is the worker for distributed NAADSM. It is responsible for communicate
 * the NAADSM project with controllers.
 * 
 * @author Zhiquan (Simon) Sui
 * 
 */

public class Worker extends MapReduceBase implements JavaMessageHandler {
	JavaGenericComputation jc;
	// private float leftB;
	// private float rightB;
	// private float upperB;
	// private float lowerB;
	private boolean active;
	private boolean finished;
	private byte[] buffer;
	FileWriter writer;
	BufferedWriter out;
	boolean checkpointProcess;
	boolean started;
	String sysCommand;
	private Results exchangeResults;
	private boolean silent;
	File output;

	/**
	 * Constructor. Initialize the variables and the byte array.
	 */
	public Worker() {
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		started = false;
		finished = false;
		silent = true;
		active = false;
		jc = new JavaGenericComputation("Worker");
		buffer = new byte[0];
		exchangeResults = createResults(false, false);
		checkpointProcess = false;
	}

	@Override
	public void execute() throws ProcessingException {
		// long time1 = System.currentTimeMillis();
		try {
			int num = Integer.parseInt(this.getProcessingDirectives()
					.getProperty("Worker Number"));
			if (!started) {
				// The default boundaries are the original ones.
				// leftB = Float.parseFloat(this.getProcessingDirectives()
				// .getProperty("Left"));
				// rightB = Float.parseFloat(this.getProcessingDirectives()
				// .getProperty("Right"));
				// upperB = Float.parseFloat(this.getProcessingDirectives()
				// .getProperty("Upper"));
				// lowerB = Float.parseFloat(this.getProcessingDirectives()
				// .getProperty("Down"));
				startSpare();
				started = true;
				if (num == 0)
					this.active = true;
			}
			StreamingAccess access = null;
			String datasetIdentifier = "WORKER_DATA"
					+ this.getProcessingDirectives().getProperty(
							"Worker Number");
			DatasetCollection datasetCollection = getDatasetCollection();
			if (datasetCollection == null)
				return;
			try {
				Dataset dataset = datasetCollection
						.getDataset(datasetIdentifier);
				if (dataset == null) {
					System.err.println("DATASET IS NULL!!!!!!!!!!!!!!!");
					return;
				}
				if (dataset.getDatasetType() == Dataset.STREAMS) {
					access = (StreamingAccess) dataset;
				} else {
					System.err.println("Incorrect dataset:"
							+ dataset.getDatasetType());
					return;
				}
				if (access != null) {
					while (dataset.isDataAvailable()) {
						NBEvent nbEvent = access.getStreamingData();
						if (nbEvent == null) {
							continue;
						}
						if (nbEvent.getContentSynopsis().equals(
								"Sync/DownStream")) {
							// System.out.println("Downstream");
							downStreamProcess(nbEvent);
						} else if (nbEvent.getContentSynopsis().equals(
								"Sync/CheckpointRequest")) {
							System.out.println("Worker " + num
									+ " got checkpoint request!");
							if (!active)
								return;
							Results resultsEx = ResultsFactory.getInstance()
									.getResults(nbEvent.getContentPayload());
							byte[] content = resultsEx.getResultPayload();
							RequestForFTCheckpoint requestForFTCheckpoint = (RequestForFTCheckpoint) EventTypeFactory
									.createEventType(content);
							requestForFTCheckpoint.setWorkerID(num);
							if (buffer == null)
								buffer = requestForFTCheckpoint.marshalling();
							else if (buffer.length == 0)
								buffer = requestForFTCheckpoint.marshalling();
							else
								buffer = Controller.combineByte(buffer,
										requestForFTCheckpoint.marshalling());
							checkpointProcess = true;
						} else if (nbEvent.getContentSynopsis().equals(
								"Sync/DownStream/" + num)) {
							System.out.println("Downstream only for me");
							byte[] resultsPayload = nbEvent.getContentPayload();
							Results results = ResultsFactory.getInstance()
									.getResults(resultsPayload);
							byte[] bb = results.getResultPayload();
							EventType eventType = EventTypeFactory
									.createEventType(bb);
							if (eventType.getType() == EventType.RESTORE) {
								// Restore restore = (Restore) eventType;
								// if (restore.getOverwrite())
								System.out.println("restoring");
								activation(nbEvent);
							} else {
								if (!active)
									return;
								if (eventType.getType() == EventType.REQUEST_FOR_FT_CHECKPOINT) {
									RequestForFTCheckpoint requestForFTCheckpoint = (RequestForFTCheckpoint) eventType;
									if (!requestForFTCheckpoint.active()) {
										this.active = false;
										System.out.println("I am #" + num);
										System.out.println("I am now inactive");
									}
								}
								// Re-calculate the boundaries when split
								// float coordinator = (leftB + rightB) / 2;
								// SplitEvent splitEvent = (SplitEvent)
								// eventType;
								// splitEvent.coordinate = coordinator;
								JavaByteMessage jbm = new JavaByteMessage(
										ControlMessage.DATA);
								byte[] bytes = eventType.marshalling();
								jbm.setInput(bytes);
								byte[] totalMessage = jbm.toByteArray();
								if (!silent) {
									System.out
											.println("----------------ALL Send to NAADSM:-------------------");
									for (int i = 0; i < totalMessage.length; i++)
										System.out.print(Integer
												.toHexString(totalMessage[i])
												+ " ");
									System.out.println();
									System.out
											.println("---------------------------------------------------");
								}
								try {
									jbm.setDescription(MD5Checksum
											.getMD5Checksum(bytes));
								} catch (NoSuchAlgorithmException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
								jc.sendMessage(jbm);
								System.out
										.println("Sent to NAADSM checkpoint message");
							}

						}
					}
				} else {
					System.err.println("The dataset is null!");
				}
			} catch (DatasetException e) {
				System.out.println("DatasetException");
				e.printStackTrace();
			} catch (ServiceException e) {
				System.out.println("ServiceException");
				e.printStackTrace();
			} catch (NBEventException e) {
				System.out.println("NBEventException");
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("IOException");
				System.out.println(e.getMessage());
				e.printStackTrace();
			} catch (MapReduceException e) {
				System.out.println("MapReduceException");
				e.printStackTrace();
			} catch (MarshallingException e) {
				System.out.println("MarshallingException");
				e.printStackTrace();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Activate a spare worker into active worker
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private synchronized void activation(NBEvent nbEvent) throws IOException,
			MarshallingException {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		this.active = true;
		Restore restore = new Restore();
		restore = restore.unmarshalling(ResultsFactory.getInstance()
				.getResults(nbEvent.getContentPayload()).getResultPayload());
		// restore = restore.getBoundaries(ResultsFactory.getInstance()
		// .getResults(nbEvent.getContentPayload()).getResultPayload());
		// Get new boundary of the current worker
		// this.leftB = restore.left;
		// this.rightB = restore.right;
		// this.lowerB = restore.down;
		// this.upperB = restore.up;
		System.out.println("restore worker id=" + restore.getWorkerID());
		JavaByteMessage jbm = null;
		byte[] bytes = null;
		try {
			jbm = new JavaByteMessage(ControlMessage.DATA);
			bytes = restore.marshalling();
			// System.out.println("Restore message marshalling successfully.");
			jbm.setInput(bytes);
			// System.out.println("Input set successfully.");
			if (!silent) {
				System.out
						.println("----------------Send to NAADSM:-------------------");
				for (int i = 0; i < bytes.length; i++)
					System.out.print(Integer.toHexString(bytes[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			if (!silent) {
				byte[] totalMessage = jbm.toByteArray();
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			// System.out.println("Prepare to print.");
			System.out.println(restore.getType());
			System.out.println(bytes.length);
			System.out.println("************ Worker " + num + " is active!");
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			jbm.setDescription(MD5Checksum.getMD5Checksum(bytes));
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		jc.sendMessage(jbm);
	}

	/**
	 * Start NAADSM as a spare worker
	 */
	private synchronized void startSpare() {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// Default command line
		sysCommand = "/s/chopin/e/proj/granules/simonsui/dis-naadsm/naadsm-3.2.18__NN/bin/naadsm -V 0 --rng-seed 3 -o ./tmp/naadsm_output_.log -h /s/chopin/e/proj/granules/simonsui/dis-naadsm/660000_population_scaled_from_kansas_9_infected.xml /s/chopin/e/proj/granules/simonsui/dis-naadsm/Kansas_params_SlowerDetection_HigherInfectionRate.xml";
		if (this.getProcessingDirectives().getProperty("cmdLine") != "")
			sysCommand = this.getProcessingDirectives().getProperty("cmdLine");
		try {
			jc.startExecution(sysCommand, ControlMessage.UPIPES, this);
			System.out.println("Worker " + num + " is spared!");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Process messages with synopsis of DownStream
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws ServiceException
	 * @throws NBEventException
	 * @throws IOException
	 * @throws MapReduceException
	 * @throws MarshallingException
	 * @throws NullPointerException
	 * @throws NoSuchAlgorithmException
	 */
	private synchronized void downStreamProcess(NBEvent nbEvent) throws ServiceException,
			NBEventException, IOException, MapReduceException,
			MarshallingException, NullPointerException,
			NoSuchAlgorithmException {
		byte[] extraSend = null;
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		byte[] eventPayload = nbEvent.getContentPayload();
		if (eventPayload == null) {
			System.out.println("Payload NULL");
			return;
		}
		Results resultsEx = ResultsFactory.getInstance().getResults(
				eventPayload);
		if (resultsEx == null) {
			return;
		}

		byte[] send = resultsEx.getResultPayload();
		if (send == null) {
			return;
		}
		// System.out.println();
		// System.out
		// .println("---------------------------------------------------");
		// if (!silent) {
		File temp = new File("receive" + num + ".txt");
		if (temp.exists()) {
			temp.delete();
			temp.createNewFile();
		}
		FileWriter fw = new FileWriter(temp);
		fw.write(("----------------Send to NAADSM:-------------------"));
		for (int i = 0; i < send.length; i++)
			fw.write(Integer.toHexString(send[i]) + " ");
		fw.write("\n");
		fw.write("---------------------------------------------------");
		fw.close();

		if (buffer != null) {
			extraSend = new byte[send.length + buffer.length];
			for (int i = 0; i < buffer.length; i++) {
				extraSend[i] = buffer[i];
			}
			for (int i = 0; i < send.length; i++) {
				extraSend[i + buffer.length] = send[i];
			}
			buffer = new byte[0];
		} else {
			extraSend = send;
		}
		if (!active) {
			System.out.println("worker " + num + " is inactive!!!!");
		}
		// The last digit is 0 when START_NEW_DAY and 1 when END_SIMULATION
		if (extraSend[extraSend.length - 1] == 0 && active) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] start = new byte[EventType.EventSize[EventType.START_NEW_DAY]];
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				start[i] = extraSend[extraSend.length - 1
						- EventType.EventSize[EventType.START_NEW_DAY] + i];
			}
			StartNewDayEvent startEvent = (StartNewDayEvent) EventTypeFactory
					.createEventType(start);
			// System.out.print("======== day =======" +
			// startEvent.getDayNumber()
			// + "  ");

			startEvent.setWorker(num); // Modify the worker number property in
										// START_NEW_DAY message
										// System.out.println("======== worker ======="
			// + startEvent.getWorkerID());
			// try {
			// System.out.print("**********Checksum before copy*********"
			// + MD5Checksum.getMD5Checksum(extraSend));
			// } catch (NoSuchAlgorithmException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }
			start = startEvent.marshalling();
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			// System.out.println("**********Checksum after copy*********"
			// + MD5Checksum.getMD5Checksum(bytes));
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				bytes[bytes.length
						- EventType.EventSize[EventType.START_NEW_DAY] + i] = start[i];
			}
			jbm.setInput(bytes);
			// System.out
			// .println("----------------Send to NAADSM:-------------------");
			// for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY];
			// i++)
			// System.out.print(Integer.toHexString(bytes[bytes.length
			// - EventType.EventSize[EventType.START_NEW_DAY] + i])
			// + " ");
			// System.out.println();
			// System.out
			// .println("---------------------------------------------------");
			// if (!silent) {
			File temp1 = new File("temp" + num + ".txt");
			if (temp1.exists()) {
				temp1.delete();
				temp1.createNewFile();
			}
			FileWriter fw1 = new FileWriter(temp1);
			fw1.write(("----------------Send to NAADSM:-------------------"));
			for (int i = 0; i < bytes.length; i++)
				fw1.write(Integer.toHexString(bytes[i]) + " ");
			fw1.write("\n");
			fw1.write("---------------------------------------------------");
			fw1.close();
			// }
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			try {
				jbm.setDescription(MD5Checksum.getMD5Checksum(bytes));
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			jc.sendMessage(jbm);
			// System.out.println("&&&&&&&" + num + "&&&&&&&");

		} else if (extraSend[extraSend.length - 1] == 1) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			jbm.setInput(bytes);
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			try {
				jbm.setDescription(MD5Checksum.getMD5Checksum(bytes));
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			jc.sendMessage(jbm);
			System.out.println("&&&&&&&" + num + "&&&&&&&");

			Results suspendResponse;
			suspendResponse = createResults(true, true);
			suspendResponse.setResultPayload(new byte[1]);
			writeResults("Sync/WorkerSuspendResponse", suspendResponse);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ds.granules.bridge.JavaMessageHandler#handleMessage(byte[]) Once got
	 * message from NAADSM project, this function will be activated.
	 */
	@Override
	public synchronized void handleMessage(final byte[] msg) {
		// System.out.println("Got From NAADSM");
		final byte[] par = msg;
		if (!silent) {
			try {
				System.out
						.println("----------------Got From NAADSM:-------------------");
				for (int i = 0; i < par.length; i++)
					System.out.print(Integer.toHexString(par[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		JavaByteMessage jbm = new JavaByteMessage(par);

		byte[] bytes = jbm.getInput();
		try {
			if (!jbm.getDescription().equals(MD5Checksum.getMD5Checksum(bytes))) {
				System.out
						.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Error in byte Array!");
			}
		} catch (NoSuchAlgorithmException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		boolean isCheckpoint;
		try {
			if (EventTypeFactory.createEventType(bytes).getType() == EventType.CHECKPOINT) {
				isCheckpoint = true;
			} else
				isCheckpoint = false;
		} catch (Exception e) {
			isCheckpoint = false;
		}
		try {

			if (isCheckpoint) {
				if (!checkpointProcess) {
					Checkpoint checkpoint = (Checkpoint) EventTypeFactory
							.createEventType(bytes);
					// checkpoint.left = (leftB + rightB) / 2;
					// checkpoint.right = rightB;
					// checkpoint.up = upperB;
					// checkpoint.down = lowerB;
					System.out.println("Checkpoint message bytes number: "
							+ checkpoint.marshalling().length);
					// this.rightB = (this.leftB + this.rightB) / 2;
					// byte[] res = checkpoint.addBoundaries();
					byte[] res = checkpoint.marshalling();
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(res);
					System.out.println("Worker sends out bytes number: "
							+ res.length);
					writeResults("Sync/Checkpoint", exchangeResults);
				} else {
					Checkpoint checkpoint = (Checkpoint) EventTypeFactory
							.createEventType(bytes);
					byte[] checkpointBytes = checkpoint.marshalling();
					byte[] remainingBytes = new byte[bytes.length
							- checkpointBytes.length];
					for (int i = 0; i < remainingBytes.length; i++) {
						remainingBytes[i] = bytes[i + checkpointBytes.length];
					}
					Results exchangeResultsToController = createResults(false,
							false);
					exchangeResultsToController
							.setResultPayload(remainingBytes);
					writeResults("Sync/UpStream", exchangeResultsToController);
					Results exchangeResultsToSpeculator = createResults(false,
							false);
					exchangeResultsToSpeculator
							.setResultPayload(checkpointBytes);
					writeResults("Async/Checkpoint",
							exchangeResultsToSpeculator);
                    checkpointProcess = false;
				}
			} else {
				int num = Integer.parseInt(this.getProcessingDirectives()
						.getProperty("Worker Number"));
				System.out.println(num + " has done.");
				exchangeResults = createResults(false, false);
				exchangeResults.setResultPayload(bytes);
				writeResults("Sync/UpStream", exchangeResults);
			}
		} catch (MapReduceException e) {
			e.printStackTrace();
		}
	}

}
