/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicmerge;

import java.io.Serializable;

import java.util.LinkedList;

/**
 * This class maintains a list of workers, their attributes and corresponding
 * functions.
 * 
 * @author simonsui
 * 
 */
public class WorkerList implements Serializable {
	public LinkedList<WorkerNode> list;
	public LinkedList<WorkerNode> activeList;
	int numberOfWorkers;
	int numberOfActiveWorkers;
	public int currentDay;

	public WorkerList(int num) {
		numberOfWorkers = num;
		numberOfActiveWorkers = 0;
		list = new LinkedList<WorkerNode>();
		activeList = new LinkedList<WorkerNode>();
		for (int i = 0; i < num; i++) {
			list.add(new WorkerNode(i));
		}
	}

	public WorkerList() {
		list = new LinkedList<WorkerNode>();
		activeList = new LinkedList<WorkerNode>();
		numberOfWorkers = 0;
		numberOfActiveWorkers = 0;
	}
	/**
	 * Get the list that contains all the split workers.
	 * @param num
	 * @return
	 */
	public int getSplittedWorker(int num) {
		return list.indexOf(list.get(activeList.indexOf(list.get(num)) + 1));
	}
	/**
	 * Get the execution time of all the workers.
	 * @param num
	 * @return
	 */
	public long executionTime(int num) {
		return list.get(num).executionTime;
	}
	
	/**
	 * Get total number of workers.
	 * @return
	 */
	public int numberOfWorkers() {
		return numberOfWorkers;
	}
	/**
	 * Get total number of active workers.
	 * @return
	 */
	public int numberOfActiveWorkers() {
		return numberOfActiveWorkers;
	}

	/**
	 * Get number of spare workers.
	 * @return
	 */
	public int numberOfSpareWorkers() {
		return numberOfWorkers - numberOfActiveWorkers;
	}

	/**
	 * Finish one worker with worker number.
	 * @param num
	 */
	public void finishWorker(int num) {
		list.get(num).finish();
	}

	/**
	 * Start new day for all the workers.
	 */
	public void startAll() {
		for (int i = 0; i < numberOfWorkers; i++) {
			list.get(i).justMerged = false;
			list.get(i).start();
		}
	}

	/**
	 * Stop one worker with worker number.
	 * @param num
	 */
	public void stop(int num) {
		list.get(num).stop();
	}

	/**
	 * Activate one worker.
	 * @param num
	 * @param pos
	 * @return
	 */
	public boolean activateWorker(int num, int pos) {
		boolean success = list.get(num).activate();
		if (success) {
			activeList.add(pos, list.get(num));
			numberOfActiveWorkers++;
		}
		return success;
	}

	/**
	 * Set one worker as inactive.
	 * @param num
	 * @return
	 */
	public boolean deactivateWorker(int num) {
		boolean success = list.get(num).deactivate();
		if (success) {
			success = activeList.remove(list.get(num));
		}
		if (success)
			numberOfActiveWorkers--;
		return success;
	}

	/**
	 * Get the worker number of the next spare worker.
	 * @return
	 */
	public int nextSpareWorker() {
		for (int i = 0; i < list.size(); i++) {
			if (!list.get(i).isActive())
				return i;
		}
		return -1;
	}

	/**
	 * Split one worker based on equal area .
	 * @param num
	 * @return boundary where the split is
	 */
	public float splitWorker(int num) {
		WorkerNode worker = list.get(num);
		if (!worker.isActive())
			return Float.NaN;
		else {
			System.out.println(worker.leftBound + " " + worker.rightBound + " "
					+ worker.upperBound + " " + worker.lowerBound);
			WorkerNode newWorker = list.get(nextSpareWorker());
			newWorker.leftBound = (worker.leftBound + worker.rightBound)
					* (float) 0.5;
			newWorker.rightBound = worker.rightBound;
			newWorker.upperBound = worker.upperBound;
			newWorker.lowerBound = worker.lowerBound;
			System.out.println(newWorker.leftBound + " " + newWorker.rightBound
					+ " " + newWorker.upperBound + " " + newWorker.lowerBound);
			activateWorker(nextSpareWorker(),
					activeList.indexOf(list.get(num)) + 1);
			worker.rightBound = (worker.leftBound + worker.rightBound)
					* (float) 0.5;
			return newWorker.leftBound;
		}
	}

	/**
	 * Merge two adjacent workers together.
	 * @param worker1
	 * @param worker2
	 * @return
	 */
	public boolean mergeWorkers(int worker1, int worker2) {
		WorkerNode w1 = list.get(worker1);
		WorkerNode w2 = list.get(worker2);
		if (Math.abs((activeList.indexOf(w1) - activeList.indexOf(w2))) != 1) {
			// Not adjacent nodes
			return false;
		} else {
			w1.justMerged = true;
			w1.leftBound = Math.min(w1.leftBound, w2.leftBound);
			w1.rightBound = Math.max(w1.rightBound, w2.rightBound);
			deactivateWorker(worker2);
		}
		return true;
	}

	/**
	 * Check whether all the workers have finished the whole simulation.
	 * @return
	 */
	public boolean isAllFinished() {
		boolean flag = true;
		for (int i = 0; i < list.size(); i++) {
			if ((!list.get(i).isFinished()) && (list.get(i).isActive())) {
				flag = false;
				break;
			}
		}
		return flag;
	}

	/**
	 * Check whether all the workers have finished the simulation of current day.
	 * @return
	 */
	public boolean isAllDone() {
		boolean flag = true;
		for (int i = 0; i < numberOfWorkers; i++) {
			if ((!list.get(i).isDone()) && (list.get(i).isActive())) {
				flag = false;
				break;
			}
		}
		return flag;
	}
}
