/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicmerge.event;

import java.io.Serializable;

/**
 * Base class for all event classes
 * 
 * @author simonsui
 * 
 */
public class EventType implements Serializable {
	public static int START_NEW_DAY = 0;
	public static int END_OF_DAY = 1;
	public static int END_OF_SIMULATION = 2;
	public static int EXPOSURE = 3;
	public static int PUBLIC_ANNOUNCEMENT = 4;
	public static int QUARANTINE = 5;
	public static int DESTRUCTION = 6;
	public static int SPLIT = 7;
	public static int TRACE_RESULT = 8;
	public static int REQUEST_OF_DESTRUCTION = 9;
	public static int ATTEMPT_TO_INFECT = 10;
	public static int REQUEST_FOR_ZONE_FOCUS = 11;
	public static int CHECKPOINT = 12;
	public static int RESTORE = 13;
	public static int REQUEST_FOR_FT_CHECKPOINT = 14;
	public static int OUTPUT_VARIABLE_VALUE = 15;
	public static int REQUEST_FOR_VACCINATION = 16;
	public static int VACCINATION_CANCELED = 17;
	public static int VACCINATION = 18;
	public static int DETECTION = 20;
	public static int[] EventSize = { 12, 13, 9, 26, 12, 16, 16, 16, 28, 24,
			40, 16, 12, 13, 10, 21, 29, 20, 16, 16, 24 };
	int type;

	public int getType() {
		return type;
	}

	void getNextBytes(int size) {

	}

	public byte[] marshalling() {
		switch (type) {
		case 0:
			StartNewDayEvent startNewDayEvent = (StartNewDayEvent) this;
			return startNewDayEvent.marshalling();
		case 1:
			EndOfDayEvent endOfDayEvent = (EndOfDayEvent) this;
			return endOfDayEvent.marshalling();
		case 2:
			EndOfSimulationEvent endOfSimulationEvent = (EndOfSimulationEvent) this;
			return endOfSimulationEvent.marshalling();
		case 3:
			ExposureEvent exposureEvent = (ExposureEvent) this;
			return exposureEvent.marshalling();
		case 4:
			PublicAnnouncementEvent publicAnnoucementEvent = (PublicAnnouncementEvent) this;
			return publicAnnoucementEvent.marshalling();
		case 5:
			QuarantineEvent quarantineEvent = (QuarantineEvent) this;
			return quarantineEvent.marshalling();
		case 6:
			DestructionEvent destructionEvent = (DestructionEvent) this;
			return destructionEvent.marshalling();
		case 7:
			SplitEvent splitEvent = (SplitEvent) this;
			return splitEvent.marshalling();
		case 8:
			TraceResult traceResult = (TraceResult) this;
			return traceResult.marshalling();
		case 9:
			RequestOfDestruction requestOfDestruction = (RequestOfDestruction) this;
			return requestOfDestruction.marshalling();
		case 10:
			AttemptToInfect attemptToInfect = (AttemptToInfect) this;
			return attemptToInfect.marshalling();
		case 11:
			RequestForZoneFocus requestForZoneFocus = (RequestForZoneFocus) this;
			return requestForZoneFocus.marshalling();
		case 12:
			Checkpoint checkpoint = (Checkpoint) this;
			return checkpoint.marshalling();
		case 13:
			Restore restore = (Restore) this;
			return restore.marshalling();
		case 14:
			RequestForFTCheckpoint requestForFTCheckpoint = (RequestForFTCheckpoint) this;
			return requestForFTCheckpoint.marshalling();
		case 15:
			OutputVariableValue outputVariableValue = (OutputVariableValue) this;
			return outputVariableValue.marshalling();
		case 16:
			RequestForVaccination requestForVaccination = (RequestForVaccination) this;
			return requestForVaccination.marshalling();
		case 17:
			VaccinationCanceled vaccinationCanceled = (VaccinationCanceled) this;
			return vaccinationCanceled.marshalling();
		case 18:
			Vaccination vaccination = (Vaccination) this;
			return vaccination.marshalling();
		case 20:
			Detection detection = (Detection) this;
			return detection.marshalling();
		}
		return null;
	}

	public EventType unmarshalling(byte[] bytes) {
		EventType eventType = null;
		byte[] temp = new byte[4];
		temp[0] = bytes[0];
		temp[1] = bytes[1];
		temp[2] = bytes[2];
		temp[3] = bytes[3];
		int type = EventTypeFactory.bytes2int(temp);
		switch (type) {
		case 0:
			StartNewDayEvent startNewDayEvent = new StartNewDayEvent();
			eventType = startNewDayEvent.unmarshalling(bytes);
			break;
		case 1:
			EndOfDayEvent endOfDayEvent = new EndOfDayEvent();
			eventType = endOfDayEvent.unmarshalling(bytes);
			break;
		case 2:
			EndOfSimulationEvent endOfSimulationEvent = new EndOfSimulationEvent();
			eventType = endOfSimulationEvent.unmarshalling(bytes);
			break;
		case 3:
			ExposureEvent exposureEvent = new ExposureEvent();
			eventType = exposureEvent.unmarshalling(bytes);
			break;
		case 4:
			PublicAnnouncementEvent publicAnnoucementEvent = new PublicAnnouncementEvent();
			eventType = publicAnnoucementEvent.unmarshalling(bytes);
			break;
		case 5:
			QuarantineEvent quarantineEvent = new QuarantineEvent();
			eventType = quarantineEvent.unmarshalling(bytes);
			break;
		case 6:
			DestructionEvent destructionEvent = new DestructionEvent();
			eventType = destructionEvent.unmarshalling(bytes);
			break;
		case 7:
			SplitEvent splitEvent = new SplitEvent();
			eventType = splitEvent.unmarshalling(bytes);
			break;
		case 8:
			TraceResult traceResult = new TraceResult();
			eventType = traceResult.unmarshalling(bytes);
			break;
		case 9:
			RequestOfDestruction requestOfDestruction = new RequestOfDestruction();
			eventType = requestOfDestruction.unmarshalling(bytes);
			break;
		case 10:
			AttemptToInfect attemptToInfect = new AttemptToInfect();
			eventType = attemptToInfect.unmarshalling(bytes);
			break;
		case 11:
			RequestForZoneFocus requestForZoneFocus = new RequestForZoneFocus();
			eventType = requestForZoneFocus.unmarshalling(bytes);
			break;
		case 12:
			Checkpoint checkpoint = new Checkpoint();
			eventType = checkpoint.unmarshalling(bytes);
			break;
		case 13:
			Restore restore = new Restore();
			eventType = restore.unmarshalling(bytes);
			break;
		case 14:
			RequestForFTCheckpoint requestForFTCheckpoint = new RequestForFTCheckpoint();
			eventType = requestForFTCheckpoint.unmarshalling(bytes);
			break;
		case 15:
			OutputVariableValue outputVariableValue = new OutputVariableValue();
			eventType = outputVariableValue.unmarshalling(bytes);
			break;
		case 16:
			RequestForVaccination requestForVaccination = new RequestForVaccination();
			eventType = requestForVaccination.unmarshalling(bytes);
			break;
		case 17:
			VaccinationCanceled vaccinationCanceled = new VaccinationCanceled();
			eventType = vaccinationCanceled.unmarshalling(bytes);
			break;
		case 18:
			Vaccination vaccination = new Vaccination();
			eventType = vaccination.unmarshalling(bytes);
			break;
		case 20:
			Detection detection = new Detection();
			eventType = detection.unmarshalling(bytes);
			break;
		}
		return eventType;
	}

	public void getNextValue(int size) {

	}

}
