/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicmerge.event;

/**
 * ATTEMPT_TO_INFECT message
 * 
 * @author simonsui
 * 
 */
public class OutputVariableValue extends EventType {
	int index;
	int worker_ID;
	int varCode;
	byte dataType;
	private static byte DT_INTEGER = 0;
	private static byte DT_FLOAT = 0;
	int iValue;
	float fValue;
	byte[] bytes;
	byte[] temp;
	int length;
	char[] name;
	public static int VAR_NUM = 12;
	private static int OTHER_VAR_CODE = 0;

	public OutputVariableValue() {
		type = EventType.OUTPUT_VARIABLE_VALUE;
		index = 0;
		bytes = new byte[EventType.EventSize[type]];
		temp = new byte[4];
	}

	public int getVarCode() {
		return varCode;
	}

	public int getWorkerID() {
		return worker_ID;
	}

	public int getIValue() {
		return iValue;
	}

	public float getFValue() {
		return fValue;
	}

	@Override
	public byte[] marshalling() {
		bytes = new byte[EventType.EventSize[type]];
		temp = new byte[4];
		index = 0;
		temp = EventTypeFactory.int2bytes(type);
		getNextBytes(4);
		temp = EventTypeFactory.int2bytes(worker_ID);
		getNextBytes(4);
		temp = EventTypeFactory.int2bytes(varCode);
		getNextBytes(4);
		if (varCode == OTHER_VAR_CODE) {
			temp = EventTypeFactory.int2bytes(length);
			getNextBytes(4);
			for (int i = 0; i < length; i++) {
				temp[0] = (byte) name[i];
				getNextBytes(1);
			}
		}
		temp[0] = dataType;
		getNextBytes(1);
		if (dataType == DT_INTEGER) {
			temp = EventTypeFactory.int2bytes(iValue);
			getNextBytes(4);
		} else if (dataType == DT_FLOAT) {
			temp = EventTypeFactory.float2bytes(fValue);
			getNextBytes(4);
		}
		return bytes;
	}

	@Override
	public OutputVariableValue unmarshalling(byte[] bytes) {
		this.bytes = bytes;
		index = 4;
		getNextValue(4);
		worker_ID = EventTypeFactory.bytes2int(temp);
		getNextValue(4);
		varCode = EventTypeFactory.bytes2int(temp);
		if (varCode == OTHER_VAR_CODE) {
			getNextValue(4);
			length = EventTypeFactory.bytes2int(temp);
			name = new char[length];
			for (int i = 0; i < length; i++) {
				getNextValue(1);
				name[i] = (char) temp[0];
			}
		}
		getNextValue(1);
		dataType = temp[0];
		getNextValue(4);
		if (dataType == DT_INTEGER) {
			iValue = EventTypeFactory.bytes2int(temp);
		} else if (dataType == DT_FLOAT) {
			fValue = EventTypeFactory.bytes2float(temp);
		}
		return this;
	}

	@Override
	public void getNextBytes(int size) {
		for (int i = 0; i < size; i++) {
			bytes[i + index] = temp[i];
		}
		index += size;
	}

	@Override
	public void getNextValue(int size) {
		for (int i = 0; i < size; i++) {
			temp[i] = bytes[i + index];
		}
		index += size;
	}

	@Override
	public int getType() {
		return 15;
	}
}
