/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicsplit;

import java.io.IOException;
import java.util.Properties;
import java.util.Random;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import cgl.narada.matching.Profile;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.EventConsumer;
import cgl.narada.service.client.EventProducer;
import cgl.narada.service.client.NBEventListener;
import cgl.narada.service.client.SessionService;
import ds.granules.exception.MarshallingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;
import ds.granules.naadsm.dynamicsplit.event.EventType;
import ds.granules.naadsm.dynamicsplit.event.EventTypeFactory;
import ds.granules.naadsm.dynamicsplit.event.ExposureEvent;
/**
 * Load generator is to start the experiment by sending out the BOOTSTRAP
 * message and finalize the experiment.
 * 
 * @author Zhiquan (Simon) Sui
 * 
 */
public class LoadGenerator implements NBEventListener {

	private int entityId;

	private ClientService clientService;

	private String moduleName = "SimpleClient: ";

	private EventProducer producer;

	private Profile profile;

	private EventConsumer consumer;

	Properties props;

	String transport = "niotcp";

	String topic = "Sync/Result";

	int[] exposure;

	long startTime;
	long endTime;

	public void initializeConsumer() throws ServiceException {
		profile = clientService.createProfile(
				TemplateProfileAndSynopsisTypes.STRING, topic);
		consumer = clientService.createEventConsumer(this);
		consumer.subscribeTo(profile);
	}

	public LoadGenerator(Properties property) throws ServiceException {
		startTime = System.currentTimeMillis();
		entityId = new Random().nextInt();
		clientService = SessionService.getClientService(entityId);
		props = (Properties) property.clone();
		exposure = new int[2];
	}

	public void initializeBrokerCommunications(Properties props, String commType)
			throws ServiceException {
		clientService.initializeBrokerCommunications(props, commType);
	}

	/**
	 * Closes the connection to the broker.
	 * 
	 * @throws ServiceException
	 */
	public void closeBrokerConnection() throws ServiceException {
		clientService.closeBrokerConnection();
		clientService.terminateServices();
		try {
			/** Make sure that the transmission tracker is shutdown gracefully. */
			Thread.sleep(100);
		} catch (InterruptedException e) {
			System.out
					.println(moduleName
							+ "Problems sleeping while waiting for transfer completion");
		}
	}

	public void initializeProducer() throws ServiceException {
		producer = clientService.createEventProducer();

		producer.generateEventIdentifier(true);
		producer.setTemplateId(12345);
		producer.setDisableTimestamp(false);

		producer.setSuppressRedistributionToSource(true);
	}

	public void publishEvent(String publishTopic, byte[] msg)
			throws ServiceException, NBEventException {

		NBEvent nbEvent = producer.generateEvent(
				TemplateProfileAndSynopsisTypes.STRING, publishTopic, msg);
		producer.publishEvent(nbEvent);
	}

	public void onEvent(NBEvent nbEvent) {

		ResultsFactory resultsFactory = ResultsFactory.getInstance();
		try {
			Results results = resultsFactory.getResults(nbEvent
					.getContentPayload());
			if (results.getResultPayload().length == 3) {
				for (int i = 0; i < exposure.length; i++) {
					System.out.println("Worker " + i + " had " + exposure[i]
							+ " exposures");
				}
				closeBrokerConnection();
				endTime = System.currentTimeMillis();
				System.out.println("Elapsed Time = " + (endTime - startTime));
				System.exit(0);
			} else {
				EventType event = EventTypeFactory.createEventType(results
						.getResultPayload());
				if (event.getType() == EventType.EXPOSURE) {
					ExposureEvent exposureEvent = (ExposureEvent) event;
					exposure[exposureEvent.getWorkerID()]++;
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (MarshallingException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}

	}

	public void createEvent(String topic, byte[] message)
			throws ServiceException, NBEventException {
		/** Initializing communications using specified transport */
		this.initializeBrokerCommunications(props, transport);
		this.initializeProducer();
		this.publishEvent(topic, message);
	}

	/**
	 * 
	 * @param args
	 *            args[0] is the host name of narada brokering. args[1] is the
	 *            port number of narada brokering.
	 */
	public static void main(String[] args) {
		String hostname = args[0];
		String portnum = args[1];
		String transport = "niotcp";
		Properties props = new Properties();
		props.put("hostname", hostname);
		props.put("portnum", portnum);
		try {
			LoadGenerator client = new LoadGenerator(props);
			client.initializeBrokerCommunications(props, transport);
			client.initializeConsumer();
			byte[] nothing = new byte[1];
			client.createEvent("Sync/Bootstrap", nothing);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
}
