/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicsplit;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaGenericComputation;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.dynamicsplit.event.Checkpoint;
import ds.granules.naadsm.dynamicsplit.event.EventType;
import ds.granules.naadsm.dynamicsplit.event.EventTypeFactory;
import ds.granules.naadsm.dynamicsplit.event.Restore;
import ds.granules.naadsm.dynamicsplit.event.SplitEvent;
import ds.granules.naadsm.dynamicsplit.event.StartNewDayEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

/**
 * This is the worker for distributed NAADSM. It is responsible for communicate
 * the NAADSM project with controllers.
 * 
 * @author Zhiquan (Simon) Sui
 * 
 */

public class Worker extends MapReduceBase implements JavaMessageHandler {
	JavaGenericComputation jc;
	private float leftB;
	private float rightB;
	private float upperB;
	private float lowerB;
	private boolean active;
	private boolean finished;
	private byte[] buffer;
	FileWriter writer;
	BufferedWriter out;

	boolean started;
	String sysCommand;
	private Results exchangeResults;
	private boolean silent;
	File output;

	/**
	 * Constructor. Initialize the variables and the byte array.
	 */
	public Worker() {
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		started = false;
		finished = false;
		silent = true;
		active = false;
		jc = new JavaGenericComputation("Worker");
		buffer = new byte[0];
		exchangeResults = createResults(false, false);
	}

	@Override
	public void execute() throws ProcessingException {
		long time1 = System.currentTimeMillis();
		try {
			if (!started) {
				// The default boundaries are the original ones.
				leftB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Left"));
				rightB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Right"));
				upperB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Upper"));
				lowerB = Float.parseFloat(this.getProcessingDirectives()
						.getProperty("Down"));
				startSpare();
				started = true;
			}
			StreamingAccess access = null;
			String datasetIdentifier = "WORKER_DATA"
					+ this.getProcessingDirectives().getProperty(
							"Worker Number");
			DatasetCollection datasetCollection = getDatasetCollection();
			if (datasetCollection == null)
				return;
			try {
				Dataset dataset = datasetCollection
						.getDataset(datasetIdentifier);
				if (dataset == null) {
					System.err.println("DATASET IS NULL!!!!!!!!!!!!!!!");
					return;
				}
				if (dataset.getDatasetType() == Dataset.STREAMS) {
					access = (StreamingAccess) dataset;
				} else {
					System.err.println("Incorrect dataset:"
							+ dataset.getDatasetType());
					return;
				}
				if (access != null) {
					while (dataset.isDataAvailable()) {
						NBEvent nbEvent = access.getStreamingData();
						if (nbEvent == null) {
							continue;
						}
						int num = Integer.parseInt(this
								.getProcessingDirectives().getProperty(
										"Worker Number"));
						if (num == 0)
							this.active = true;
						if (active
								&& nbEvent.getContentSynopsis().equals(
										"Sync/DownStream")) {
							downStreamProcess(nbEvent);
							if (finished)
								;
						} else if (nbEvent.getContentSynopsis().equals(
								"Sync/DownStream/" + num)) {
							byte[] resultsPayload = nbEvent.getContentPayload();
							Results results = ResultsFactory.getInstance()
									.getResults(resultsPayload);
							byte[] bb = results.getResultPayload();
							EventType eventType = EventTypeFactory
									.createEventType(bb);
							if (eventType.getType() == EventType.RESTORE) {
								activation(nbEvent);
							} else if (eventType.getType() == EventType.SPLIT) {
								// Re-calculate the boundaries when split
								float coordinator = (leftB + rightB) / 2;
								SplitEvent splitEvent = (SplitEvent) eventType;
								splitEvent.coordinate = coordinator;
								JavaByteMessage jbm = new JavaByteMessage(
										ControlMessage.DATA);
								jbm.setInput(splitEvent.marshalling());
								byte[] totalMessage = jbm.toByteArray();
								if (!silent) {
									System.out
											.println("----------------ALL Send to NAADSM:-------------------");
									for (int i = 0; i < totalMessage.length; i++)
										System.out.print(Integer
												.toHexString(totalMessage[i])
												+ " ");
									System.out.println();
									System.out
											.println("---------------------------------------------------");
								}

								jc.sendMessage(jbm);
							}

						}
					}
				} else {
					System.err.println("The dataset is null!");
				}
			} catch (DatasetException e) {
				System.out.println("DatasetException");
				e.printStackTrace();
			} catch (ServiceException e) {
				System.out.println("ServiceException");
				e.printStackTrace();
			} catch (NBEventException e) {
				System.out.println("NBEventException");
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("IOException");
				e.printStackTrace();
			} catch (MapReduceException e) {
				System.out.println("MapReduceException");
				e.printStackTrace();
			} catch (MarshallingException e) {
				System.out.println("MarshallingException");
				e.printStackTrace();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Activate a spare worker into active worker
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private void activation(NBEvent nbEvent) throws IOException,
			MarshallingException {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		this.active = true;
		Restore restore = new Restore();
		restore = restore.getBoundaries(ResultsFactory.getInstance()
				.getResults(nbEvent.getContentPayload()).getResultPayload());
		// Get new boundary of the current worker
		this.leftB = restore.left;
		this.rightB = restore.right;
		this.lowerB = restore.down;
		this.upperB = restore.up;
		JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
		byte[] bytes = restore.marshalling();
		jbm.setInput(bytes);
		if (!silent) {
			System.out
					.println("----------------Send to NAADSM:-------------------");
			for (int i = 0; i < bytes.length; i++)
				System.out.print(Integer.toHexString(bytes[i]) + " ");
			System.out.println();
			System.out
					.println("---------------------------------------------------");
		}
		byte[] totalMessage = jbm.toByteArray();
		if (!silent) {
			System.out
					.println("----------------ALL Send to NAADSM:-------------------");
			for (int i = 0; i < totalMessage.length; i++)
				System.out.print(Integer.toHexString(totalMessage[i]) + " ");
			System.out.println();
			System.out
					.println("---------------------------------------------------");
		}

		System.out.println("************ Worker " + num + " is active!");
		jc.sendMessage(jbm);
	}

	/**
	 * Start NAADSM as a spare worker
	 */
	private void startSpare() {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// Default command line
		sysCommand = "/s/chopin/e/proj/granules/simonsui/dis-naadsm/naadsm-3.2.10__DynamicSplitFinal//bin/naadsm -V 0 --rng-seed 3 -o naadsm_output_.log -h /s/chopin/e/proj/granules/simonsui/dis-naadsm/660000_population_scaled_from_kansas_9_infected.xml /s/chopin/e/proj/granules/simonsui/dis-naadsm/Kansas_params_SlowerDetection_HigherInfectionRate.xml";
		if (this.getProcessingDirectives().getProperty("cmdLine") != "")
			sysCommand = this.getProcessingDirectives().getProperty("cmdLine");
		try {
			jc.startExecution(sysCommand, ControlMessage.UPIPES, this);
			System.out.println("Worker " + num + " is spared!");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Process messages with synopsis of DownStream
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws ServiceException
	 * @throws NBEventException
	 * @throws IOException
	 * @throws MapReduceException
	 * @throws MarshallingException
	 * @throws NullPointerException
	 */
	private void downStreamProcess(NBEvent nbEvent) throws ServiceException,
			NBEventException, IOException, MapReduceException,
			MarshallingException, NullPointerException {
		byte[] extraSend = null;
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		byte[] eventPayload = nbEvent.getContentPayload();
		if (eventPayload == null) {
			System.out.println("Payload NULL");
			return;
		}
		Results resultsEx = ResultsFactory.getInstance().getResults(
				eventPayload);
		if (resultsEx == null) {
			return;
		}

		byte[] send = resultsEx.getResultPayload();
		if (send == null) {
			return;
		}

		if (buffer != null) {
			extraSend = new byte[send.length + buffer.length];
			for (int i = 0; i < buffer.length; i++) {
				extraSend[i] = buffer[i];
			}
			for (int i = 0; i < send.length; i++) {
				extraSend[i + buffer.length] = send[i];
			}
			buffer = null;
		} else {
			extraSend = send;
		}
		// The last digit is 0 when START_NEW_DAY and 1 when END_SIMULATION
		if (extraSend[extraSend.length - 1] == 0) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] start = new byte[EventType.EventSize[EventType.START_NEW_DAY]];
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				start[i] = extraSend[extraSend.length - 1
						- EventType.EventSize[EventType.START_NEW_DAY] + i];
			}
			StartNewDayEvent startEvent = (StartNewDayEvent) EventTypeFactory
					.createEventType(start);
			startEvent.setWorker(num); // Modify the worker number property in
										// START_NEW_DAY message
			start = startEvent.marshalling();
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				bytes[bytes.length
						- EventType.EventSize[EventType.START_NEW_DAY] + i] = start[i];
			}
			jbm.setInput(bytes);
			if (!silent) {
				System.out
						.println("----------------Send to NAADSM:-------------------");
				for (int i = 0; i < bytes.length; i++)
					System.out.print(Integer.toHexString(bytes[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			jc.sendMessage(jbm);

		} else if (extraSend[extraSend.length - 1] == 1) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			byte[] bytes = new byte[extraSend.length - 1];
			for (int i = 0; i < extraSend.length - 1; i++) {
				bytes[i] = extraSend[i];
			}
			jbm.setInput(bytes);
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			jc.sendMessage(jbm);
		}

	}

	/*
	 * (non-Javadoc)
	 * @see ds.granules.bridge.JavaMessageHandler#handleMessage(byte[])
	 * Once got message from NAADSM project, this function will be activated.
	 */
	@Override
	public void handleMessage(final byte[] msg) {
		final byte[] par = msg;
		if (!silent) {
			try {
				System.out
						.println("----------------Got From NAADSM:-------------------");
				for (int i = 0; i < par.length; i++)
					System.out.print(Integer.toHexString(par[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		synchronized (exchangeResults) {
			JavaByteMessage jbm = new JavaByteMessage(par);

			byte[] bytes = jbm.getInput();

			boolean isCheckpoint;
			try {
				if (EventTypeFactory.createEventType(bytes).getType() == EventType.CHECKPOINT) {
					isCheckpoint = true;
				} else
					isCheckpoint = false;
			} catch (Exception e) {
				isCheckpoint = false;
			}
			try {

				if (isCheckpoint) {
					Checkpoint checkpoint = (Checkpoint) EventTypeFactory
							.createEventType(bytes);
					checkpoint.left = (leftB + rightB) / 2;
					checkpoint.right = rightB;
					checkpoint.up = upperB;
					checkpoint.down = lowerB;
					System.out.println("Checkpoint message bytes number: "
							+ checkpoint.marshalling().length);
					this.rightB = (this.leftB + this.rightB) / 2;
					byte[] res = checkpoint.addBoundaries();
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(res);
					System.out.println("Worker sends out bytes number: "
							+ res.length);
					writeResults("Sync/Checkpoint", exchangeResults);
				} else {
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(bytes);
					writeResults("Sync/UpStream", exchangeResults);
				}
			} catch (MapReduceException e) {
				e.printStackTrace();
			}
		}
	}
}
