/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.dynamicsplit.event;

/**
 * Factory to create all the events. Some static methods are included.
 * 
 * @author simonsui
 * 
 */
public class EventTypeFactory {

	public static EventType createEventType(byte[] bytes) {
		EventType eventType = null;
		try {
			byte[] temp = new byte[4];
			temp[0] = bytes[0];
			temp[1] = bytes[1];
			temp[2] = bytes[2];
			temp[3] = bytes[3];
			int type = bytes2int(temp);
			switch (type) {
			case 0:
				StartNewDayEvent startNewDayEvent = new StartNewDayEvent();
				eventType = startNewDayEvent.unmarshalling(bytes);
				break;
			case 1:
				EndOfDayEvent endOfDayEvent = new EndOfDayEvent();
				eventType = endOfDayEvent.unmarshalling(bytes);
				break;
			case 2:
				EndOfSimulationEvent endOfSimulationEvent = new EndOfSimulationEvent();
				eventType = endOfSimulationEvent.unmarshalling(bytes);
				break;
			case 3:
				ExposureEvent exposureEvent = new ExposureEvent();
				eventType = exposureEvent.unmarshalling(bytes);
				break;
			case 4:
				PublicAnnouncementEvent publicAnnoucementEvent = new PublicAnnouncementEvent();
				eventType = publicAnnoucementEvent.unmarshalling(bytes);
				break;
			case 5:
				QuarantineEvent quarantineEvent = new QuarantineEvent();
				eventType = quarantineEvent.unmarshalling(bytes);
				break;
			case 6:
				DestructionEvent destructionEvent = new DestructionEvent();
				eventType = destructionEvent.unmarshalling(bytes);
				break;
			case 7:
				SplitEvent splitEvent = new SplitEvent();
				eventType = splitEvent.unmarshalling(bytes);
				break;
			case 8:
				TraceResult traceResult = new TraceResult();
				eventType = traceResult.unmarshalling(bytes);
				break;
			case 9:
				RequestOfDestruction requestOfDestruction = new RequestOfDestruction();
				eventType = requestOfDestruction.unmarshalling(bytes);
				break;
			case 10:
				AttemptToInfect attemptToInfect = new AttemptToInfect();
				eventType = attemptToInfect.unmarshalling(bytes);
				break;
			case 11:
				RequestForZoneFocus requestForZoneFocus = new RequestForZoneFocus();
				eventType = requestForZoneFocus.unmarshalling(bytes);
				break;
			case 12:
				Checkpoint checkpoint = new Checkpoint();
				eventType = checkpoint.unmarshalling(bytes);
				break;
			case 13:
				Restore restore = new Restore();
				eventType = restore.unmarshalling(bytes);
				break;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return eventType;
	}

	public static int bytes2int(byte[] b) {
		return getInt(b);
	}

	public static byte[] int2bytes(int num) {
		return getBytes(num);
	}

	/**
	 * Returns the bytes that are associated with an integer value.
	 * 
	 * @param value
	 *            The integer for which we are seeking an equivalent byte stream
	 *            representation.
	 * @return The int value as a sequence of bytes
	 */
	public static byte[] getBytes(int value) {
		byte[] temp = new byte[4];
		temp[0] = (byte) ((value >>> 24) & 0xFF);
		temp[1] = (byte) ((value >>> 16) & 0xFF);
		temp[2] = (byte) ((value >>> 8) & 0xFF);
		temp[3] = (byte) ((value >>> 0) & 0xFF);
		return temp;
	}

	/**
	 * Given a sequence of bytes, this method returns the int value associated
	 * with the sequence of bytes. This sequence of bytes passed as the argument
	 * should be of length=4.
	 * 
	 * @param data
	 *            The sequence of bytes representing a int value.
	 * @return The integer value associated with the sequence of bytes
	 */
	public static int getInt(byte[] data) {
		int w, x, y, z;
		w = data[0];
		x = data[1];
		y = data[2];
		z = data[3];
		if (w < 0)
			w += 256;
		if (x < 0)
			x += 256;
		if (y < 0)
			y += 256;
		if (z < 0)
			z += 256;
		return ((w << 24) + (x << 16) + (y << 8) + (z << 0));
	}
	
	/**
	 * Float number to 4 digit of byte array
	 * @param num
	 *         Float number
	 * @return
	 *         4 digit byte array
	 */
	public static byte[] float2bytes(float num) {
		int IEEE754 = Float.floatToIntBits(num);
		return EventTypeFactory.int2bytes(IEEE754);
	}

	/**
	 * 4 digit of byte array to float number
	 * @param bytes
	 *         Byte array containing 4 digit
	 * @return
	 *         Float number that the byte array converted into
	 */
	public static float bytes2float(byte[] bytes) {
		int IEEE754 = EventTypeFactory.bytes2int(bytes);
		return Float.intBitsToFloat(IEEE754);
	}
}
