/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.ft;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import ds.granules.naadsm.dynamicmerge.WorkerList;

import ds.granules.naadsm.dynamicmerge.event.Checkpoint;

/**
 * State information from every Worker in a distributed NAADSM execution, plus
 * state from the Controller makes up a system Snapshot that can be used to
 * resume execution or roll back to a previous state.
 */
public class Snapshot {

    private int day;
    private long creationTime;
    private WorkerList workerList = null;
    private byte[] globalState = null;
    private List<Checkpoint> checkpoints = new ArrayList<Checkpoint>();

    public Snapshot(int day) {
        this.day = day;
        creationTime = System.nanoTime();
    }

    public void setWorkerList(WorkerList list) {
        workerList = list;
    }

    public void setGlobalState(byte[] globalState) {
        this.globalState = globalState;
    }

    public void addCheckpoint(Checkpoint checkpoint) {
        checkpoints.add(checkpoint);
    }

    /**
     * A Snapshot is only complete when it has information from every active
     * Worker.
     */
    public boolean isComplete() {
        /* If we have global state information and a checkpoint for all the
         * Workers, then this snapshot is complete. */

        if (workerList == null || globalState == null) {
            return false;
        } else {
            return checkpoints.size() == workerList.activeList.size();
        }
    }

    /**
     * Resets a snapshot, clearing the state information it contains.
     */
    public void clear() {
        workerList = null;
        globalState = null;
        checkpoints.clear();
        checkpoints = null;
    }

    public int getDay() {
        return day;
    }

    public Date getCreationTime() {
        return new Date(creationTime);
    }

    public WorkerList getWorkerList() {
        return workerList;
    }

    public byte[] getGlobalState() {
        return globalState;
    }

    public Checkpoint[] getCheckpoints() {
        Checkpoint[] allCheckpoints = new Checkpoint[checkpoints.size()];
        checkpoints.toArray(allCheckpoints);
        return allCheckpoints;
    }
}
