/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.ml;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.encog.ml.data.MLData;
import org.encog.ml.data.MLDataPair;
import org.encog.ml.data.MLDataSet;
import org.encog.ml.data.basic.BasicMLData;
import org.encog.ml.data.basic.BasicMLDataSet;
import org.encog.ml.train.MLTrain;
import org.encog.neural.networks.BasicNetwork;
import org.encog.neural.networks.layers.BasicLayer;
import org.encog.neural.networks.training.propagation.resilient.ResilientPropagation;
import org.encog.util.obj.SerializeObject;

import ds.granules.naadsm.dynamicmerge.event.OutputVariableValue;

/**
 * This class contains training, predicting and serialization of artificial
 * neural network.
 * 
 * @author simonsui
 * 
 */
public class NeuralPredictor implements Predictor {

	private NeuralState state = new NeuralState();

	public static void main(String[] args) throws Exception {
		NeuralPredictor np = new NeuralPredictor();
		// np.train("/s/chopin/k/grad/simonsui/TrainingData.txt");
		// np.serialize("output");
		np.deserialize("/s/chopin/e/proj/granules/simonsui/Granules/trunk/bin/Simple.ANN");
		// double[] d= {
		// 0.011785751070881121,0.3446713096369503,0.0013404825737265416,0.034687262695042534,0.35,0.0,0.0,0.017142857142857144,0.0012716998788285965,0.0,0.0,0.0
		// };
		double[] d = { 660204.0, 4920767.5, 1.0, 24.30853271484375,
				1.3333333730697632, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21382.0 };
		double[] s = { 5126.0, 16783.078, 59.0, 35.674862, 4.4210525, 0.0, 0.0,
				5.0, 72877.0, 0.0, 0.0, 0.0, 10560.0 };
		System.out.println(np.predict(new BasicMLData(s)));

	}

	/**
	 * Serialize the current network into file.
	 */
	public void serialize(String fileName) throws IOException {
		SerializeObject.save(new File(fileName), state);
	}

	/**
	 * Deserialize one file into a network.
	 */
	public void deserialize(String fileName) throws IOException,
			ClassNotFoundException {
		state = (NeuralState) SerializeObject.load(new File(fileName));
	}

	/**
	 * Predict one data with the current network.
	 */
	public double predict(MLData data) {
		/* Normalize the inputs. */
		for (int i = 0; i < data.size(); ++i) {
			double sample = data.getData(i);
			sample = (sample - state.inputMin[i])
					/ (state.inputMax[i] - state.inputMin[i]);
			data.setData(i, sample);
		}
		MLData output = state.network.compute(data);
		return output.getData(0) * (state.targetMax - state.targetMin)
				+ state.targetMin;
	}

	/**
	 * Use input file to train the neural network.
	 */
	public void train(String fileName) throws IOException {
		File source = new File(fileName);
		int numSamples = 0;
		try {
			BufferedReader fr = new BufferedReader(new FileReader(source));
			String line;
			do {
				line = fr.readLine();
				numSamples++;
			} while (line != "" && line != null);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		numSamples--;
		int numFeatures = OutputVariableValue.VAR_NUM + 1;
		double x[][] = new double[numSamples][numFeatures];
		double y[][] = new double[numSamples][1];
		double maxX[] = new double[numFeatures];
		double minX[] = new double[numFeatures];
		for (int i = 0; i < numFeatures; i++) {
			maxX[i] = Double.MIN_VALUE;
			minX[i] = Double.MAX_VALUE;
		}
		double maxY = Double.MIN_VALUE;
		double minY = Double.MAX_VALUE;
		try {
			BufferedReader fr = new BufferedReader(new FileReader(source));
			String line;
			int num = 0;
			do {
				line = fr.readLine();
				if (line == null)
					break;
				String[] words = line.split(" ");
				for (int i = 0; i < numFeatures; i++) {
					x[num][i] = Double.parseDouble(words[i]);
					if (maxX[i] < x[num][i])
						maxX[i] = x[num][i];
					if (minX[i] > x[num][i])
						minX[i] = x[num][i];
				}
				y[num][0] = Double.parseDouble(words[numFeatures]);
				if (maxY < y[num][0])
					maxY = y[num][0];
				if (minY > y[num][0])
					minY = y[num][0];
				num++;
			} while (line != "" && line != null);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		double[][] normalX = new double[numSamples][numFeatures];
		double[][] normalY = new double[numSamples][1];

		for (int i = 0; i < numSamples; i++) {
			for (int j = 0; j < numFeatures; j++) {
				normalX[i][j] = (x[i][j] - minX[j]) / (maxX[j] - minX[j]);
			}
			normalY[i][0] = (y[i][0] - minY) / (maxY - minY);
			if (normalY[i][0] < 0)
				System.out.println(i);
			// System.out.println("Y=" + y[i][0]);
		}
		BasicNetwork network = new BasicNetwork();
		state.network = network;

		network.addLayer(new BasicLayer(numFeatures));
		// network.addLayer(new BasicLayer(15));
		// network.addLayer(new BasicLayer(10));
		network.addLayer(new BasicLayer(30));
		network.addLayer(new BasicLayer(1));
		network.getStructure().finalizeStructure();
		network.reset();

		double[][] trainX = new double[numSamples / 2 + 1][numFeatures];
		double[][] trainY = new double[numSamples / 2 + 1][1];
		double[][] testX = new double[numSamples / 2 + 1][numFeatures];
		double[][] testY = new double[numSamples / 2 + 1][1];
		for (int i = 0; i < numSamples; i += 2) {
			for (int j = 0; j < numFeatures; j++) {
				trainX[i / 2][j] = normalX[i][j];
				if (i + 1 < numSamples)
					testX[i / 2][j] = normalX[i + 1][j];
			}
			trainY[i / 2][0] = normalY[i][0];
			if (i + 1 < numSamples)
				testY[i / 2][0] = normalY[i + 1][0];
		}
		MLDataSet trainingSet = new BasicMLDataSet(trainX, trainY);
		MLDataSet testSet = new BasicMLDataSet(testX, testY);
		MLTrain train = new ResilientPropagation(network, trainingSet);

		double error = 100;
		do {
			if (train.getIteration() % 10 == 0) {
				if (Math.abs(train.getError() - error) < 1E-6)
					break;
				error = train.getError();
			}
			train.iteration();
			System.out.println("Iteration:" + train.getIteration() + " Error:"
					+ train.getError());
		} while (true);
		train.finishTraining();
		for (MLDataPair pair : trainingSet) {
			MLData output = network.compute(pair.getInput());
			// for (int i = 0; i < pair.getInput().getData().length; i++)
			// System.out.print(pair.getInput().getData(i) + " ");
			// System.out.println();
			System.out.print(output.getData(0) * (maxY - minY) + minY + "  ");
			System.out.print(pair.getIdeal().getData(0) * (maxY - minY) + minY
					+ "  ");
			System.out
					.println(pair.getInputArray()[pair.getInputArray().length - 1]
							* (maxX[pair.getInputArray().length - 1] - minX[pair
									.getInputArray().length - 1])
							+ minX[pair.getInputArray().length - 1]);
		}

		state.targetMax = maxY;
		state.targetMin = minY;
		state.inputMax = maxX;
		state.inputMin = minX;

		// System.out.println("--------------------------------");
		// for (MLDataPair pair : trainingSet) {
		// MLData output = network.compute(pair.getInput());
		// System.out.print(output.getData(0) + ":");
		// System.out.println(pair.getIdeal().getData(0));
		// }

	}
}
