/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.portionaldynamicmerge;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * This class contains a list of {@link LBDayNode}. We maintain this for re-run
 * the same load balancing operations in different simulations.
 * 
 * @author simonsui
 * 
 */
public class DayList {
	LinkedList<LBDayNode> dayList;
	int currentDay;

	/**
	 * Get the {@link LBDayNode} of current day.
	 * 
	 * @return
	 */
	public LBDayNode getCurrentDay() {
		if (currentDay >= dayList.size())
			return null;
		LBDayNode node = dayList.get(currentDay);
		currentDay++;
		return node;
	}

	public void resetCurrentDay() {
		currentDay = 0;
	}

	public DayList() {
		dayList = new LinkedList<LBDayNode>();
		currentDay = 0;
	}

	public void addNextDay(LBDayNode node) {
		dayList.add(node);
	}

	/**
	 * From the list of strings, create list of {@link LBDayNode}
	 * @param list
	 * @return
	 */
	private LinkedList<LBOperationNode> putListIntoNodes(String[] list) {
		LinkedList<LBOperationNode> nodeList = new LinkedList<LBOperationNode>();
		if (list.length <= 1)
			return nodeList;
		for (int i = 0; i < list.length; i += 2) {
			LBOperationNode operationNode = new LBOperationNode(
					Integer.parseInt(list[i]), Integer.parseInt(list[i + 1]));
			nodeList.add(operationNode);
		}
		return nodeList;
	}

	/**
	 * Create the list from a file.
	 * @param Filename
	 */
	public void readListFromFile(String Filename) {
		dayList = new LinkedList<LBDayNode>();
		File mapFile = new File(Filename);
		try {
			BufferedReader reader = new BufferedReader(new FileReader(mapFile));
			String line;
			while ((line = reader.readLine()) != null) {
				String parts[] = line.split("\\|");
				LBDayNode node = new LBDayNode();
				node.setDayNum(Integer.parseInt(parts[0].trim()));
				String mergeList[] = parts[1].trim().split(" ");
				String splitList[] = parts[2].trim().split(" ");
				node.setMergeList(putListIntoNodes(mergeList));
				node.setSplitList(putListIntoNodes(splitList));
				addNextDay(node);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Pretty print the list.
	 */
	public void printMap() {
		LBDayNode currentNode = new LBDayNode();
		for (Iterator<LBDayNode> i = dayList.iterator(); i.hasNext();) {
			currentNode = i.next();
			System.out.print(currentNode.getDayNum());
			System.out.print("|");
			LBOperationNode node;
			while ((node = currentNode.getMergeNode()) != null) {
				System.out.print(node.getSource() + " " + node.getTarget()
						+ " ");
			}
			System.out.print("|");
			while ((node = currentNode.getSplitNode()) != null) {
				System.out.print(node.getSource() + " " + node.getTarget()
						+ " ");
			}
			System.out.println("");
			currentNode.resetMergeIterator();
			currentNode.resetSplitIterator();
		}

	}

	/**
	 * Write the list into a file
	 * @param filename
	 */
	public void writeMapToFile(String filename) {
		File mapFile = new File(filename);
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(mapFile));
			LBDayNode currentNode = new LBDayNode();
			for (Iterator<LBDayNode> i = dayList.iterator(); i.hasNext();) {
				currentNode = i.next();
				String line = new String();
				line = line + currentNode.getDayNum();
				line = line + "|";
				LBOperationNode node;
				line = line + " ";
				while ((node = currentNode.getMergeNode()) != null) {
					line = line + node.getSource() + " " + node.getTarget()
							+ " ";
				}
				line = line + "| ";
				while ((node = currentNode.getSplitNode()) != null) {
					line = line + node.getSource() + " " + node.getTarget()
							+ " ";
				}
				line = line + "\n";
				writer.write(line);
				writer.flush();
				currentNode.resetMergeIterator();
				currentNode.resetSplitIterator();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public int getDayNum() {
		return dayList.size();
	}

	public static void main(String[] args) {
		// String s = "1|2||3";
		// s = s.replace('|', '-');
		// System.out.println(s.split("-").length);
		DayList list = new DayList();
		list.readListFromFile("/s/chopin/e/proj/granules/simonsui/Granules/trunk/bin/testfile.txt");
		list.printMap();
		list.writeMapToFile("/s/chopin/e/proj/granules/simonsui/Granules/trunk/bin/testfile2.txt");
	}

}
