/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.portionaldynamicmerge;

import java.util.LinkedList;

/**
 * This class contains load balancing operation information of one simulation
 * day. It is used for re-run the same load balancing operations in different
 * simulations.
 * 
 * @author simonsui
 * 
 */
public class LBDayNode {
	int dayNum;
	LinkedList<LBOperationNode> splitList;
	LinkedList<LBOperationNode> mergeList;
	int splitIterator;
	int mergeIterator;

	void resetSplitIterator() {
		splitIterator = 0;
	}

	void resetMergeIterator() {
		mergeIterator = 0;
	}

	int getDayNum() {
		return dayNum;
	}

	void setDayNum(int day) {
		this.dayNum = day;
	}

	public LBDayNode() {
		splitList = new LinkedList<LBOperationNode>();
		mergeList = new LinkedList<LBOperationNode>();
		splitIterator = 0;
		mergeIterator = 0;
	}

	public void addSplitNode(int source, int target) {
		LBOperationNode splitNode = new LBOperationNode(source, target);
		splitList.add(splitNode);
	}

	public void addMergeNode(int source, int target) {
		LBOperationNode mergeNode = new LBOperationNode(source, target);
		splitList.add(mergeNode);
	}

	public LBOperationNode getSplitNode() {
		if (splitIterator >= splitList.size())
			return null;
		LBOperationNode splitNode = splitList.get(splitIterator);
		splitIterator++;
		return splitNode;
	}

	public LBOperationNode getMergeNode() {
		if (mergeIterator >= mergeList.size())
			return null;
		LBOperationNode mergeNode = mergeList.get(mergeIterator);
		mergeIterator++;
		return mergeNode;
	}

	public void setSplitList(LinkedList<LBOperationNode> splitList) {
		this.splitList = splitList;
	}

	public void setMergeList(LinkedList<LBOperationNode> mergeList) {
		this.mergeList = mergeList;
	}
}
