/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.naadsm.resplit;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.LinkedList;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.service.ServiceException;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaGenericComputation;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.naadsm.resplit.event.Checkpoint;
import ds.granules.naadsm.resplit.event.EventType;
import ds.granules.naadsm.resplit.event.EventTypeFactory;
import ds.granules.naadsm.resplit.event.RequestForFTCheckpoint;
import ds.granules.naadsm.resplit.event.StartNewDayEvent;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;
import ds.granules.util.MD5Checksum;

/**
 * This is the worker for distributed NAADSM. It is responsible for communicate
 * the NAADSM project with controllers.
 * 
 * @author Zhiquan (Simon) Sui
 * 
 */

public class Worker extends MapReduceBase implements JavaMessageHandler {
	JavaGenericComputation jc;
	private boolean active;
	private boolean finished;
	private byte[] buffer;
	FileWriter writer;
	BufferedWriter out;
	boolean checkpointProcess;
	boolean started;
	String sysCommand;
	private Results exchangeResults;
	private boolean silent;
	File output;
	private int workerNumber;

	/**
	 * Constructor. Initialize the variables and the byte array.
	 */
	public Worker() {
		setDomain("Generic");
		setVersionInformation(200);
		setOperationIdentifier("ABCDEFGHIJKLMN");
		started = false;
		finished = false;
		silent = true;
		active = false;
		jc = new JavaGenericComputation("Worker");
		buffer = new byte[0];
		exchangeResults = createResults(false, false);
		checkpointProcess = false;
	}

	private synchronized void syncExecution() {
		try {
			int num = Integer.parseInt(this.getProcessingDirectives()
					.getProperty("Worker Number"));
			workerNumber = num;
			if (!started) {
				startSpare();
				started = true;
				if (num == 0)
					this.active = true;
			}
			StreamingAccess access = null;
			String datasetIdentifier = "WORKER_DATA"
					+ this.getProcessingDirectives().getProperty(
							"Worker Number");
			DatasetCollection datasetCollection = getDatasetCollection();
			if (datasetCollection == null)
				return;

			Dataset dataset = datasetCollection.getDataset(datasetIdentifier);
			if (dataset == null) {
				System.err.println("DATASET IS NULL!!!!!!!!!!!!!!!");
				return;
			}
			if (dataset.getDatasetType() == Dataset.STREAMS) {
				access = (StreamingAccess) dataset;
			} else {
				System.err.println("Incorrect dataset:"
						+ dataset.getDatasetType());
				return;
			}
			if (access != null) {
				while (dataset.isDataAvailable()) {
					NBEvent nbEvent = access.getStreamingData();
					if (nbEvent == null) {
						continue;
					}
					if (nbEvent.getContentSynopsis().equals("Sync/DownStream")) {
						downStreamProcess(nbEvent);
					} else if (nbEvent.getContentSynopsis().equals(
							"Sync/CheckpointRequest")) {
						System.out.println("Worker " + num
								+ " got checkpoint request!");
						if (!active)
							return;
						Results resultsEx = ResultsFactory.getInstance()
								.getResults(nbEvent.getContentPayload());
						byte[] content = resultsEx.getResultPayload();
						RequestForFTCheckpoint requestForFTCheckpoint = (RequestForFTCheckpoint) EventTypeFactory
								.createEventType(content);
						requestForFTCheckpoint.setWorkerID(num);
						if (buffer == null)
							buffer = requestForFTCheckpoint.marshalling();
						else if (buffer.length == 0)
							buffer = requestForFTCheckpoint.marshalling();
						else
							buffer = Controller.combineByte(buffer,
									requestForFTCheckpoint.marshalling());
						checkpointProcess = true;
					} else if (nbEvent.getContentSynopsis().equals(
							"Sync/DownStream/" + num)) {
						byte[] resultsPayload = nbEvent.getContentPayload();
						Results results = ResultsFactory.getInstance()
								.getResults(resultsPayload);
						byte[] bb = results.getResultPayload();
						EventType eventType = Controller.parseEvents(bb)
								.getFirst();
						if (eventType.getType() == EventType.RESTORE
								|| eventType.getType() == EventType.MULTI_RESTORE) {
							activation(nbEvent);
						} else {
							if (!active)
								return;
							if (eventType.getType() == EventType.REQUEST_FOR_FT_CHECKPOINT) {
								RequestForFTCheckpoint requestForFTCheckpoint = (RequestForFTCheckpoint) eventType;
								if (!requestForFTCheckpoint.active()) {
									this.active = false;
								}
							}
							// Re-calculate the boundaries when split
							// float coordinator = (leftB + rightB) / 2;
							// SplitEvent splitEvent = (SplitEvent)
							// eventType;
							// splitEvent.coordinate = coordinator;
							JavaByteMessage jbm = new JavaByteMessage(
									ControlMessage.DATA);
							byte[] bytes = eventType.marshalling();
							jbm.setInput(bytes);
							byte[] totalMessage = jbm.toByteArray();
							if (!silent) {
								System.out
										.println("----------------ALL Send to NAADSM:-------------------");
								for (int i = 0; i < totalMessage.length; i++)
									System.out
											.print(Integer
													.toHexString(totalMessage[i])
													+ " ");
								System.out.println();
								System.out
										.println("---------------------------------------------------");
							}
							try {
								jbm.setDescription(MD5Checksum
										.getMD5Checksum(bytes));
							} catch (NoSuchAlgorithmException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							jc.sendMessage(jbm);
							// System.out
							// .println("Sent to NAADSM checkpoint message");
						}

					}
				}
			} else {
				System.err.println("The dataset is null!");
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void execute() throws ProcessingException {
		syncExecution();
	}

	/**
	 * Activate a spare worker into active worker
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws IOException
	 * @throws MarshallingException
	 */
	private synchronized void activation(NBEvent nbEvent) throws IOException,
			MarshallingException {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		this.active = true;

		// restore = restore.getBoundaries(ResultsFactory.getInstance()
		// .getResults(nbEvent.getContentPayload()).getResultPayload());
		// Get new boundary of the current worker
		// this.leftB = restore.left;
		// this.rightB = restore.right;
		// this.lowerB = restore.down;
		// this.upperB = restore.up;
		// System.out.println("restore worker id=" + restore.getWorkerID());
		Results resultsEx = ResultsFactory.getInstance().getResults(
				nbEvent.getContentPayload());
		JavaByteMessage jbm = null;
		byte[] bytes = resultsEx.getResultPayload();
		try {
			jbm = new JavaByteMessage(ControlMessage.DATA);
			// System.out.println("Restore message marshalling successfully.");
			jbm.setInput(bytes);
			jbm.setDescription(MD5Checksum.getMD5Checksum(bytes));
			jc.sendMessage(jbm);
			// System.out.println("Input set successfully.");
			if (!silent) {
				System.out
						.println("----------------Send to NAADSM:-------------------");
				for (int i = 0; i < bytes.length; i++)
					System.out.print(Integer.toHexString(bytes[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}

			if (!silent) {
				byte[] totalMessage = jbm.toByteArray();
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			// System.out.println("Prepare to print.");
			// System.out.println(restore.getType());
			// System.out.println(bytes.length);
			System.out.println("************ Worker " + num + " is active!"
					+ " The size of the message is:" + bytes.length);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Start NAADSM as a spare worker
	 */
	private synchronized void startSpare() {
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// Default command line
		sysCommand = "/s/chopin/e/proj/granules/simonsui/dis-naadsm/naadsm-3.2.18__PortionalDynamicMerge/bin/naadsm -V 0 --rng-seed 3 -o ./tmp/naadsm_output_.log -h /s/chopin/e/proj/granules/simonsui/dis-naadsm/660000_population_scaled_from_kansas_9_infected.xml /s/chopin/e/proj/granules/simonsui/param_20130219/param_20130219.xml";
		if (this.getProcessingDirectives().getProperty("cmdLine") != "")
			sysCommand = this.getProcessingDirectives().getProperty("cmdLine");
		try {
			jc.startExecution(sysCommand, ControlMessage.UPIPES, this);
			// System.out.println("Worker " + num + " is spared!");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Process messages with synopsis of DownStream
	 * 
	 * @param nbEvent
	 *            Message containing all information
	 * @throws ServiceException
	 * @throws NBEventException
	 * @throws IOException
	 * @throws MapReduceException
	 * @throws MarshallingException
	 * @throws NullPointerException
	 * @throws NoSuchAlgorithmException
	 */
	private synchronized void downStreamProcess(NBEvent nbEvent)
			throws ServiceException, NBEventException, IOException,
			MapReduceException, MarshallingException, NullPointerException,
			NoSuchAlgorithmException {
		byte[] extraSend = null;
		int num = Integer.parseInt(this.getProcessingDirectives().getProperty(
				"Worker Number"));
		// System.out.println("------------>" + num + ":" + active);
		byte[] send = null;
		EventType last = null;
		try {
			byte[] eventPayload = nbEvent.getContentPayload();
			if (eventPayload == null) {
				System.out.println("Payload NULL");
				return;
			}
			Results resultsEx = ResultsFactory.getInstance().getResults(
					eventPayload);
			if (resultsEx == null) {
				return;
			}

			send = resultsEx.getResultPayload();
			if (send == null) {
				return;
			}

			// File file = new File(
			// "/s/chopin/e/proj/granules/simonsui/Granules/trunk/GotFromController"
			// + workerNumber + ".txt");
			//
			// FileWriter writer = new FileWriter(file);
			// writer.append("Size:" + send.length + "\n");
			// writer.flush();
			// writer.append("Message: ");
			// for (int i = 0; i < send.length; i++)
			// writer.append(send[i] + " ");
			// writer.append('\n');
			// writer.close();
		} catch (RuntimeException e1) {
			System.out.println("Matthew told me to do this!");
		} catch (Exception e) {
			System.out.println("Worker:" + num + " has error");
			e.printStackTrace();
		}
		last = Controller.parseEvents(send).getLast();
		int eventType = last.getType();
		if ((eventType == EventType.START_NEW_DAY) && active) {
			StartNewDayEvent startEvent = (StartNewDayEvent) last;
			startEvent.setWorker(num); // Modify the worker number property in
										// START_NEW_DAY message
			byte[] start = startEvent.marshalling();
			// System.out.println("Worker: " + num + " Day: "
			// + startEvent.getDayNumber());
			// replace the START_NEW_DAY message with the correct START_NEW_DAY
			// message
			for (int i = 0; i < EventType.EventSize[EventType.START_NEW_DAY]; i++) {
				send[send.length - EventType.EventSize[EventType.START_NEW_DAY]
						+ i] = start[i];
			}
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			jbm.setInput(send);
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			try {
				jbm.setDescription(MD5Checksum.getMD5Checksum(send));
			} catch (NoSuchAlgorithmException e) {
				e.printStackTrace();
			}
			jc.sendMessage(jbm);
		} else if (eventType == EventType.END_OF_SIMULATION) {
			JavaByteMessage jbm = new JavaByteMessage(ControlMessage.DATA);
			jbm.setInput(extraSend);
			byte[] totalMessage = jbm.toByteArray();
			if (!silent) {
				System.out
						.println("----------------ALL Send to NAADSM:-------------------");
				for (int i = 0; i < totalMessage.length; i++)
					System.out
							.print(Integer.toHexString(totalMessage[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			}
			try {
				jbm.setDescription(MD5Checksum.getMD5Checksum(extraSend));
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			jc.sendMessage(jbm);
			Results suspendResponse;
			suspendResponse = createResults(true, true);
			suspendResponse.setResultPayload(new byte[1]);
			writeResults("Sync/WorkerSuspendResponse", suspendResponse);
			System.exit(0);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ds.granules.bridge.JavaMessageHandler#handleMessage(byte[]) Once got
	 * message from NAADSM project, this function will be activated.
	 */
	@Override
	public synchronized void handleMessage(final byte[] msg) {
		// System.out.println("Got From NAADSM");
		final byte[] par = msg;
		if (!silent) {
			try {
				System.out
						.println("----------------Got From NAADSM:-------------------");
				for (int i = 0; i < par.length; i++)
					System.out.print(Integer.toHexString(par[i]) + " ");
				System.out.println();
				System.out
						.println("---------------------------------------------------");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		JavaByteMessage jbm = new JavaByteMessage(par);

		byte[] bytes = jbm.getInput();
		// File file = new File(
		// "/s/chopin/e/proj/granules/simonsui/Granules/trunk/Worker"
		// + workerNumber + ".txt");
		// try {
		// LinkedList<EventType> list = Controller.parseEvents(bytes);
		// FileWriter writer = new FileWriter(file);
		// writer.append("=============================\n");
		// for (int i = 0; i < list.size(); i++) {
		// EventType event = list.get(i);
		// byte[] eventMessage = event.marshalling();
		// for (int j = 0; j < eventMessage.length; j++) {
		// writer.append(eventMessage[j] + " ");
		// }
		// writer.append('\n');
		// }
		// writer.append('\n');
		// writer.append("=============================\n");
		// writer.append("CHECKSUM 1: " + MD5Checksum.getMD5Checksum(bytes)
		// + "\n");
		// writer.append("CHECKSUM 2: " + jbm.getDescription() + "\n");
		// writer.append("Message: ");
		// for (int i = 0; i < bytes.length; i++)
		// writer.append(bytes[i] + " ");
		// writer.append('\n');
		// writer.close();
		// } catch (Exception e2) {
		// // TODO Auto-generated catch block
		// e2.printStackTrace();
		// }

		try {
			if (!jbm.getDescription().equals(MD5Checksum.getMD5Checksum(bytes))) {
				System.out
						.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Error in byte Array!");
			}
		} catch (NoSuchAlgorithmException e1) {
			e1.printStackTrace();
		}

		boolean isCheckpoint;
		try {
			if (EventTypeFactory.createEventType(bytes).getType() == EventType.CHECKPOINT) {
				isCheckpoint = true;
			} else
				isCheckpoint = false;
		} catch (Exception e) {
			isCheckpoint = false;
		}
		try {
			if (isCheckpoint) {
				Checkpoint checkpoint = (Checkpoint) EventTypeFactory
						.createEventType(bytes);
				if (!checkpointProcess) {
					// Record the time if received checkpoint message
					int num = Integer.parseInt(this.getProcessingDirectives()
							.getProperty("Worker Number"));
					byte[] res = checkpoint.marshalling();
					exchangeResults = createResults(false, false);
					exchangeResults.setResultPayload(res);
					writeResults("Sync/Checkpoint", exchangeResults);
				} else {

					byte[] checkpointBytes = checkpoint.marshalling();
					byte[] remainingBytes = new byte[bytes.length
							- checkpointBytes.length];
					for (int i = 0; i < remainingBytes.length; i++) {
						remainingBytes[i] = bytes[i + checkpointBytes.length];
					}
					Results exchangeResultsToController = createResults(false,
							false);
					exchangeResultsToController
							.setResultPayload(remainingBytes);
					writeResults("Sync/UpStream", exchangeResultsToController);
					Results exchangeResultsToSpeculator = createResults(false,
							false);
					exchangeResultsToSpeculator
							.setResultPayload(checkpointBytes);
					writeResults("Async/Checkpoint",
							exchangeResultsToSpeculator);
					checkpointProcess = false;
				}
			} else {
				int num = Integer.parseInt(this.getProcessingDirectives()
						.getProperty("Worker Number"));
				System.out.println(num + " has done.");
				exchangeResults = createResults(false, false);
				exchangeResults.setResultPayload(bytes);
				writeResults("Sync/UpStream", exchangeResults);
			}
		} catch (MapReduceException e) {
			e.printStackTrace();
		}
	}

}
