/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.operation;

import java.util.Properties;

import ds.granules.dataset.DatasetCollection;

/**
 * This is an interface that allows application instances to be configured. It
 * contains setter method for several fields that are part of the
 * <code>Operation</code> interface.
 * 
 * @author Shrideep Pallickara
 * 
 */
public interface OperationConfiguration {

    /**
     * Sets the domain for this operation
     * 
     * @param domain
     */
    public void setDomain(String domain);


    /**
     * This method sets the operation identifier for a given operation
     * 
     * @param operationIdentifier
     */
    public void setOperationIdentifier(String operationIdentifier);


    /**
     * Sets the instance identifier associated with a given operation.
     * 
     * @param instanceIdentifier
     */
    public void setInstanceIdentifier(String instanceIdentifier);


    /**
     * Sets the version information for this operation.
     * 
     * @param version
     */
    public void setVersionInformation(long version);


    /**
     * Set the processing directives for the operation
     * 
     * @param processingDirectives
     */
    public void setProcessingDirectives(Properties processingDirectives);


    /**
     * Establish the dataset needed by the application in question
     * 
     * @param datasetCollection
     */
    public void setDatasetCollection(DatasetCollection datasetCollection);


    /**
     * Mark the initialization of the application as having completed
     */
    public void setInitializationComplete();


    /**
     * Mark the termination condition of this operation as having reached. If
     * the execution profile of the operation is isStayAlive() the operation
     * cannot be garbage collected till such time that this flag has been set.
     */
    public void setTeminationConditionReached();


    /**
     * Set/Unset whether this operation needs initialization.
     * 
     * @param needsInitialization
     */
    public void setInitializationNeeded(boolean needsInitialization);


    /**
     * Sets the operation as being executable exactly once.
     * 
     */
    public void setAsExecuteOnce();


    /**
     * Sets the operation as being executable periodically at the specified
     * intervals. If the specified execution interval is 0, the method simply
     * returns without performing any actions. It would be as if this method was
     * never invoked. This method will introduce additional upper and lower
     * bounds subsequently.
     * 
     * @param executionInterval
     */
    public void setAsExecutePeriodically(long executionInterval);


    /**
     * Sets the operation as being executable a fixed number of times. If the
     * value that is specified is either 0 or 1, this method simply returns
     * without performing any actions. It would be as if this method was never
     * invoked.
     * 
     * @param numOfTimes
     */
    public void setAsExecuteFixedNumberOfTimes(int numOfTimes);


    /**
     * Sets the operation as being executable when data is available
     */
    public void setAsExecuteWhenDataAvailable();


    /**
     * Sets the operation as being stayAlive. The operation will not be
     * dismounted till such time that the termination condition associated with
     * the operation has been set.
     * 
     */
    public void setAsStayAlive();

}
