/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.operation;

import java.util.Enumeration;

import ds.granules.scheduler.TaskLifecycleMetrics;

/**
 * The <code>ProgressTracker</code> encapsulates information about the status of
 * jobs. One can also track the status of the various tasks that constitute the
 * job.
 * 
 * @author Shrideep Pallickara
 * 
 */

public interface ProgressTracker {

    /** Retrieve the Job identifier */
    public String getIdentifier();


    /**
     * Check to see if the Job has completed.
     * 
     * @return true if it is complete; false otherwise.
     */
    public boolean isComplete();


    /**
     * Check to see if the Job completed successfully.
     * 
     * @return true if all the constiuent tasks completed successfully; false
     *         otherwise.
     */
    public boolean isSuccessful();


    /**
     * Gets the number of concurrent deployments. This reflects the number of
     * tasks that have been deployed successfully on a set of resources.
     * 
     * @return The number of concurrent deployments.
     */
    public int getNumberOfConcurrentDeployments();


    /**
     * Gets the task identifiers associated with the concurrent deployments
     * 
     * @return An enumeration of the task identifiers.
     */
    public Enumeration<String> getTaskIdentifiers();


    /**
     * Retrieve the TaskLifecycleMetrics associated with a given task.
     * 
     * @param taskIdentifier
     *            The ID associated with a given task.
     * @return The lifecycle metrics for the task in question.
     */
    public TaskLifecycleMetrics getTaskLifecycleMetrics(String taskIdentifier);


    /**
     * Retrieve the completion status for a specific taskIdentifier
     * 
     * @param taskIdentifier
     * @return The completion status
     */
    public int getCompletionStatus(String taskIdentifier);


    /**
     * Retrieve the deployment stream associated with a given task.
     * 
     * @param taskIdentifier
     *            The ID associated with a given task.
     * @return The deployment stream associated with the given task.
     */
    public String getDeploymentStream(String taskIdentifier);

}
