/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.operation;

import java.util.Enumeration;
import java.util.Iterator;

/**
 * This interface corresponds to the <i>Reduce</i> role in a Map-Reduce
 * operation. It incorporates support for adding and removing Map functions.
 * Additionally, the Reducer can keep track of Map from which Outputs have been
 * received, and those from which Outputs are currently pending.
 * 
 * @author Shrideep Pallickara
 * 
 */

public interface Reduce extends MapReduce {

    /**
     * Retrieves the number of registered Map instances
     * 
     * @return The number of <code>Map</code>s
     */
    public int getNumberOfMaps();


    /**
     * Retrieves the set of Map identifiers
     * 
     * @return An enumeration of the list of registered identifiers
     */
    public Enumeration<String> getMapIdentifiers();


    /**
     * Check to see is the Reducer is missing outputs from any of the registered
     * Maps.
     * 
     * @return <code>true</code> is the Reducer is still awaiting outputs;
     *         <code>false</code> otherwise.
     */
    public boolean isAwaitingOutputs();


    /**
     * Retrieves the set of Map identifiers from which an Output has not been
     * received so far.
     * 
     * @return An iterator of the pending identifiers
     */
    public Iterator<String> getAwaitingOutputsFrom();


    /**
     * This method resets the output tracker associated with the reducer. This
     * is useful when the stages are part of an iterative computation.
     */
    public void resetOutputTracker();


    /**
     * Mark the receipt of Output from a given Map instance
     * 
     * @param mapIdentifier
     *            The ID of the Map from which an output has been received.
     */
    public void setReceivedOutputFrom(String mapIdentifier);


    /**
     * Adds a Map that will be reduced.
     * 
     * @param map
     *            Add a Map instance for this reducer
     * @return The number of maps that are registered with this reducer
     * @throws MapReduceException
     */
    public int addMap(Map map) throws MapReduceException;


    /**
     * Removes a Map from the list of Maps that are registered to this reducer.
     * 
     * @param mapIdentifier
     *            The identifier of the Map to removed
     * @return The number of maps that are registered with this reducer
     * @throws MapReduceException
     *             If there are problems removing the Map.
     */
    public int removeMap(String mapIdentifier) throws MapReduceException;
}
