/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.results;

import java.io.IOException;

import cgl.narada.util.UUIDRetriever;
import ds.granules.exception.MarshallingException;

/**
 * This is a singleton-class that is used for creating and reconstructing
 * <code>Results</code> and the elements that comprise it such as
 * <code>SequencingInfo</code>.
 * 
 * @author Shrideep Pallickara
 * 
 */
public class ResultsFactory {
    private static final ResultsFactory instance = new ResultsFactory();
    private UUIDRetriever retriever;


    private ResultsFactory() {
        retriever = UUIDRetriever.getInstance();
    }


    /**
     * Retrieve the singleton instance of the ResultsFactory.
     * 
     * @return The singleton instance.
     */
    public static ResultsFactory getInstance() {
        return instance;
    }


    /**
     * This method creates a Results object.
     * 
     * @param instanceIdentifier
     *            The instance identifier of the application instance.
     * @param completeResult
     *            Indicates whether this is a complete result or a partial one.
     * @param processingComplete
     *            Indicates whether the processing is complete.
     * @return The Results object duly initialized.
     * 
     * @throws NullPointerException
     *             If the specified instance identifier is null.
     */
    public Results createResult(String instanceIdentifier,
                                boolean completeResult,
                                boolean processingComplete) {
        if (instanceIdentifier == null) {
            throw new NullPointerException("Instance identifier is null.");
        }
        String resultsIdentifier = retriever.getTimeBasedUUIDAsString();

        ResultsWidget results =
            new ResultsWidget(instanceIdentifier, resultsIdentifier,
                              completeResult, processingComplete);

        return results;
    }


    /**
     * Recreate the Results object from a byte[].
     * 
     * @param marshalledBytes
     *            The byte[] representation of the Results.
     * @return The Results object duly initialized.
     * @throws IOException
     *             If there are problems recreating the object.
     * @throws MarshallingException
     *             If there are problems recreating the object.
     */
    public Results getResults(byte[] marshalledBytes) throws IOException,
        MarshallingException {
        Results results = new ResultsWidget(marshalledBytes);
        return results;
    }


    /**
     * Creates a sequencing info object.
     * 
     * @return A newly created <code>SequenceInfo</code> object
     */
    public SequencingInfo createSequencingInfo() {
        SequencingWidget sequencingWidget = new SequencingWidget();
        return sequencingWidget;
    }


    /**
     * Creates a sequencing info from a set of bytes.
     * 
     * @param marshalledBytes
     * @return The recreated <code>SequenceInfo</code> object
     * @throws IOException
     *             If there are problems recreating the object.
     */
    public SequencingInfo getSequencingInfo(byte[] marshalledBytes)
        throws IOException {
        SequencingWidget sequencingWidget =
            new SequencingWidget(marshalledBytes);
        return sequencingWidget;
    }


    /**
     * @param args
     */
    public static void main(String[] args) {
        // TODO Auto-generated method stub

    }

}
