/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.results;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * This class is an implementation of the <code>SequencingInfo</code> interface.
 * 
 * @author Shrideep Pallickara
 * 
 */

public class SequencingWidget implements SequencingInfo {
    private long lower = 0;
    private long upper = 0;
    private long maxSequence = 0;
    private long singleSequence = 0;
    private boolean hasSequenceRange = false;
    private boolean hasMaxSequence = false;
    private boolean hasSingleSequence = false;
    private boolean mutable = true;


    protected SequencingWidget() {

    }


    @Override
    public boolean hasMaxSequence() {
        return hasMaxSequence;
    }


    @Override
    public void setMaxSequence(long maxSequence) {
        checkForIllegalModification();
        this.maxSequence = maxSequence;
        hasMaxSequence = true;
    }


    @Override
    public long getMaxSequence() {
        return maxSequence;
    }


    @Override
    public boolean hasSingleSequence() {
        return hasSingleSequence;
    }


    @Override
    public void setSingleSequence(long singleSequence) {
        checkForIllegalModification();
        this.singleSequence = singleSequence;
        hasSingleSequence = true;
    }


    @Override
    public long getSingleSequence() {
        return singleSequence;
    }


    @Override
    public boolean hasSequenceRange() {
        return hasSequenceRange;
    }


    @Override
    public void setSequenceRange(long lower, long upper) {
        checkForIllegalModification();
        this.lower = lower;
        this.upper = upper;
        hasSequenceRange = true;
    }


    @Override
    public long getLower() {
        return lower;
    }


    @Override
    public long getUpper() {
        return upper;
    }


    @Override
    public boolean isMutable() {
        return mutable;
    }


    protected void setImmutable() {
        mutable = false;
    }


    public byte[] getBytes() throws IOException {
        byte[] marshalledBytes = null;
        ByteArrayOutputStream baOutputStream = new ByteArrayOutputStream();
        DataOutputStream dout =
            new DataOutputStream(new BufferedOutputStream(baOutputStream));

        dout.writeBoolean(hasSingleSequence);
        if (hasSingleSequence) {
            dout.writeLong(singleSequence);
        }

        dout.writeBoolean(hasMaxSequence);
        if (hasMaxSequence) {
            dout.writeLong(maxSequence);
        }

        dout.writeBoolean(hasSequenceRange);
        if (hasSequenceRange) {
            dout.writeLong(lower);
            dout.writeLong(upper);
        }
        mutable = false;
        dout.writeBoolean(mutable); /* This should be false by now. */

        dout.flush();
        marshalledBytes = baOutputStream.toByteArray();
        return marshalledBytes;
    }


    protected SequencingWidget(byte[] marshalledBytes) throws IOException {
        ByteArrayInputStream baInputStream =
            new ByteArrayInputStream(marshalledBytes);
        DataInputStream din =
            new DataInputStream(new BufferedInputStream(baInputStream));

        hasSingleSequence = din.readBoolean();
        if (hasSingleSequence) {
            singleSequence = din.readLong();
        }

        hasMaxSequence = din.readBoolean();
        if (hasMaxSequence) {
            maxSequence = din.readLong();
        }

        hasSequenceRange = din.readBoolean();
        if (hasSequenceRange) {
            lower = din.readLong();
            upper = din.readLong();
        }
        mutable = din.readBoolean();
        if (mutable) {
            System.out.println("SequencingInfo: Reset to immutable");
            mutable = false; /* This should be false. */
        }

    }


    private void checkForIllegalModification() {
        if (!mutable) {
            String errorReport =
                "SequencingInfo object is closed for modifications.";
            throw new IllegalModificationException(errorReport);
        }
    }
}
