/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
/***
 * Modified driver for load testing
 * This driver decodes and returns a DATA message or
 * will execute fibonacci on a START_STOP command
 * 
 * This version supports either sockets or pipes for communication
 * 
 * @author K Ericson
 */

#include "fibWrapper.c"
#include "../../../bridge/candcpp/helpers.h"
#include <sys/timeb.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

//typedef std::basic_string<unsigned char>u_char_str;

//Timing helpers
long milliDiff(struct timeb start, struct timeb stop)
{
	long mstart = start.time * 1000 + start.millitm;
	long mstop = stop.time * 1000 + stop.millitm;
	return mstop - mstart;
}

long microDiff(struct timeval start,struct timeval stop)
{
	long sa = start.tv_sec * 1000000 + start.tv_usec;
	long so = stop.tv_sec * 1000000 + stop.tv_usec;
	return so - sa;
}

/***
 * Main driver, as in the Java version, it assumes that the first 4 bytes of communication
 * will always indicate the length of the rest of the message
 *
 * Socket version needs a port number
 */
int main(int argc, char *argv[]){
	char isSocket;
	char iSize[4]; //grabs the initial int - length of the rest of the message;
	
	struct fibWrapper *fw;
	struct javaMessage *back;

	struct sockaddr_in serv_addr;
	struct hostent *server;
	int fdin,fdout; //input file descriptor, output file descriptor

	//always assuming stderr for error
	if(argc >=2){
		//assume sockets
		isSocket = (0==0);
		
		int portNum = atoi(argv[1]);
		int sockfd = socket(AF_INET,SOCK_STREAM,0);
		
		server = gethostbyname("localhost");
		if (server == NULL) {
			fprintf(stderr,"ERROR, no such host\n");
			exit(0);
		}
	 	bzero((char *) &serv_addr, sizeof(serv_addr));
		serv_addr.sin_family = AF_INET;
		bcopy((char *)server->h_addr, 
			(char *)&serv_addr.sin_addr.s_addr,
			server->h_length);
    		serv_addr.sin_port = htons(portNum);
   		if (connect(sockfd,(struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0) 
		fprintf(stderr,"ERROR!");
		fdin = sockfd;
		fdout = sockfd;
	}
	else{ //not sockets
		isSocket = (1==0);
		fdin = 0;		//cin
		fdout = 1;	//cout
	}

	while((0==0)){
		back = NULL;
		fprintf(stderr,"waiting for input\n");
		int size = 0;
		int r = 0;
		while (r < 4)
		{
			r = read(fdin,iSize+r,4-r) + r; //grab the size of the message
		}
		size = readInt(iSize);
		fprintf(stderr,"msg size: %d\n",size);
		char input[size];
		r = 0;
		while (r < size)
			r = read(fdin,input + r,size-r) + r;


		struct javaMessage jm = javaMessageFromBlob(input); //try to initialize


//debugging information
		fprintf(stderr,"Type: %d\ninput length: %d\noutput length: %d\nstream: %s\ndescription: %s\nfrom java? %d\n", jm.mType, jm.inputSize, jm.outputSize, jm.stream, jm.description,jm.fromJava);



		//this section is responsible for initial evaluation of messages, and
		//delegation of tasks to handle a message
		if(jm.mType == INITIALIZE)
		{
			fprintf(stderr,"initializing\n");
			struct fibWrapper f = fibWrapper(jm);
			fw = &f;
			//send back the status
			char oa[4 + 4 + fw->sSize];
			char* o = oa;//int for type, int for length, actual string
			int start = 0;
			writeInt(o,STRING,&start);
			writeInt(o+start,fw->sSize,&start);
			memcpy(o+start,fw->status,fw->sSize);
			struct javaMessage b = javaMessage(STATUS);
			back = &b;
			back->inputSize=(jm.inputSize);
			back->input=(jm.input);
			back->outputSize=(4 + 4 + fw->sSize);
			back->output=(o);
			back->streamSize = fw->sSize;
			back->stream=(jm.stream);
			back->descSize = jm.descSize;
			back->description=(jm.description);
		}
		else if(jm.mType == START_STOP)
		{
			char * stat = fw->status;
			if(stat == "RUNNING")
			{
				stop(fw);
				return(0);
			}
			else
			{
				//start timing
				struct timeval sta;
				struct timeval sto;
				gettimeofday(&sta,0);
				run(jm,fw);
				gettimeofday(&sto,0);
				long timing = microDiff(sta, sto);
				//stop timing
				//should check that the result was valid here
				//struct javaMessage j = getTimingResults(timing, fw);
				struct javaMessage j = getResults(fw);
				back = &j;
				//back = getResults(fw);
				fprintf(stderr,"back type: %d, back description %s, input size: %d, output size: %d, back size %d\n",back->mType, back->description, back->inputSize, back->outputSize, byteSize(back));
				//send back the original input as well as the output
				fprintf(stderr,"back from fib\n");

				int tpe;
	char bob[4];
	char* temp = bob;//temp holder for int bytes
	int st2 = 0; //index to keep track of where we are in the char*

	memcpy(temp,back->output+st2,4);
	st2+=4;
	tpe=readInt(temp);
	char bobl[8];
	char*temp2 = bobl;
	memcpy(temp2,back->output+st2,8);
	long res = readLong(temp2);
	
	fprintf(stderr,"type readback: %d, res readback: %d\n",tpe, res);
			}
		}
		else if(jm.mType == DATA)
		{
			//just timing from Java, so just send back a Results
			fprintf(stderr,"in DATA block!!!!!!!");
			struct javaMessage b = javaMessage(RESULTS);
			back = &b;
			back->output=(jm.input);
			back->outputSize=(jm.inputSize);
			char inp[0];
			back->input=(inp);
			back->inputSize=(0);
			back->stream=(jm.stream);
			back->streamSize = jm.streamSize;
			back->description = jm.description;
			back->descSize = jm.descSize;
		}
		else if(jm.mType == CHANGE_BRIDGE_TYPE)
		{
			fprintf(stderr, "changing bridge");
			//switch to the new communication type
			char* inp = jm.input;
			int s;
			memcpy(&s,inp,4);
			//Should be an int;
			int btype = readInt(inp+4);
			if(btype == 0)
			{
				//switch to pipes
				//try to close socket:
				if(isSocket)
				{
					close(fdin);
					close(fdout); //needed? or will it cause it to crash??
					isSocket = (1==0);
				}
				fdin = 0;
				fdout = 1;
				fprintf(stderr,"switched to pipes\n");

			}
			else if (btype == 2)
			{
				if(!isSocket)
				{
					//assuming that port is the next bit of data
					int port = readInt(inp+12);
					//connect
					int portNum = port;
					int sockfd = socket(AF_INET,SOCK_STREAM,0);
				
					server = gethostbyname("localhost");
		   			if (server == NULL) {
		   			     fprintf(stderr,"ERROR, no such host\n");
   					     exit(0);
		  			}
		    			bzero((char *) &serv_addr, sizeof(serv_addr));
    					serv_addr.sin_family = AF_INET;
		    			bcopy((char *)server->h_addr, 
		        		(char *)&serv_addr.sin_addr.s_addr,
		       			  server->h_length);
		     			  serv_addr.sin_port = htons(portNum);
		    			if (connect(sockfd,(struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0) 
						fprintf(stderr,"ERROR!\n");
					fdin = sockfd;
					fdout = sockfd;
					isSocket = (0==0);
				}
			}
			else //unknown
			{
				//default to pipes
				if(isSocket)
				{
					close(fdin);
					close(fdout); //needed? or will it cause it to crash??
					isSocket = (1==0);
				}
				fdin = 0;
				fdout = 1;
				fprintf(stderr,"switched to pipes\n");
			}
			struct javaMessage b = javaMessage(CHANGE_BRIDGE_TYPE);
			back = &b;
			back ->inputSize=(jm.inputSize);
			back ->input=(jm.input);
			back->outputSize=(jm.outputSize);
			back->output=(jm.output);
			back->stream=(jm.stream);
			back->streamSize = jm.streamSize;
			back->description=(jm.description);
			back->descSize = jm.descSize;
		}
		else {
			//just send back a generic message
			fprintf(stderr,"sending back generic message\n");
			struct javaMessage b = javaMessage(ABORT);
			back = &b; //just hard coding this back...
			back->inputSize=(jm.inputSize);
		
			back->input=(jm.input);
			back->outputSize=(jm.outputSize);

			back->output=(jm.output);
			back->stream=(jm.stream);
			back->streamSize=jm.streamSize;
			back->description=(jm.description);
			back->descSize=jm.descSize;
		}
		int rsize;
		rsize = byteSize(back);
		fprintf(stderr,"back.size: %d, rsize: %d\n", byteSize(back),rsize);
		char rs[4];
		char* retSize = rs;
		int b = 0;
		char ret[rsize];
		char* returning = ret;
		getBytes(returning,back);
		writeInt(retSize,rsize,&b);

		write(fdout,retSize,4);
		write(fdout,returning,rsize);
	}
}
