/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
/***
 * Modified driver for load testing
 * This driver decodes and returns a DATA message or
 * will execute fibonacci on a START_STOP command
 * 
 * This version supports either sockets or pipes for communication
 * 
 * @author K Ericson
 */

#include "fibWrapper.cpp"
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include "../../../bridge/candcpp/helpers.cpp"
#include <sys/timeb.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

//typedef std::basic_string<unsigned char>u_char_str;

//quick helper...
void reverse(char* toRev)
{
	for(int i =0; i < strlen(toRev)/2; i++)
	{
		std::swap(*(toRev+i),*(toRev +(strlen(toRev)-i-1)));
	}
}

//Timing helpers
long milliDiff(timeb start, timeb stop)
{
	long mstart = start.time * 1000 + start.millitm;
	long mstop = stop.time * 1000 + stop.millitm;
	return mstop - mstart;
}

long microDiff(timeval start, timeval stop)
{
	long sa = start.tv_sec * 1000000 + start.tv_usec;
	long so = stop.tv_sec * 1000000 + stop.tv_usec;
	return so - sa;
}

/***
 * Main driver, as in the Java version, it assumes that the first 4 bytes of communication
 * will always indicate the length of the rest of the message
 *
 * Socket version needs a port number
 */
int main(int argc, char *argv[]){
	bool isSocket;
	char iSize[4]; //grabs the initial int - length of the rest of the message;
	
	fibWrapper* fw;
	javaMessage* back;

	struct sockaddr_in serv_addr;
	struct hostent *server;
	int fdin,fdout; //input file descriptor, output file descriptor

	if(argc >=2){
		//assume sockets
		isSocket = true;
		
		int portNum = atoi(argv[1]);
		int sockfd = socket(AF_INET,SOCK_STREAM,0);
		
		server = gethostbyname("localhost");
		if (server == NULL) {
			fprintf(stderr,"ERROR, no such host\n");
			exit(0);
		}
	 	bzero((char *) &serv_addr, sizeof(serv_addr));
		serv_addr.sin_family = AF_INET;
		bcopy((char *)server->h_addr, 
			(char *)&serv_addr.sin_addr.s_addr,
			server->h_length);
    		serv_addr.sin_port = htons(portNum);
   		if (connect(sockfd,(struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0) 
			std::cerr << "error!" << std::endl;
		fdin = sockfd;
		fdout = sockfd;
	}
	else{ //not sockets
		isSocket = false;
		fdin = 0;		//cin
		fdout = 1;	//cout
	}

	while(true){
		std::cerr << "waiting for input" << std::endl;
		int size = 0;
		int r = 0;
		while (r < 4)
		{
			r = read(fdin,iSize+r,4-r) + r; //grab the size of the message
		}
		size = readInt(iSize);
		//std::cerr << "message size is " << size << " bytes" << std::endl;
		char input[size];
		r = 0;
		while (r < size)
			r = read(fdin,input + r,size-r) + r;


		javaMessage jm = javaMessage(input); //try to initialize


//debugging information
		/*std::cerr<<"Type: " <<jm.getType()<< std::endl <<
			"input length: " << jm.getInputSize() << std::endl <<
			"output length: " << jm.getOutputSize() << std::endl <<
			"stream: " << jm.getStream() << std::endl <<
			"description: " << jm.getDescription() << std::endl <<
			"from java? " << jm.isFromJava() << std::endl;
		*/



		//this section is responsible for initial evaluation of messages, and
		//delegation of tasks to handle a message
		if(jm.getType() == INITIALIZE)
		{
			std::cerr << "initializing!" << std::endl;
			fw = new fibWrapper(jm);
			//send back the status
			std::string stat = fw->getStatus();
			char* o = new char[4 + 4 + stat.length()];//int for type, int for length, actual string
			int start = 0;
			writeInt(o,STRING,start);
			writeInt(o+start,stat.length(),start);
			memcpy(o+start,stat.c_str(),stat.length());
			back = new javaMessage(STATUS);
			back->setInputSize(jm.getInputSize());
			back->setInput(jm.getInput());
			back->setOutputSize(4 + 4 + stat.length());
			back->setOutput(o);
			back->setStream(jm.getStream());
			back->setDescription(jm.getDescription());
		}
		else if(jm.getType() == START_STOP)
		{
			std::string stat = fw->getStatus();
			if(stat == "RUNNING")
			{
				fw->stop();
				return(0);
			}
			else
			{
				//start timing
				struct timeval sta;
				struct timeval sto;
				gettimeofday(&sta,0);
				fw->run(jm);
				gettimeofday(&sto,0);
				long timing = microDiff(sta, sto);
				//stop timing
				//should check that the result was valid here
				back = fw->getResults(timing);
				//send back the original input as well as the output
				std::cerr << "back from fib" << std::endl;
			}
		}
		else if(jm.getType() == DATA)
		{
			//just timing from Java, so just send back a Results
			back = new javaMessage(RESULTS);
			back->setOutput(jm.getInput());
			back->setOutputSize(jm.getInputSize());
			back->setInput(new char[0]);
			back->setInputSize(0);
			back->setStream(jm.getStream());
			back->setDescription(jm.getDescription());
		}
		else if(jm.getType() == CHANGE_BRIDGE_TYPE)
		{
			//switch to the new communication type
			char* inp = jm.getInput();
			int s;
			memcpy(&s,inp,4);
			//Should be an int;
			int btype = readInt(inp+4);
			if(btype == 0)
			{
				//switch to pipes
				//try to close socket:
				if(isSocket)
				{
					close(fdin);
					close(fdout); //needed? or will it cause it to crash??
					isSocket = false;
				}
				fdin = 0;
				fdout = 1;
				std::cerr << "switched to pipes" << std::endl;

			}
			else if (btype == 2)
			{
				if(!isSocket)
				{
					//assuming that port is the next bit of data
					int port = readInt(inp+12);
					//connect
					int portNum = port;
					int sockfd = socket(AF_INET,SOCK_STREAM,0);
				
					server = gethostbyname("localhost");
		   			if (server == NULL) {
		   			     fprintf(stderr,"ERROR, no such host\n");
   					     exit(0);
		  			}
		    			bzero((char *) &serv_addr, sizeof(serv_addr));
    					serv_addr.sin_family = AF_INET;
		    			bcopy((char *)server->h_addr, 
		        		(char *)&serv_addr.sin_addr.s_addr,
		       			  server->h_length);
		     			  serv_addr.sin_port = htons(portNum);
		    			if (connect(sockfd,(struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0) 
						std::cerr << "error!" << std::endl;
					fdin = sockfd;
					fdout = sockfd;
					isSocket = true;
				}
			}
			else //unknown
			{
				//default to pipes
				if(isSocket)
				{
					close(fdin);
					close(fdout); //needed? or will it cause it to crash??
					isSocket = false;
				}
				fdin = 0;
				fdout = 1;
				std::cerr << "switched to pipes" << std::endl;
			}
			back = new javaMessage(CHANGE_BRIDGE_TYPE);
			back ->setInputSize(jm.getInputSize());
			back ->setInput(jm.getInput());
			back->setOutputSize(jm.getOutputSize());
			back->setOutput(jm.getOutput());
			back->setStream(jm.getStream());
			back->setDescription(jm.getDescription());
		}
		else {
			//just send back a generic message
			std::cerr << "sending back generic message" << std::endl;
			back = new javaMessage(ABORT); //just hard coding this back...
			back->setInputSize(jm.getInputSize());
		
			back->setInput(jm.getInput());
			back->setOutputSize(jm.getOutputSize());

			back->setOutput(jm.getOutput());
			back->setStream(jm.getStream());
			back->setDescription(jm.getDescription());
		}
		int rsize = back->byteSize();
		std::cerr << "back.size: " << rsize << std::endl;
		char* retSize = new char[4];
		int b = 0;
		char* returning = new char[rsize];
		back->getBytes(returning);
		writeInt(retSize,rsize,b);

		write(fdout,retSize,4);
		write(fdout,returning,rsize);
	}
}
