/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
/***
 * A wrapper for a simple fibonacci function - this is responsible for handling messages,
 * and controlling the behavior of the function.
 * This is also responsible for keeping track of the state of an execution instance
 *
 * @author K Ericson
 */
#include "fib.c"
#include "../../../bridge/candcpp/javaMessages.c"
#include "../../../bridge/candcpp/constants.h"

struct fibWrapper{
	char* status;
	long results;
	char* attachedStream;
	long input;
	
	//helpers
	int sSize;
	int asSize;

};

void initialize(struct javaMessage jm,struct fibWrapper *fw);

/***
 * Initializes a fibWrapper
 *
 * @param javaMessage
 * 			Requires a message of type INITIALIZE
 */
struct fibWrapper fibWrapper(struct javaMessage jm)
{
	struct fibWrapper f;
	initialize(jm, &f);
	return f;
}


/***
 * Very simple implementation for starting an execution sequence of a fibonacci program.
 * For best performance, this should be threaded.
 *
 * @param javaMessage
 * 			Requires a message of type START_STOP
 */
void run(struct javaMessage jm, struct fibWrapper *fw){
	//assume that I want the input of the java Message
	 char* toParse = jm.input;
	int s;
	memcpy(&s,toParse,4); //get int to see what type the input is
	if (s==LONG) //LONG defined in constants.h 
	{
		long d = readLong(toParse+4);
		fw ->input = d;
		fw ->status = "RUNNING";
		fw ->sSize = 7;
		fw ->results = fib(d);
		fw ->status = "COMPLETED";
		fw ->sSize = 9;
	}
	else
	{
		fw ->status = "FAILED";
		fw ->sSize = 6;
		fw ->results = -1;
	}
}

void initialize(struct javaMessage jm,struct fibWrapper *fw) {
	fw ->attachedStream = jm.stream;
	fw ->asSize = jm.streamSize;
	fw ->status = "INITIALIZED";
	fw ->sSize = 11;
}


//not really effective until threading is implemented

void stop(struct fibWrapper *fw) {
	
}
/*
void abort(struct fibWrapper *fw) {

}*/

/***
 * Overloaded version of the function to return the result of the last run
 * This version takes a long, and appends it to the output
 * after the result of the last run.
 */
struct javaMessage getTimingResults(long t, struct fibWrapper *fw) {
	struct javaMessage j = javaMessage(RESULTS); //from constants.h
	struct javaMessage* jm = &j;
	char o[4*2 + 8 * 2];
	char *ot = o; //2 ints and 2 longs
	int start = 0;
	writeInt(ot,LONG,&start);
	fprintf(stderr, "results: %d\n",fw->results);
	writeLong(ot+start,fw->results,&start);
	writeInt(ot+start,LONG,&start);
	writeLong(ot+start,t,&start);
	jm->output = (char*)malloc(sizeof(char)*(4*2+8*2));
	memcpy(jm->output,ot,4*2+8*2);
	char i[4+8];
	 char* it = i;
	start = 0;
	writeInt(it,LONG,&start);
	writeLong(it+start,fw->input,&start);
	jm->input = (it);
	jm->stream = (fw->attachedStream);
	jm->description = (fw->status); //does it make more sense for this to be send as output?
	jm -> descSize = fw ->asSize;
	jm->inputSize = 4+8;
	jm->outputSize = 4*2+8*2;
	jm->streamSize = fw -> sSize;
	return j;
}


/***
 * Generates a <code>javaMessage</code> to return the result of the last run
 */
struct javaMessage getResults(struct fibWrapper *fw) {
	struct javaMessage j = javaMessage(RESULTS); //from constants.h
	struct javaMessage *jm = &j;
	
	char t[12];
	char*b = t;
	int start = 0;
	writeInt(b+start,LONG,&start);
	writeLong(b+start,fw->results,&start);

	jm->output = (char*)malloc(sizeof(char)*12);
	memcpy(jm->output,b,12);

	char i[4+8];
	char* it = i;
	start = 0;
	writeInt(it,LONG,&start);
	writeLong(it+start,fw->input,&start);
	jm->input=(it);
	jm->stream = fw->attachedStream;
	jm -> streamSize = fw ->asSize;
	jm->description = fw ->status; //does it make more sense for this to be send as output?
	jm -> descSize = fw -> sSize;
	jm->inputSize = (4+8);
	jm->outputSize = (4+8);
	return j;
}
