/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
/***
 * A wrapper for a simple fibonacci function - this is responsible for handling messages,
 * and controlling the behavior of the function.
 * This is also responsible for keeping track of the state of an execution instance
 *
 * @author K Ericson
 */

#include <iostream>
#include <string>
#include "fib.cpp"
#include "../../../bridge/candcpp/javaMessages.cpp"
#include "../../../bridge/candcpp/constants.h"
#include "../../../bridge/candcpp/wrapper.cpp"

class fibWrapper :wrapper {
public:
	std::string getStatus();
	void run(javaMessage); //grabs input from the javaMessage
	void stop(); //figures out which one to kill based off of 
	void abort();
	javaMessage* getResults();
	javaMessage* getResults(long);
	void initialize(javaMessage);
	fibWrapper(javaMessage);

private:
	std::string status;
	long results;
	std::string attachedStream;
	long input;

};

/***
 * Initializes a fibWrapper
 *
 * @param javaMessage
 * 			Requires a message of type INITIALIZE
 */
fibWrapper::fibWrapper(javaMessage jm)
{
	initialize(jm);
}

std::string fibWrapper::getStatus(){
	return status;
}

/***
 * Very simple implementation for starting an execution sequence of a fibonacci program.
 * For best performance, this should be threaded.
 *
 * @param javaMessage
 * 			Requires a message of type START_STOP
 */
void fibWrapper::run(javaMessage jm){
	//assume that I want the input of the java Message
	 char* toParse = jm.getInput();
	int s;
	std::cerr << "starting!" << std::endl;
	memcpy(&s,toParse,4); //get int to see what type the input is
	if (s==LONG) //LONG defined in constants.h 
	{
		long d = readLong(toParse+4);
		std::cerr << "result: " << d << std::endl;
		input = d;
		status = "RUNNING";
		results = fib(d);
		status = "COMPLETED";
	}
	else
	{
		status = "FAILED";
		std::cerr << "INVALID INPUT" << std::endl;
		results = -1;
	}
}

void fibWrapper::initialize(javaMessage jm) {
	attachedStream = jm.getStream();
	status = "INITIALIZED";
}


//not really effective until threading is implemented

void fibWrapper::stop() {
	
}

void fibWrapper::abort() {

}

/***
 * Overloaded version of the function to return the result of the last run
 * This version takes a long, and appends it to the output
 * after the result of the last run.
 */
javaMessage* fibWrapper::getResults(long t) {
javaMessage* jm = new javaMessage(RESULTS); //from constants.h
	char *ot = new char[4*2 + 8*2]; //2 ints and 2 longs
	int start = 0;
	writeInt(ot,LONG,start);
	writeLong(ot+start,results,start);
	writeInt(ot+start,LONG,start);
	writeLong(ot+start,t,start);
	jm->setOutput(ot);
	 char* it = new char[4+8];
	start = 0;
	writeInt(it,LONG,start);
	writeLong(it+start,input,start);
	jm->setInput(it);
	jm->setStream(attachedStream);
	jm->setDescription(status); //does it make more sense for this to be send as output?
	jm->setInputSize(4+8);
	jm->setOutputSize(4*2+8*2);
	return jm;
}


/***
 * Generates a <code>javaMessage</code> to return the result of the last run
 */
javaMessage* fibWrapper::getResults() {
	javaMessage* jm = new javaMessage(RESULTS); //from constants.h
	char *ot = new char[4 + 8];
	int start = 0;
	writeInt(ot,LONG,start);
	writeLong(ot+start,results,start);
	jm->setOutput(ot);
	 char* it = new char[4+8];
	start = 0;
	writeInt(it,LONG,start);
	writeLong(it+start,input,start);
	jm->setInput(it);
	jm->setStream(attachedStream);
	jm->setDescription(status); //does it make more sense for this to be send as output?
	jm->setInputSize(4+8);
	jm->setOutputSize(4+8);
	return jm;
}
