﻿/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;
using System.Timers;
using System.Net.Sockets;

namespace csBridge
{
    /*
     * This class is responsible for handling actual communication between
     * Java and the fibonacci program
     */
    class fibController
    {
        static void Main(string [] args)
        {
            System.Diagnostics.Stopwatch watch = new System.Diagnostics.Stopwatch();
            BinaryReader b;
            BinaryWriter bw;
            if (args.Length < 1)
            { //should use pipes

                Stream st = Console.OpenStandardInput();
                b = new BinaryReader(st);
                Stream sto = Console.OpenStandardOutput();
                bw = new BinaryWriter(sto);
            }
            else
            { //it's tcp sockets
                Console.Error.WriteLine("Using Sockets");
                Console.Error.Flush();
                int saddr = Convert.ToInt32(args[0]);
                TcpClient tcp = new TcpClient("localhost",saddr);
                NetworkStream ns = tcp.GetStream();
                b = new BinaryReader(ns);
                bw = new BinaryWriter(ns);
            }

            JavaMessage back, jm;
            fibWrapper fw = null;

            bool cont = true;



            while (cont)
            {
                Console.Error.WriteLine("Wating for input");
                Console.Error.Flush();

                int sz = b.ReadInt32();
                sz = IPAddress.NetworkToHostOrder(sz);

                //byte[] msg = br.ReadBytes(size);
                byte[] msg = b.ReadBytes(sz);
                jm = new JavaMessage(msg);



                int type = jm.getMessageType();
                switch (type)
                {
                    case ControlMessage.INITIALIZE:
                        Console.Error.WriteLine("initializing");
                        Console.Error.Flush();
                        fw = new fibWrapper(jm);
                        //prepare a status message to send back
                        back = genStatusMessage(fw, jm);
                        break;
                    case ControlMessage.START_STOP:
                        Console.Error.WriteLine("START_STOP command");
                        string stat = fw.getStatus();
                        if (stat.Equals("RUNNING"))
                        {
                            //not threaded, so shouldn't ever be hit
                            fw.stop();
                            cont = false;
                            back = genStatusMessage(fw, jm);
                        }
                        else
                        {
                            //Start a run
                            watch.Start();
                            fw.run(jm);
                            watch.Stop();
                            TimeSpan t = watch.Elapsed;
                            long time = (long)(t.TotalMilliseconds / 1000); //to microseconds
                            watch.Reset();
                            //If this was fully threaded, this would not be included here
                            back = fw.getResults(time);
                            Console.Error.WriteLine("Run completed");
                            Console.Error.Flush();
                        }
                        break;
                    case ControlMessage.DATA:
                        Console.Error.WriteLine("DATA command");
                        back = new JavaMessage(ControlMessage.RESULTS);
                        back.setOutput(jm.getInput());
                        back.setInput(new byte[0]);
                        back.setStreamID(jm.getStreamID());
                        back.setDescription(jm.getDescription());
                        break;
                    default:
                        //just send back a generic message
                        back = genStatusMessage(fw, jm);
                        break;
                }
                byte[] m2 = back.toByteArray();
                int goingOut = IPAddress.HostToNetworkOrder(m2.Length);
                bw.Write(goingOut);
                //bw.Write(m2.Length);
                bw.Write(m2);
                //bw.Write(m2);
                bw.Flush();
            }

        }

        private static JavaMessage genStatusMessage(fibWrapper fw, JavaMessage incoming)
        {
            JavaMessage toRet = new JavaMessage(ControlMessage.STATUS);
            string st = fw.getStatus();
            MemoryStream ms = new MemoryStream();
            BinaryWriter bw = new BinaryWriter(ms);

            bw.Write(IPAddress.HostToNetworkOrder(ControlMessage.STRING));
            byte[] s = System.Text.UTF8Encoding.UTF8.GetBytes(st);
            bw.Write(IPAddress.HostToNetworkOrder(s.Length));
            bw.Write(s);
            bw.Flush();
            toRet.setOutput(ms.ToArray());
            toRet.setInput(incoming.getInput());
            toRet.setStreamID(incoming.getStreamID());
            toRet.setDescription(incoming.getDescription());

            return toRet;
        }
    }
}
