﻿/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;

namespace csBridge
{
    /*
     * Wrapper specific for the Fibonacci example,
     * with a few modifications could be used for other examples
     */
    class fibWrapper : Wrapper
    {
        private string status;
        private long results;
        private string attachedStream;
        long input;

        /**
         * Initializes a fibWrapper
         * @param javaMessage
         *      Requires a message of type INITIALIZE
         */
        public fibWrapper(JavaMessage jm)
        {
            initialize(jm);
        }

        public string getStatus()
        {
            return status;
        }

        /**
         * Very simple implementation for starting an execution sequence of a fibonacci program
         * For best performance, this should be threaded
         * 
         * @param javaMessage
         *      Requires a message of type START_STOP
         */
        public void run(JavaMessage j)
        {
            //assuming that we're grabbing input
            byte[] incoming = j.getInput();
            System.IO.MemoryStream m = new System.IO.MemoryStream(incoming);
            System.IO.BinaryReader reader = new System.IO.BinaryReader(m);
            int type = IPAddress.NetworkToHostOrder(reader.ReadInt32());
            if (type == ControlMessage.LONG) //only continue if there's a long there
            {
                long d = IPAddress.NetworkToHostOrder(reader.ReadInt64());
                input = d;
                status = "RUNNING";
                fibonacci f = new fibonacci(); //may be better to have this outside here
                results = f.fib(input);
                status = "COMPLETED";
            }
            else
            {
                status = "FAILED";
                results = -1;
            }
        }

        /**
         * Invalid until threading is implemented
         */
        public void stop()
        {
            throw new NotImplementedException();
        }

        public void abort()
        {
            throw new NotImplementedException();
        }

        /**
         * Generates a javaMessage to return the result of the last run
         */
        public JavaMessage getResults()
        {
            JavaMessage jm = new JavaMessage(ControlMessage.RESULTS);
            byte[] rType = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(ControlMessage.LONG));
            byte[] res = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(results));
            byte[] fullRes = new byte[rType.Length + res.Length];
            Array.Copy(rType,0,fullRes,0,rType.Length);
            Array.Copy(res,0,fullRes,rType.Length,res.Length);
          
            jm.setOutput(fullRes);

            byte[] iType = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(ControlMessage.LONG));
            byte[] inp = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(input));
            byte[] fullInp = new byte[iType.Length + inp.Length];
            Array.Copy(iType, 0, fullInp, 0, iType.Length);
            Array.Copy(inp, 0, fullInp, iType.Length, inp.Length);

            jm.setInput(fullInp);

            jm.setStreamID(attachedStream);
            jm.setDescription(status);

            return jm;
        }

        public void initialize(JavaMessage j)
        {
            attachedStream = j.getStreamID();
            status = "INITIALIZED";
        }

        internal JavaMessage getResults(long time)
        {
            JavaMessage jm = new JavaMessage(ControlMessage.RESULTS);
            byte[] rType = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(ControlMessage.LONG));
            byte[] res = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(results));
            byte[] t = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(time));
            byte[] fullRes = new byte[rType.Length*2 + res.Length + t.Length];
            Array.Copy(rType, 0, fullRes, 0, rType.Length);
            Array.Copy(res, 0, fullRes, rType.Length, res.Length);
            Array.Copy(rType, 0, fullRes, rType.Length + res.Length, rType.Length);
            Array.Copy(t, 0, fullRes, rType.Length * 2 + res.Length, t.Length);

            jm.setOutput(fullRes);

            byte[] iType = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(ControlMessage.LONG));
            byte[] inp = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(input));
            byte[] fullInp = new byte[iType.Length + inp.Length];
            Array.Copy(iType, 0, fullInp, 0, iType.Length);
            Array.Copy(inp, 0, fullInp, iType.Length, inp.Length);

            jm.setInput(fullInp);

            jm.setStreamID(attachedStream);
            jm.setDescription(status);

            return jm;
        }
    }
}
