package ds.granules.samples.checksum;

import java.math.BigInteger;

import java.util.Properties;

import cgl.narada.event.NBEvent;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;

import ds.granules.util.Checksum;

public class ChecksumMap extends MapReduceBase {

    private Checksum SHA1;

    int messagesReceived = 0;
    int goodMessages = 0;
    int badMessages = 0;

    public ChecksumMap() {
        setDomain("Generic");
        setVersionInformation(200);
        setOperationIdentifier("ChecksumID");
        setInitializationNeeded(true);
    }

    @Override
    public void initialize(Properties processingDirectives) {
        try {
            SHA1 = new Checksum("SHA1");
        } catch (Exception e) {
            System.out.println("Could not initialize Checksum utility!");
        }
        setInitializationComplete();
    }

    @Override
    public void execute() throws ProcessingException {

        String datasetIdentifier = "ChecksumDataset";
        DatasetCollection datasetCollection = getDatasetCollection();
        StreamingAccess access = null;

        try {
            Dataset dataset = datasetCollection.getDataset(datasetIdentifier);

            if (dataset.getDatasetType() == Dataset.STREAMS) {
                access = (StreamingAccess) dataset;
            } else {
                System.out.println("Incorrect datatset: "
                        + dataset.getDatasetType() +
                        " initialized. Returning ...");
                return;
            }

            if (access != null) {
                while (dataset.isDataAvailable()) {
                    NBEvent nbEvent = access.getStreamingData();
                    processStreamPacket(nbEvent);
                }
            }

        } catch (DatasetException e) {
            e.printStackTrace();

        } catch (MapReduceException e) {
            System.out.println("Problems writing Results: " + e.getMessage());
            e.printStackTrace();
        }
    }


    private void processStreamPacket(NBEvent nbEvent)
    throws MapReduceException {
        if (nbEvent == null) {
            return;
        }

        byte[] message = nbEvent.getContentPayload();
        byte[] remoteChecksum = new byte[20];
        byte[] payload = new byte[message.length - 20];
        System.arraycopy(message, 0, remoteChecksum, 0, 20);
        System.arraycopy(message, 20, payload, 0, payload.length);

        /* The server-side checksum for the payload */
        byte[] localChecksum = SHA1.hash(payload);

        BigInteger remoteVal = new BigInteger(1, remoteChecksum);
        BigInteger localVal = new BigInteger(1, localChecksum);

        if (remoteVal.equals(localVal)) {
            goodMessages++;
        } else {
            badMessages++;
            System.out.println(localVal.toString(16) + " =/= " +
                    remoteVal.toString(16));
        }

        /* Print a summary every 10,000 messages */
        messagesReceived++;
        if (messagesReceived % 10000 == 0) {
            System.out.println("Stats: " + goodMessages + " good, " +
                    badMessages + " corrupted");
        }
    }
}
