package ds.granules.samples.checksum;

import java.util.Properties;
import java.util.Random;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.EventProducer;
import cgl.narada.service.client.SessionService;

import ds.granules.util.Checksum;

public class Client {

    private int entityId;

    private ClientService clientService;

    private EventProducer producer;

    public Client() throws ServiceException {
        entityId = new Random().nextInt();
        clientService = SessionService.getClientService(entityId);
    }

    public void initializeBrokerCommunications(
            Properties props, String commType)
    throws ServiceException {
        clientService.initializeBrokerCommunications(props, commType);
    }

    public void closeBrokerConnection() throws ServiceException {
        clientService.closeBrokerConnection();
        clientService.terminateServices();
        try {
            /* Make sure that the transmission tracker is shutdown
             * gracefully. */
            Thread.sleep(100);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void initializeProducer() throws ServiceException {
        producer = clientService.createEventProducer();

        producer.generateEventIdentifier(true);
        producer.setTemplateId(12345);
        producer.setDisableTimestamp(false);

        producer.setSuppressRedistributionToSource(true);
    }

    public void publishEvent(byte[] transferBytes) throws ServiceException,
        NBEventException {
        NBEvent nbEvent =
            producer.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                   "WorkerStreams/Checksum", transferBytes);
        producer.publishEvent(nbEvent);
    }

    public static void main(String[] args)
    throws Exception {
        if (args.length < 3) {
            String name = "ds.granules.samples.checksum.Client";
            String usage = "Usage: " + name + " <hostname> <port> <# messages>";

            System.out.println(usage);
            System.exit(1);
        }

        Properties props = new Properties();
        props.put("hostname", args[0]);
        props.put("portnum", args[1]);
        int numMessages = Integer.parseInt(args[2]);

        Client client = new Client();
        client.initializeBrokerCommunications(props, "niotcp");
        client.initializeProducer();


        Checksum SHA1 = new Checksum("SHA1");
        Random randomGenerator = new Random(System.nanoTime());

        for (int i = 0; i < numMessages; ++i) {
            int payloadSize = 8192;
            byte[] payload = new byte[payloadSize];
            randomGenerator.nextBytes(payload);

            byte[] checksum = SHA1.hash(payload);

            byte[] message = new byte[checksum.length + payload.length];
            System.arraycopy(checksum, 0, message, 0, checksum.length);
            System.arraycopy(payload, 0, message,
                    checksum.length, payload.length);

            System.out.println(SHA1.hashToHexString(checksum));

            client.publishEvent(message);
        }

        client.closeBrokerConnection();
    }
}
