/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.flip;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Properties;
import java.util.Random;

import cgl.narada.event.EventProperties;
import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import cgl.narada.matching.Profile;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.EventConsumer;
import cgl.narada.service.client.EventProducer;
import cgl.narada.service.client.NBEventListener;
import cgl.narada.service.client.SessionService;
import ds.granules.exception.MarshallingException;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

public class LoadGenerator implements NBEventListener {

    private int entityId;

    private ClientService clientService;

    private String moduleName = "SimpleClient: ";

    private EventConsumer consumer;

    private EventProducer producer;

    private Profile profile;

    private boolean addEventProperty = false;

    private String topic = "WorkerStream/Result";

    private int tracker = 0;


    public LoadGenerator() throws ServiceException {
        entityId = new Random().nextInt();
        clientService = SessionService.getClientService(entityId);
    }


    public void initializeBrokerCommunications(Properties props, String commType)
        throws ServiceException {
        clientService.initializeBrokerCommunications(props, commType);
    }


    /**
     * Closes the connection to the broker.
     * 
     * @throws ServiceException
     */
    public void closeBrokerConnection() throws ServiceException {
        unsubscribe();
        clientService.closeBrokerConnection();
        clientService.terminateServices();
        try {
            /** Make sure that the transmission tracker is shutdown gracefully. */
            Thread.sleep(100);
        } catch (InterruptedException e) {
            System.out.println(moduleName
                + "Problems sleeping while waiting for transfer completion");
        }
    }


    public void initializeProducer() throws ServiceException {
        producer = clientService.createEventProducer();

        producer.generateEventIdentifier(true);
        producer.setTemplateId(12345);
        producer.setDisableTimestamp(false);

        producer.setSuppressRedistributionToSource(true);
    }


    public void initializeConsumer() throws ServiceException {
        profile =
            clientService.createProfile(TemplateProfileAndSynopsisTypes.STRING,
                                        topic);
        consumer = clientService.createEventConsumer(this);
        consumer.subscribeTo(profile);
    }


    public void unsubscribe() throws ServiceException {
        consumer.unSubscribe(profile);
    }


    public void publishEvent(String stringMsg) throws ServiceException,
        NBEventException {
        if (stringMsg.equals("")) {
            stringMsg += System.currentTimeMillis();
        }

        String publishTopic = "WorkerStreams/Flip";
        NBEvent nbEvent =
            producer.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                   publishTopic, stringMsg.getBytes());

        if (addEventProperty) {
            EventProperties properties = nbEvent.getEventProperties();
            properties.setMutableProperty("Tennis", "US Open",
                                          new Integer(entityId));
        }
        producer.publishEvent(nbEvent);
    }


    public void publishEvent(byte[] transferBytes) throws ServiceException,
        NBEventException {
        NBEvent nbEvent =
            producer.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                   topic, transferBytes);
        producer.publishEvent(nbEvent);
    }


    public void onEvent(NBEvent nbEvent) {
        String synopsis = (String) nbEvent.getContentSynopsis();
        System.out.println(moduleName + "Received results on {" + synopsis
            + "}");

        ResultsFactory resultsFactory = ResultsFactory.getInstance();
        try {
            Results results =
                resultsFactory.getResults(nbEvent.getContentPayload());
            System.out.println(new String(results.getResultPayload()));
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (MarshallingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }


    public static void main(String[] args) {
        String module = "SimpleClient.main() ->";
        String transport = "niotcp";
        if (args.length < 2) {
            String errorReport =
                module + "The Correct arguments are \n"
                    + "java ds.granules.samples.flip.LoadGenerator <hostname>"
                    + "<portnum>";
            System.out.println(errorReport);
            System.exit(0);
        }

        Properties props = new Properties();

        /** These properties pertain to setting up a TCP link */
        props.put("hostname", args[0]);
        props.put("portnum", args[1]);

        String helpInfo =
            "\nSimpleClient is meant to demonstrate some basic "
                + "capabilities. The various options currently available include\n"
                + "ANY TEXT : If you start typing followed by a carriage return, "
                + " a message is sent\n"
                + "u        : Perform an unsubscribe operation.\n"
                + "i        : Initialize consumer.\n"
                + "p        : Toggle for the generation of event properties"
                + "h        : Displays HELP options\n"
                + "q        : Close broker connections and quit the system\n";

        try {
            LoadGenerator client = new LoadGenerator();
            /** Initializing communications using specified transport */
            client.initializeBrokerCommunications(props, transport);
            client.initializeProducer();
            client.initializeConsumer();

            InputStreamReader isr = new InputStreamReader(System.in);
            BufferedReader commandLine = new BufferedReader(isr);

            System.out.println(helpInfo);
            int tracker = 0;
            while (true) {
                String s = commandLine.readLine();
                if (s == null) {
                    System.out.println(module + "String is null!!!");
                    break;
                }

                if (s.equals("h")) {
                    System.out.println(helpInfo);
                    continue;
                }

                if (s.equals("q")) {
                    client.closeBrokerConnection();
                    System.out.print(module + "Closed Broker Connection");
                    break;
                }

                if (s.equals("u")) {
                    client.unsubscribe();
                    continue;
                }

                if (s.equals("s")) {
                    client.initializeConsumer();
                    continue;
                }

                tracker++;
                client.publishEvent(s + " [" + tracker + "]");

            }
            /** end while-true */
            System.out.println(module + "exiting ");

        } catch (Exception e) {
            System.out.println(module + e.getMessage());
            e.printStackTrace();
        }

    }
}
