package ds.granules.samples.kmeans;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;

/**
 * A widget to hold vector data.
 * 
 * @author Jaliya Ekanayake
 * 
 */
public class KMVectorData implements Serializable {

    private static final long serialVersionUID = 6379328560718847730L;

    int numVectors;

    int vecLen;

    double[][] data;


    public KMVectorData() {
        super();
    }


    public KMVectorData(int numCentroids, int vecLen, double[][] data) {
        super();
        this.numVectors = numCentroids;
        this.vecLen = vecLen;
        this.data = data;
    }


    public double[][] getData() {
        return data;
    }


    public void setData(double[][] data) {
        this.data = data;
    }


    public int getNumVectors() {
        return numVectors;
    }


    public void setNumVectors(int numCentroids) {
        this.numVectors = numCentroids;
    }


    public int getVecLen() {
        return vecLen;
    }


    public void setVecLen(int vecLen) {
        this.vecLen = vecLen;
    }


    /**
     * Read data from a given file. The data file is a text file with each line
     * representing a single vector of double values. Space character is used to
     * separate double values of a given vector dimensions.
     * 
     * @param fileName
     * @return
     * @throws IOException
     */
    public KMVectorData(String fileName) throws IOException {

        File file = new File(fileName);
        BufferedReader reader = new BufferedReader(new FileReader(file));
        int numRecords = 0;
        String inputLine = null;
        while ((inputLine = reader.readLine()) != null) {
            numRecords++;
        }
        reader.close();
        if (numRecords > 0) {
            reader = new BufferedReader(new FileReader(file));
            inputLine = reader.readLine();
        }
        String[] vectorValues = inputLine.split(" ");
        this.data = new double[numRecords][vectorValues.length];
        reader.close();
        this.numVectors = numRecords;
        this.vecLen = vectorValues.length;

        reader = new BufferedReader(new FileReader(file));

        numRecords = 0;
        while ((inputLine = reader.readLine()) != null) {
            vectorValues = inputLine.split(" ");
            for (int i = 0; i < vectorValues.length; i++) {
                data[numRecords][i] = Double.valueOf(vectorValues[i]);
            }
            numRecords++;
        }

    }

}
