/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.periodic;

import java.util.Properties;

import ds.granules.operation.MapReduceBase;

/**
 * An example demonstrating support for periodicity. This particular operation
 * will be scheduled for execution at the specified periodic interval.
 * 
 * @author Shrideep Pallickara
 * 
 */
public class PeriodicMap extends MapReduceBase {

    private int iteration = 0;
    private int terminateAtIteration = 20;
    private String workerType = null;
    private long toggleInterval = 1000;
    private long[] delays = new long[terminateAtIteration];
    private long lastExecutionTime = 0;
    private boolean debug = true;


    public PeriodicMap() {
        setDomain("Generic");
        setVersionInformation(200);
        setOperationIdentifier("WCPeriodic-MapFunction-GranulesSample");
        lastExecutionTime = System.currentTimeMillis();
    }


    public void execute() {
        delays[iteration] = System.currentTimeMillis() - lastExecutionTime;
        iteration++;
        if (workerType == null) {
            String property = "Worker Number";
            Properties props = getProcessingDirectives();
            if (props.containsKey(property)) {
                workerType = props.getProperty(property);
            } else {
                workerType = "[Unknown]";
            }
        }

        performSomeFunction();

        if (iteration > 0) {
            if (iteration == terminateAtIteration) {
                String report = workerType + " ";
                for (int i = 0; i < terminateAtIteration; i++) {
                    report += (delays[i] + " ");
                }

                System.out.println(report);
                setTeminationConditionReached();
            }
            return;
        }
    }


    /** Performs a very simple operation. */
    private void performSomeFunction() {
        if (debug) {
            System.out.println("PeriodicMap: " + workerType + ",   Iteration ("
                + iteration + ")");
            changePeriodicity();
        }
    }


    /** Change the periodicity associated with the operation at runtime. */
    private void changePeriodicity() {
        if (iteration == 15) {
            toggleInterval = 4000;
            changeToExecutePeriodically(toggleInterval);
        }
    }

}
