/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.pythonColl;

/***
 * Basic utility to handle bridge between C++ and Java.
 * This class is responsible for setting up the unnamed pipes, and starting the C++ application.
 * It is also responsible for starting up the threads to handle communication with the C++ application. 
 * 
 * @author K Ericson
 */

import java.io.IOException;
import java.net.Socket;

import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.bridge.StreamReader;
import ds.granules.bridge.StreamStringReader;
import ds.granules.bridge.StreamWriter;

public class JavaPythonCommunication {
    Process p;
    String cProg;
    Runtime r;
    StreamReader sr;
    StreamWriter sw;
    StreamStringReader se;
    int port;


    /***
     * Initializes and returns a reference to a JavaPythonComputation. Attaches
     * a <code>StreamReader</code>, <code>StreamWriter</code>, and
     * <code>StreamStringReader</code> to the programs standard in, out, and
     * error respectively.
     * 
     * @param program
     *            <code>String</code> identifying the C++ program to be run
     * @param cb
     *            <code>JavaMessageHandler</code> which is needed by the
     *            <code>StreamReader</code> instantiated here
     * @throws IOException
     *             If there is a problem starting up the program, or connecting
     *             the unnamed pipes to the program
     */
    public JavaPythonCommunication(String program, JavaMessageHandler cb)
        throws IOException {
        cProg = program;
        r = Runtime.getRuntime();
        p = r.exec("nice -n +19 python " + cProg);
        sr = new StreamReader(p.getInputStream(), cb);
        sw = new StreamWriter(p.getOutputStream());
        se = new StreamStringReader(p.getErrorStream());
    }


    public JavaPythonCommunication(String program, JavaMessageHandler cb,
            int pIn) throws IOException {
        port = pIn;

        cProg = program;
        r = Runtime.getRuntime();
        p = r.exec("python " + cProg); // "nice -n +19 - to make sna happy

        Socket incoming = new Socket("localhost", port); // safe to assume that
                                                         // it's just going to
                                                         // be localhost

        sr = new StreamReader(incoming.getInputStream(), cb);
        sw = new StreamWriter(incoming.getOutputStream());
        se = null; // don't know if I can handle this..

    }


    public void setSocketPort(int o) {
        port = o;
    }


    public void sendMessage(JavaByteMessage j) {
        sw.sendMessage(j.toByteArray());
    }

}
