/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.pythonColl;

/***
 * This is the mapper for the fibonacci c++ application.
 * It uses unnamed pipes to bridge communications.
 * 
 * @author K Ericson
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Properties;

import cgl.narada.event.NBEvent;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.MapReduceException;
import ds.granules.operation.ProcessingException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;

public class PyCollMap extends MapReduceBase implements JavaMessageHandler {

    JavaPythonCommunication jc;
    private String streamID;
    private boolean reducerReady;
    long microtime; // most efficient place to put it...
    private JavaByteMessage lastSSmsg;


    public PyCollMap() throws IOException {
        setDomain("Generic");
        setVersionInformation(200);
        setOperationIdentifier("ABCDEFGHIJKLMN");
        reducerReady = false;
    }


    /**
     * Simple function to actually initialize the c++ program
     */
    public void pyCollInit() {
        System.out.println("Initializing");
        Properties props = getProcessingDirectives();
        if (!props.containsKey("Program")) {
            System.out.println("Can't get program!");
            System.exit(-1);
        }
        String pyProg = props.getProperty("Program");
        String workerNum = props.getProperty("Worker Number");
        System.out.println("WorkerNum before: " + workerNum);
        String numOnly = workerNum.substring(1, workerNum.length() - 1); // cut
                                                                         // off
                                                                         // leading
                                                                         // and
                                                                         // trailing
                                                                         // "()"
        System.out.println("WorkerNum after: " + numOnly);
        int k = Integer.parseInt(numOnly);
        int nMaps = Integer.parseInt(props.getProperty("Number of Workers"));
        try {
            jc = new JavaPythonCommunication(pyProg, this);
        } catch (IOException e) {
            System.out.println("Couldn't get file");
            e.printStackTrace();
        }
        ResultPayload rp = ResultPayload.getInstance();
        try {
            byte[] ind = rp.getBytes(k);
            byte[] numMaps = rp.getBytes(nMaps);
            byte[] isInt = rp.getBytes(ControlMessage.INT);
            byte[] input =
                new byte[isInt.length * 2 + numMaps.length + ind.length];
            System.arraycopy(isInt, 0, input, 0, isInt.length);
            System.arraycopy(ind, 0, input, isInt.length, ind.length);
            System.arraycopy(isInt, 0, input, isInt.length + ind.length,
                             isInt.length);
            System.arraycopy(numMaps, 0, input, isInt.length * 2 + ind.length,
                             numMaps.length);

            streamID = "Worker Number: " + workerNum;

            JavaByteMessage jm = new JavaByteMessage(ControlMessage.INITIALIZE);
            jm.setInput(input);
            jm.setStreamIdentifier(streamID); // need to grab stream ID
            jm.setDescription("initializing"); // figure out what this should be
                                               // again
            System.out.println("Sending: " + jm.toString());
            jc.sendMessage(jm);
        } catch (MarshallingException e) {
            System.out.println("Marshalling error: " + e.getMessage());
            e.printStackTrace();
        }
    }


    @Override
    public void execute() throws ProcessingException {
        // moving check for null up here...
        if (jc == null) {
            System.out.println("need to initialize");
            pyCollInit();
        }

        String datasetIdentifier = "PyCollStreamDatasetID";

        System.out.println("Executing...");
        DatasetCollection datasetCollection = getDatasetCollection();
        StreamingAccess access = null;
        try {
            Dataset dataset = datasetCollection.getDataset(datasetIdentifier);

            if (dataset.getDatasetType() == Dataset.STREAMS) {
                access = (StreamingAccess) dataset;
            } else {
                System.out.println("Incorrect datatset: "
                    + dataset.getDatasetType() + " initialized. Returning ...");
                return;
            }

            if (access != null) {

                while (dataset.isDataAvailable()) {
                    NBEvent nbEvent = access.getStreamingData();
                    processStreamPacket(nbEvent);
                }

            } else {
                System.out.println("\nThe INPUT Dataset is NULL!\n");
            }

        } catch (DatasetException e) {
            e.printStackTrace();

        } catch (MapReduceException e) {
            System.out.println("Problems writing Results: " + e.getMessage());
            e.printStackTrace();
        } catch (MarshallingException e) {
            System.out.println("Problems marshalling input: " + e.getMessage());
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("IO Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }


    private void processStreamPacket(NBEvent nbEvent)
        throws MapReduceException, MarshallingException, IOException {
        // first check to see if the program hasn't been initialized yet:
        ResultPayload resultPayload;
        resultPayload = ResultPayload.getInstance();

        System.out.println("Processing stream fragment: " + nbEvent);
        if (nbEvent == null) {
            return;
        }
        System.out.println("Received Stream Packet-> "
            + nbEvent.getContentSynopsis());
        byte[] payload = nbEvent.getContentPayload();

        System.out.println("Got content Payload");

        int s = nbEvent.getContentPayloadSize();
        System.out.println("Content payload size: " + s);

        // byte[] actual = rp.

        // a simple one-shot: all the data will be there
        // assuming payload will be a list of the needed data:
        // String filename (int size, string)
        // int width
        // int height
        // int zoomMult

        System.out.println("Building streams...");

        ByteArrayInputStream is = new ByteArrayInputStream(payload);
        DataInputStream dis = new DataInputStream(is);

        System.out.println("Done Building streams");

        int sSize = dis.readInt();
        System.out.println("String size: " + sSize);
        byte[] fname = new byte[sSize];
        dis.read(fname, 0, sSize);

        String filename = new String(fname);
        System.out.println("File name: " + fname);
        int width = dis.readInt();
        int height = dis.readInt();
        int zoomMult = dis.readInt();

        System.out.println("Got file, generating new javaMessage");

        ByteArrayOutputStream os = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(os);

        dos.writeInt(ControlMessage.STRING);
        dos.writeInt(fname.length);
        dos.writeBytes(filename);
        dos.writeInt(ControlMessage.INT);
        dos.writeInt(width);
        dos.writeInt(ControlMessage.INT);
        dos.writeInt(height);
        dos.writeInt(ControlMessage.INT);
        dos.writeInt(zoomMult);

        dos.flush();

        System.out.println("Working with file: " + filename);
        // send out a new message here
        lastSSmsg = new JavaByteMessage(ControlMessage.START_STOP);
        lastSSmsg.setStreamIdentifier(streamID);
        lastSSmsg.setDescription("beginning a run");
        lastSSmsg.setInput(os.toByteArray());
        System.out.println("Done Building START_STOP message...");
        sendSTART_STOP();

    }


    private void sendSTART_STOP() {
        microtime = System.currentTimeMillis();
        jc.sendMessage(lastSSmsg);
    }


    @Override
    public void handleMessage(byte[] msg) {
        // fudging things to make asynchronous
        final byte[] par = msg;
        new Thread(new Runnable() {
            public void run() {
                messageSwitch(par);
            }
        }).start();
    }


    private void messageSwitch(byte[] msg) {
        ResultPayload resultPayload;
        resultPayload = ResultPayload.getInstance();
        JavaByteMessage jm = new JavaByteMessage(msg);
        int type = jm.getCONTROL();
        switch (type) {
        case ControlMessage.RESULTS:
            long diff = System.currentTimeMillis() - microtime;
            byte[] res = jm.getOutput();
            // need to forward data to the reducer
            // add time to the end so it will be: mapID(int) usefulData(Strings)
            // timeFromMapper(long) timefromJavaMapper(long)

            try {
                byte[] intType = resultPayload.getBytes(ControlMessage.LONG);
                byte[] time = resultPayload.getBytes(diff);
                byte[] newRes = new byte[res.length + 4 + 8]; // original + four
                                                              // for type + 8
                                                              // for long
                System.arraycopy(res, 0, newRes, 0, res.length);
                System.arraycopy(intType, 0, newRes, res.length, intType.length);
                System.arraycopy(time, 0, newRes, res.length + intType.length,
                                 time.length);

                Results results = createResults(true, true);
                System.out.println("Got Results!!!");
                JavaByteMessage j2 =
                    new JavaByteMessage(ControlMessage.START_STOP);
                j2.setDescription("Continuing run...");
                j2.setStreamIdentifier(streamID);
                j2.setInput(newRes);

                results.setResultPayload(j2.toByteArray());

                System.out.println("Writing results...");
                writeResults(results);
                System.out.println("Results Written to reducer");
            } catch (MapReduceException e1) {
                System.out.println("MapReduceException: " + e1.getMessage());
                e1.printStackTrace();
            } catch (MarshallingException e2) {
                // TODO Auto-generated catch block
                e2.printStackTrace();
            }
            break;
        case ControlMessage.STATUS:
            System.out.print("Program Status:");
            byte[] r2 = jm.getOutput();
            // first 4 bytes are type - expecting STRING (-2), so ignoring
            byte[] lengthbyte = new byte[4];
            System.arraycopy(r2, 4, lengthbyte, 0, lengthbyte.length);
            int size;
            try {
                size = resultPayload.getInt(lengthbyte);
                byte[] stringbyte = new byte[size];
                System.arraycopy(r2, 8, stringbyte, 0, size);
                String stat = new String(stringbyte);
                System.out.println(stat);
                if (stat.equalsIgnoreCase("completed")) { // grab results..
                    JavaByteMessage j3 =
                        new JavaByteMessage(ControlMessage.RESULTS);
                    j3.setDescription("grabbing results");
                    j3.setStreamIdentifier(streamID);
                    jc.sendMessage(j3);
                }

            } catch (MarshallingException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            // sendSTART_STOP();
            break;
        default:
            // otherwise, not sure what's going on
            System.out.println("Unhandled message type: " + type);
            break;
        }
    }
}
