/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.pythonColl;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Properties;

import cgl.narada.event.NBEvent;
import ds.granules.bridge.ControlMessage;
import ds.granules.bridge.JavaByteMessage;
import ds.granules.bridge.JavaMessageHandler;
import ds.granules.dataset.Dataset;
import ds.granules.dataset.DatasetCollection;
import ds.granules.dataset.DatasetException;
import ds.granules.dataset.StreamingAccess;
import ds.granules.exception.MarshallingException;
import ds.granules.operation.MapReduceBase;
import ds.granules.operation.ProcessingException;
import ds.granules.results.ResultPayload;
import ds.granules.results.Results;
import ds.granules.results.ResultsFactory;

public class PyCollReduceRep extends MapReduceBase implements
        JavaMessageHandler {
    JavaPythonCommunication jc;
    private String streamID;
    private String output;
    long startTime; // for holding microseconds
    long javaTotal;
    long pythonTotal;
    int numMaps;


    public PyCollReduceRep() {
        setDomain("Generic");
        setVersionInformation(200);
        setOperationIdentifier("ZYXWVUTSRQPONML");
        javaTotal = 0;
        pythonTotal = 0;
    }


    public void pyCollReduceInit() {
        System.out.println("Initializing Reducer");
        Properties props = getProcessingDirectives();
        if (!props.containsKey("Program")) {
            System.out.println("Can't get program!");
            System.exit(-1);
        }
        String pyProg = props.getProperty("Program");
        numMaps = Integer.parseInt(props.getProperty("Number of Workers"));

        try {
            jc = new JavaPythonCommunication(pyProg, this);
        } catch (IOException e) {
            System.out.println("Couldn't get file");
            e.printStackTrace();
        }
    }


    private void sendInitialize(byte[] data) {
        System.out.println("Building streams...");

        ByteArrayInputStream is = new ByteArrayInputStream(data);
        DataInputStream dis = new DataInputStream(is);

        System.out.println("Done Building streams");
        try {

            int sSize = dis.readInt();
            System.out.println("String size: " + sSize);
            byte[] fname = new byte[sSize];
            dis.read(fname, 0, sSize);

            String filename = new String(fname);
            System.out.println("File name: " + fname);
            int oFileSize = dis.readInt();
            byte[] outFile = new byte[oFileSize];
            dis.read(outFile, 0, oFileSize);
            String outF = new String(outFile);
            output = outF;
            int width = dis.readInt();
            int height = dis.readInt();
            int zoomMult = dis.readInt();
            int dSize = dis.readInt();
            byte[] d = new byte[dSize];
            dis.read(d, 0, dSize);
            String description = new String(d);

            System.out.println("Got file, generating new javaMessage");

            // prepare and send initialization message
            JavaByteMessage jm = new JavaByteMessage(ControlMessage.INITIALIZE);

            // generate input byte[]
            ByteArrayOutputStream ba = new ByteArrayOutputStream();
            DataOutputStream ds = new DataOutputStream(ba);

            ds.writeInt(ControlMessage.STRING);
            byte[] s = filename.getBytes();
            ds.writeInt(s.length);
            ds.writeBytes(filename);
            ds.writeInt(ControlMessage.INT);
            ds.writeInt(numMaps);
            ds.writeInt(ControlMessage.INT);
            ds.writeInt(zoomMult);
            ds.writeInt(ControlMessage.INT);
            ds.writeInt(width);
            ds.writeInt(ControlMessage.INT);
            ds.writeInt(height);
            ds.writeInt(ControlMessage.STRING);
            s = description.getBytes();
            ds.writeInt(s.length);
            ds.writeBytes(description);
            ds.flush();
            byte[] i = ba.toByteArray();

            jm.setInput(i);
            jm.setDescription("Initialization");
            jm.setStreamIdentifier(getOperationIdentifier());
            // start timing...
            startTime = System.nanoTime() / 1000;
            jc.sendMessage(jm);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }


    @Override
    public void execute() throws ProcessingException {
        try {
            System.out.println("PyCollReduce is ...... ");
            if (jc == null)
                pyCollReduceInit();

            String datasetIdentifier = "PyCollStreamDatasetID";

            System.out.println("Executing...");
            DatasetCollection datasetCollection = getDatasetCollection();
            StreamingAccess Saccess = null;
            Dataset dataset = datasetCollection.getDataset(datasetIdentifier);

            if (dataset.getDatasetType() == Dataset.STREAMS) {
                Saccess = (StreamingAccess) dataset;
            } else {
                System.out.println("Incorrect datatset: "
                    + dataset.getDatasetType() + " initialized. Returning ...");
                return;
            }

            if (Saccess != null) {

                while (dataset.isDataAvailable()) {
                    NBEvent nbEvent = Saccess.getStreamingData();
                    processResultsFromMap(nbEvent);
                }

            }

            StreamingAccess mapReduceDataset = getMapReduceDataset();

            while (mapReduceDataset.isDataAvailable()) {
                // System.out.println("executing!");
                NBEvent nbEvent = mapReduceDataset.getStreamingData();
                processResultsFromMap(nbEvent);
            }

            if (this.terminationConditionReached()) {
                System.out.println("terminiation condition reached!");
                JavaByteMessage j = new JavaByteMessage(ControlMessage.RESULTS);
                j.setDescription("Grabbing final results");
                j.setStreamIdentifier(this.getOperationIdentifier()); // do i
                                                                      // want
                                                                      // this?

                ByteArrayOutputStream ba = new ByteArrayOutputStream();
                DataOutputStream ds = new DataOutputStream(ba);

                System.out.println("output: " + output);

                ds.writeInt(ControlMessage.STRING);
                byte[] s = output.getBytes();
                ds.writeInt(s.length);
                ds.writeBytes(output);
                ds.flush();

                j.setInput(ba.toByteArray());
                jc.sendMessage(j);
            }

        } catch (DatasetException e) {
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("Problems converting bytes to doubles");
            e.printStackTrace();
        } catch (MarshallingException e) {
            System.out.println("Problems Marshalling: " + e.getMessage());
            e.printStackTrace();
        }

    }


    private void processResultsFromMap(NBEvent nbEvent) throws IOException,
        MarshallingException {
        if (jc == null)
            pyCollReduceInit();

        String synopsis = (String) nbEvent.getContentSynopsis();
        System.out.println("Synopsis: " + synopsis);
        if (synopsis.startsWith("Reducer/Initialize")) {
            byte[] payload = nbEvent.getContentPayload();
            sendInitialize(payload);
        } else {
            ResultPayload resultPayload;
            resultPayload = ResultPayload.getInstance();

            System.out.println("Processing results from map in reducer...");

            byte[] resultBytes = nbEvent.getContentPayload();
            Results results =
                ResultsFactory.getInstance().getResults(resultBytes);
            byte[] reportBytes = results.getResultPayload();

            JavaByteMessage jm = new JavaByteMessage(reportBytes);
            // each message will be a start_stop message
            // strip off timing information from jm before sending it...
            byte[] allInp = jm.getInput();
            byte[] timeInp = new byte[24]; // two ints, two longs
            byte[] fixedInp = new byte[allInp.length - timeInp.length];

            System.arraycopy(allInp, 0, fixedInp, 0, fixedInp.length);
            System.arraycopy(allInp, fixedInp.length, timeInp, 0,
                             timeInp.length);

            byte[] pythonTime = new byte[8];
            byte[] javaTime = new byte[8];

            System.arraycopy(timeInp, 4, pythonTime, 0, pythonTime.length);
            System.arraycopy(timeInp, 16, javaTime, 0, javaTime.length);

            long pTime = resultPayload.getLong(pythonTime);
            long jTime = resultPayload.getLong(javaTime);

            javaTotal += jTime;
            pythonTotal += pTime;

            jm.setInput(fixedInp); // now i don't need to worry about extra
                                   // garbage here.

            startTime = System.currentTimeMillis();
            jc.sendMessage(jm);

            String report = new String(reportBytes);
            System.out.println(synopsis + ":-> " + report);

            String mapIdentifier = results.getInstanceIdentifier();
            setReceivedOutputFrom(mapIdentifier);

        }
        if (!isAwaitingOutputs()) {
            System.out.println("All results now available at the reducer");
            // getResults();
            // setTeminationConditionReached();
        }

    }


    @Override
    public void handleMessage(byte[] msg) {
        // fudging things to make asynchronous
        final byte[] par = msg;
        new Thread(new Runnable() {
            public void run() {
                messageSwitch(par);
            }
        }).start();

    }


    private void getResults() {
        System.out.println("Completed!!!! (getResults())");
        JavaByteMessage j = new JavaByteMessage(ControlMessage.RESULTS);
        j.setDescription("Grabbing final results");
        j.setStreamIdentifier(this.getOperationIdentifier()); // do i want this?

        ByteArrayOutputStream b = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(b);

        try {
            dos.writeInt(ControlMessage.STRING);
            byte[] s = output.getBytes();
            dos.writeInt(s.length);
            dos.writeBytes(output);
            dos.flush();
            b.flush();
            j.setInput(b.toByteArray());
            jc.sendMessage(j);
        } catch (IOException e) {
            System.out.println("Couldn't send message... " + e.getMessage());
            e.printStackTrace();
        }
        return;
    }


    protected void messageSwitch(byte[] par) {
        JavaByteMessage jm = new JavaByteMessage(par);
        {
            int mtype = jm.getCONTROL();
            DataInputStream ds;
            ByteArrayInputStream ba;
            byte[] o;
            javaTotal += System.currentTimeMillis() - startTime;
            switch (mtype) {
            case ControlMessage.STATUS:
                // should generally be this...
                o = jm.getOutput();
                ba = new ByteArrayInputStream(o);
                ds = new DataInputStream(ba);
                try {
                    if (ds.readInt() != ControlMessage.STRING) {
                        System.out.println("ERROR: result not a string");
                        break;
                    }
                    int sLength = ds.readInt();
                    byte[] str = new byte[sLength];
                    ds.read(str);
                    String status = new String(str);
                    System.out.println("Status: " + status);

                } catch (IOException e) {
                    System.out.println("IOException: " + e.getMessage());
                    e.printStackTrace();
                }
                break;
            case ControlMessage.RESULTS:
                // only when everything is done
                o = jm.getOutput();
                ba = new ByteArrayInputStream(o);
                ds = new DataInputStream(ba);
                try {
                    // first comes the time...
                    ds.readInt(); // long
                    pythonTotal += ds.readLong();
                    ds.readInt(); // string
                    int sLength = ds.readInt();
                    byte[] str = new byte[sLength];
                    ds.read(str);
                    String status = new String(str);
                    System.out.println("Status: " + status);
                    System.out.println("All done!");
                    // BufferedWriter out = new BufferedWriter(new
                    // FileWriter("/tmp/ericson/PYoutput.txt", true));
                    // out.write(javaTotal + "\t" + pythonTotal +"\n");
                    // out.close();
                } catch (IOException e) {
                    System.out.println("IOException: " + e.getMessage());
                    e.printStackTrace();
                }
                break;
            case ControlMessage.COMPLETE:
                // send request for results
                // forget about further parsing...
                System.out.println("Completed!!!! (ControlMessage.Complete)");
                JavaByteMessage j = new JavaByteMessage(ControlMessage.RESULTS);
                j.setDescription("Grabbing final results");
                j.setStreamIdentifier(this.getOperationIdentifier()); // do i
                                                                      // want
                                                                      // this?

                ByteArrayOutputStream b = new ByteArrayOutputStream();
                DataOutputStream dos = new DataOutputStream(b);

                try {
                    dos.writeInt(ControlMessage.STRING);
                    byte[] s = output.getBytes();
                    dos.writeInt(s.length);
                    dos.writeBytes(output);
                    dos.flush();
                    j.setInput(b.toByteArray());
                    startTime = System.currentTimeMillis();
                    jc.sendMessage(j);
                } catch (IOException e) {
                    System.out.println("Couldn't send message... "
                        + e.getMessage());
                    e.printStackTrace();
                }
                break;
            default:
                // don't know what message to expect...
                System.out.println("Unhandled Message of type: " + mtype);
                break;
            }
        }

    }
}
