/**
 * Software License, Version 1.0
 * 
 * Copyright 2010 Colorado State University.  All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 *  the list of authors in the original source code, this list of conditions and
 *  the disclaimer listed in this license;
 * 2) All redistributions in binary form must reproduce the above copyright 
 *  notice, this list of conditions and the disclaimer listed in this license in
 *  the documentation and/or other materials provided with the distribution;
 * 3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Department of Computer Science 
 * at Colorado State University. For further information contact the department at 
 * http://www.cs.colostate.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name Colorado State University or Granules shall not be used to endorse
 * or promote products derived from this software without prior written permission
 * from Colorado State University.  For written permission, please contact the 
 * Computer Science Department at 1100 Center Avenue, Fort Collins, CO 80523-1873.
 * 5) Products derived from this software may not be called Granules, 
 * nor may Colorado State University or Granules appear in their name, without prior
 * written permission of ARTI.
 * 
 *
 * Colorado State University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Colorado State University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. COLORADO STATE UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * COLORADO STATE UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.samples.pythonColl;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.util.Properties;
import java.util.Random;
import java.util.StringTokenizer;

import cgl.narada.event.NBEvent;
import cgl.narada.event.NBEventException;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import cgl.narada.matching.Profile;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.EventConsumer;
import cgl.narada.service.client.EventProducer;
import cgl.narada.service.client.NBEventListener;
import cgl.narada.service.client.SessionService;
import ds.granules.exception.MarshallingException;
import ds.granules.results.ResultPayload;

public class PyLoadGenRep implements NBEventListener {

    private int entityId;

    private ClientService clientService;

    private String moduleName = "SimpleClient: ";

    private EventConsumer consumer;

    private EventProducer producer;

    private Profile profile;

    private String topic = "WorkerStream/Result";

    public String rdcrTopic = "Reducer/Initialize";

    private int tracker = 0;

    private ResultPayload resultPayload;


    public PyLoadGenRep() throws ServiceException {
        entityId = new Random().nextInt();
        clientService = SessionService.getClientService(entityId);
        resultPayload = ResultPayload.getInstance();
    }


    public void initializeBrokerCommunications(Properties props, String commType)
        throws ServiceException {
        clientService.initializeBrokerCommunications(props, commType);
    }


    /**
     * Closes the connection to the broker.
     * 
     * @throws ServiceException
     */
    public void closeBrokerConnection() throws ServiceException {
        unsubscribe();
        clientService.closeBrokerConnection();
        clientService.terminateServices();
        try {
            /** Make sure that the transmission tracker is shutdown gracefully. */
            Thread.sleep(100);
        } catch (InterruptedException e) {
            System.out.println(moduleName
                + "Problems sleeping while waiting for transfer completion");
        }
    }


    public void initializeProducer() throws ServiceException {
        producer = clientService.createEventProducer();

        producer.generateEventIdentifier(true);
        producer.setTemplateId(12345);
        producer.setDisableTimestamp(false);

        producer.setSuppressRedistributionToSource(true);
    }


    public void initializeConsumer() throws ServiceException {
        profile =
            clientService.createProfile(TemplateProfileAndSynopsisTypes.STRING,
                                        topic);
        consumer = clientService.createEventConsumer(this);
        consumer.subscribeTo(profile);
    }


    public void unsubscribe() throws ServiceException {
        consumer.unSubscribe(profile);
    }


    public void publishEvent(byte[] msg) throws ServiceException,
        NBEventException, MarshallingException {

        String publishTopic = "WorkerStreams/PyColl";
        System.out.println("generating event");
        NBEvent nbEvent =
            producer.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                   publishTopic, msg);

        System.out.println("Publishing event");
        producer.publishEvent(nbEvent);
    }


    public void publishEvent(byte[] msg, String publishTopic)
        throws ServiceException {
        System.out.println("generating event");
        NBEvent nbEvent =
            producer.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                   publishTopic, msg);

        System.out.println("Publishing event");
        producer.publishEvent(nbEvent);
    }


    @Override
    public void onEvent(NBEvent arg0) {
        System.out.println("Shouldn't be here...");

    }


    public static void main(String[] args) {
        String module = "PyCollDeployer.main()->";
        if (args.length < 2) {
            System.out.println("Usage: java ds.granules.samples.pythonColl."
                + "PyCollDeployer <hostname> <port>");
            System.exit(0);
        }
        String hostname = args[0];
        String portnum = args[1];
        String pyProg1 = "/tmp/ericson/python/mainMap.py";
        String pyProg2 = "/tmp/ericson/python/mainMap.py";
        String fName = "/tmp/ericson/python/img.jpg";
        int width = 10;
        int height = 10;
        int zoom = 1;
        String fout = "/tmp/ericson/out.jpg";
        String description = "car";

        Properties streamingProperties = new Properties();
        streamingProperties.put("hostname", hostname);
        streamingProperties.put("portnum", portnum);

        String transport = "niotcp";

        try {
            PyLoadGenRep plg = new PyLoadGenRep();
            plg.initializeBrokerCommunications(streamingProperties, transport);
            plg.initializeProducer();
            plg.initializeConsumer();

            String helpInfo =
                "\n\nThis is a simple example that demonstrates bridging behavior "
                    + "via unnamed pipes using Granules. The current list of commands include: \n"
                    + "stream <python_mapper_main> <python_reducer_main> <input_image_file> <pixel_box_width> <pixel_box_height> <zoom> <output_file> <image_description>\n"
                    + "status        : Lists the status of the all Jobs\n"
                    + "h             : List the set of commands \n\n";

            StringTokenizer tok;

            InputStreamReader isr = new InputStreamReader(System.in);
            BufferedReader commandLine = new BufferedReader(isr);

            System.out.println(helpInfo);

            while (true) {
                String s = commandLine.readLine();
                if (s == null) {
                    System.out.println(module + "String is null!!!");
                    break;
                }

                if (s.startsWith("stream")) {
                    System.out.println(module
                        + "Processing the [stream] command");

                    tok = new StringTokenizer(s, " ");
                    int numOfTokens = tok.countTokens();
                    tok.nextToken();/* skip the command */

                    if (numOfTokens > 1) {
                        pyProg1 = tok.nextToken();
                    }
                    if (numOfTokens > 2) {
                        pyProg2 = tok.nextToken();
                    }
                    if (numOfTokens > 3) {
                        fName = tok.nextToken();
                    }
                    if (numOfTokens > 4) {
                        width = Integer.parseInt(tok.nextToken());
                    }
                    if (numOfTokens > 5) {
                        height = Integer.parseInt(tok.nextToken());
                    }
                    if (numOfTokens > 6) {
                        zoom = Integer.parseInt(tok.nextToken());
                    }
                    if (numOfTokens > 7) {
                        fout = tok.nextToken();
                    }
                    if (numOfTokens > 8)
                        description = tok.nextToken();

                    // generate nbevent to start mappers
                    ByteArrayOutputStream ba = new ByteArrayOutputStream();
                    DataOutputStream dos = new DataOutputStream(ba);

                    byte[] file = fName.getBytes();
                    System.out.println("file.length: " + file.length);
                    dos.writeInt(file.length);
                    dos.writeBytes(fName);
                    dos.writeInt(width);
                    dos.writeInt(height);
                    dos.writeInt(zoom);
                    dos.flush();

                    byte[] toSend = ba.toByteArray();

                    ba.reset();
                    byte[] fileIn = fName.getBytes();
                    dos.writeInt(fileIn.length);
                    dos.writeBytes(fName);
                    byte[] fileOut = fout.getBytes();
                    dos.writeInt(fileOut.length);
                    dos.writeBytes(fout);
                    dos.writeInt(width);
                    dos.writeInt(height);
                    dos.writeInt(zoom);
                    byte[] d = description.getBytes();
                    dos.writeInt(d.length);
                    dos.writeBytes(description);
                    dos.flush();

                    byte[] toRdcr = ba.toByteArray();
                    plg.publishEvent(toRdcr, plg.rdcrTopic);

                    plg.publishEvent(toSend);

                    continue;
                }

                if (s.equals("h")) {
                    System.out.println(helpInfo);
                    continue;
                }

                if (s.equals("q")) {
                    System.out.println(module
                        + "Exiting the sample instance deployer");
                    break;
                }

            }

        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
        System.exit(0);

    }

}
