#Wrapper to handle communication with the collage program, keeps status information as well

import sys
from javaMessage import *
from colMap import *
from colRdcr import *
import time # for timing stuff - should get microseconds ok

class colMapWrap:
	_jm_ = javaMessage()
	"""Initialize from a javamessage"""
	def __init__(self, jm):
		_jm_ = javaMessage()
		self.stream = jm.StreamID
		self.status = "INITIALIZED"
		self.input=jm.IOArray[0].data	#should have the map index, and numMaps - the total number of maps (both INTS)
		tmp = _jm_.dataBlock.parse(self.input)
		sys.stderr.write(str(tmp))
		sys.stderr.flush()
		self.mapID = tmp.readOne[0].data
		self.numMaps = tmp.readOne[1].data
		self.results=jm.IOArray[1].data

	def run(self, jm):
		startTime = time.time() * 1000
		_jm_ = javaMessage()
		#c = colMap()
		io = jm.IOArray
		self.input = io[0].data
		sys.stderr.write("running!!!\n")
		sys.stderr.write("input:\n" + str(self.input) + "\n")
		sys.stderr.flush()
		allInp = _jm_.dataBlock.parse(self.input)
		sys.stderr.write(str(allInp) + "\n")
		sys.stderr.flush()
		#input should contain in this order:
		#STRING (filename) INT (width) INT (height) INT (zoomMult)
		self.status = "RUNNING"
		fname = allInp.readOne[0].data
		sys.stderr.write("file: " + str(fname))
		sys.stderr.flush()
		size = (allInp.readOne[1].data,allInp.readOne[2].data)
		zoom = allInp.readOne[3].data
		list = makeCollage(fname,size,zoom,self.mapID,self.numMaps)
		#done with actual running now, so now get time difference..
		timeDiff = (time.time() * 1000) - startTime
		list.insert(0,Container(DataType="INT",data=self.mapID))
		list.append(Container(DataType="LONG",data=timeDiff))
		self.results = _jm_.dataBlock.build(Container(readOne=list))
		#sys.stderr.write("checking back output: " + str(_jm_.dataBlock.parse(self.results)) + "\n")
		#sys.stderr.flush()
		self.status = "COMPLETED"

	def getResults(self):
		_jm_ = javaMessage()
		x = _jm_.JMConvert.build(Container(MessageType="RESULTS",IOArray=[Container(length=len(self.input),data = self.input),Container(length=len(self.results),data=self.results)],StreamID = self.stream,Description=self.status,fromJava=False))
		sys.stderr.write("return msg size: " + str(len(x)) + "\n")
		sys.stderr.flush()
		return x

class colRdcrWrap:
	_jm_=javaMessage()
	def __init__(self, jm):
		sys.stderr.write("in rdcrWrap, jm: " + str(jm) + "\n")
		_jm_ = javaMessage()
		self.stream = jm.StreamID
		self.status = "INITIALIZED"
		self.input=jm.IOArray[0].data
		#should have STRING (filename), INT (#maps), INT (zoom), INT (box width), INT (box height), STRING (description of image)
		allInp = _jm_.dataBlock.parse(self.input)
		sys.stderr.write("in rdcrwrap, allInp: " + str(allInp) + "\n")
		sys.stderr.flush()
		img = allInp.readOne[0].data
		self.numMaps = allInp.readOne[1].data
		zoom = allInp.readOne[2].data
		box = (allInp.readOne[3].data,allInp.readOne[4].data)
		type = allInp.readOne[5].data
		#not sure how else to keep track of time spent here...
		self.timeSpent = 0
		timeStart = time.time() * 1000
		self.rdcr = imgCompiler(img,self.numMaps,zoom,box,type)
		self.timeSpent += time.time()*1000 - timeStart
		self.completed = 0

	def run(self, jm):
		_jm_ = javaMessage()
		#sent by granules when the reducer gets some results back
		#timing starts here...
		timeStart = time.time()*1000
		self.input = jm.IOArray[0].data
		#should have INT(mapid), STRING* (color array)
		allInp = _jm_.dataBlock.parse(self.input)
		id = allInp.readOne[0].data
		colList = []
		for i in allInp.readOne[1:]:
			colList.append(i.data)
		#sys.stderr.write("id: " + str(id) + "\nlist: " + str(colList) + "\n") # for debugging...too long otherwise
		#sys.stderr.flush()
		self.rdcr.addImages(id,colList)
		#stop timing
		self.timeSpent += time.time()*1000 - timeStart
		self.completed+=1
		self.status = "COMPLETED " + str(self.completed) + "/" + str(self.numMaps)
		sys.stderr.write("completed: " + str(self.completed) + " of: " + str(self.numMaps))
		sys.stderr.flush()
		if(self.completed == self.numMaps):
			retData = _jm_.dataBlock.build(Container(readOne=[Container(DataType="STRING",data=self.status)]))
			x=_jm_.JMConvert.build(Container(MessageType="COMPLETE",IOArray=[Container(length=len(self.input),data=self.input),Container(length=len(retData),data=retData)],StreamID=self.stream,Description=self.status,fromJava=False))
			return x
		else:
			retData = _jm_.dataBlock.build(Container(readOne=[Container(DataType="STRING",data=self.status)]))
			retMsg=_jm_.JMConvert.build(Container(MessageType="STATUS",IOArray=[Container(length=len(jm.IOArray[0].data),data=jm.IOArray[0].data),Container(length=len(retData),data=retData)],StreamID=self.stream,Description=self.status,fromJava=False))
			return retMsg

	def getResults(self, jm):
		_jm_ = javaMessage()
		#simply returns the location of the saved file
		self.input = jm.IOArray[0].data
		allInp = _jm_.dataBlock.parse(self.input) #should just be STRING filename
		#timer again...just because
		timeStart = time.time()*1000
		self.rdcr.saveResult(allInp.readOne[0].data)
		self.timeSpent += time.time()*1000 - timeStart
		retData = _jm_.dataBlock.build(Container(readOne=[Container(DataType="LONG",data=self.timeSpent),Container(DataType="STRING",data=allInp.readOne[0].data)]))
		x = _jm_.JMConvert.build(Container(MessageType="RESULTS",IOArray=[Container(length=len(self.input),data=self.input),Container(length=len(retData),data=retData)],StreamID=self.stream,Description=self.status,fromJava=False))
		self.completed = 0;
		return x
