/**
 * Software License, Version 1.0
 * 
 * Copyright 2008 The Trustees of Indiana University.  All rights reserved.
 * 
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the 
 * following acknowledgment:
 *
 *"This product includes software developed by the Community Grids Lab. For 
 * further information contact the Community Grids Lab at 
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgment may appear in the software itself, and 
 * wherever such third-party acknowledgments normally appear.
 * 
 *4) The name Indiana University or Community Grids Lab or Granules, 
 * shall not be used to endorse or promote products derived from this software 
 * without prior written permission from Indiana University.  For written 
 * permission, please contact the Advanced Research and Technology Institute 
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called Granules, 
 * nor may Indiana University or Community Grids Lab or Granules appear
 * in their name, without prior written permission of ARTI.
 * 
 *
 * Indiana University provides no reassurances that the source code provided 
 * does not infringe the patent or any other intellectual property rights of 
 * any other entity.  Indiana University disclaims any liability to any 
 * recipient for claims brought by any other entity based on infringement of 
 * intellectual property rights or otherwise.  
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. 
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", 
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION 
 * GENERATED USING SOFTWARE.
 */
package ds.granules.scheduler;

/**
 * This is an interface that snapshots the profile of a given resource.
 * 
 * @author Shrideep Pallickara
 * 
 */
public interface ResourceProfile {

    /**
     * Retrieve the processor information
     * 
     * @return Information about the processor
     */
    public String getProcessorInfo();


    /**
     * Retrieves information about the available storage.
     * 
     * @return Storage information.
     */
    public String getStorageInfo();


    /**
     * Retrieve the queue size at the resource/
     * 
     * @return The size of the processing queue at the resource
     */
    public int getQueueSize();


    /**
     * Retrieves the average execution time for a given task.
     * 
     * @return execution time in milliseconds.
     */
    public long averageTaskExecutionTime();


    /**
     * Retrieves the expect starting time for a given task's execution.
     * 
     * @return beginTime in milliseconds
     */
    public long expectedBeginTime();


    /**
     * Retrieves the time for which this resource has been up-and-running
     * continuously without failures.
     * 
     * @return The up time in milliseconds.
     */
    public long getUptime();


    /**
     * Retrieves the restart history of this resource.
     * 
     * @return the restart history for this node
     */
    public long getRestartHistory();


    /**
     * The memory utilization at the resource
     * 
     * @return The memory utilization in MB.
     */
    public long getMemoryUtilization();

}
